Package: smfa
Title: Stochastic Metafrontier Analysis
Version: 1.0.0
Authors@R: c(
    person("Sulman Olieko", "Owili", email = "oliekosulman@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7401-5326"))
  )
Description: Implements stochastic metafrontier analysis for productivity and
    performance benchmarking across firms operating under different
    technologies. Contains routines for the deterministic metafrontier envelope
    of O'Donnell et al. (2008) <doi:10.1007/s00181-007-0119-4> via linear and
    quadratic programming, and the stochastic metafrontier of Huang et al.
    (2014) <doi:10.1007/s11123-014-0402-2>. Also supports latent class
    stochastic metafrontier analysis and sample selection correction stochastic
    metafrontier models. Depends on the 'sfaR' package by Dakpo et al. (2023)
    <https://CRAN.R-project.org/package=sfaR>.
License: GPL (>= 3)
URL: https://github.com/SulmanOlieko/smfa,
        https://SulmanOlieko.github.io/smfa/
BugReports: https://github.com/SulmanOlieko/smfa/issues
Depends: R (>= 3.5.0), sfaR
Imports: stats
Suggests: knitr, rmarkdown, pkgdown, lmtest
VignetteBuilder: knitr
Config/Needs/website: pkgdown
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-24 14:26:20 UTC; macbookpro
Author: Sulman Olieko Owili [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7401-5326>)
Maintainer: Sulman Olieko Owili <oliekosulman@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 20:32:21 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,smfa)
S3method(coef,summary.smfa)
S3method(efficiencies,smfa)
S3method(fitted,smfa)
S3method(ic,smfa)
S3method(logLik,smfa)
S3method(nobs,smfa)
S3method(print,smfa)
S3method(print,summary.smfa)
S3method(residuals,smfa)
S3method(summary,smfa)
S3method(vcov,smfa)
export(smfa)
importFrom(sfaR,efficiencies)
importFrom(sfaR,ic)
importFrom(sfaR,sfacross)
importFrom(sfaR,sfalcmcross)
importFrom(sfaR,sfaselectioncross)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.pass)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,qchisq)
importFrom(stats,residuals)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(utils,capture.output)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

## ----install, eval = FALSE----------------------------------------------------
# # Install smfa from CRAN
# install.packages("smfa")
# 
# # Or install the development version from GitHub
# # if (!require("devtools")) install.packages("devtools")
# # devtools::install_github("SulmanOlieko/smfa")

## ----quickstart-data----------------------------------------------------------
library(smfa)
data("ricephil", package = "sfaR")

# Create technology groups based on farm area terciles
ricephil$group <- cut(
  ricephil$AREA,
  breaks = quantile(ricephil$AREA, probs = c(0, 1 / 3, 2 / 3, 1), na.rm = TRUE),
  labels = c("small", "medium", "large"),
  include.lowest = TRUE
)
table(ricephil$group)

## ----quickstart-fit-----------------------------------------------------------
meta_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1, # production frontier (S=1) or cost frontier (S=-1)
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)

## ----quickstart-summary-------------------------------------------------------
summary(meta_lp)

## ----quickstart-eff-----------------------------------------------------------
eff <- efficiencies(meta_lp)
head(eff)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

## ----data---------------------------------------------------------------------
library(smfa)
data("ricephil", package = "sfaR")

ricephil$group <- cut(
  ricephil$AREA,
  breaks        = quantile(ricephil$AREA, probs = c(0, 1/3, 2/3, 1), na.rm = TRUE),
  labels        = c("small", "medium", "large"),
  include.lowest = TRUE
)

table(ricephil$group)
#>  small medium  large
#>    125    104    115

## ----lp-----------------------------------------------------------------------
meta_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)
summary(meta_lp)

## ----qp-----------------------------------------------------------------------
meta_qp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "qp"
)
summary(meta_qp)

## ----huang--------------------------------------------------------------------
meta_huang <- smfa(
  formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data        = ricephil,
  group       = "group",
  S           = 1,
  udist       = "hnormal",
  groupType   = "sfacross",
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_huang)

## ----odonnell-----------------------------------------------------------------
meta_ordonnell <- smfa(
  formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data        = ricephil,
  group       = "group",
  S           = 1,
  udist       = "hnormal",
  groupType   = "sfacross",
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_ordonnell)

## ----eff----------------------------------------------------------------------
eff <- efficiencies(meta_lp)
head(eff)

# Subset for a specific group
eff_small <- eff[eff$group == "small", ]
summary(eff_small[, c("TE_group_BC", "TE_meta_BC", "MTR_BC")])

## ----other--------------------------------------------------------------------
coef(meta_qp)          # metafrontier coefficients
vcov(meta_qp)          # variance-covariance matrix
logLik(meta_lp)        # log-likelihood
ic(meta_lp)            # AIC, BIC, HQIC
nobs(meta_lp)          # number of observations
fitted(meta_lp)        # fitted values
residuals(meta_lp)     # residuals

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

## ----data---------------------------------------------------------------------
library(smfa)
data("utility", package = "sfaR")

## ----lp-----------------------------------------------------------------------
meta_lcm_lp <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility,
  S          = -1,          # cost frontier (S = -1)
  groupType  = "sfalcmcross",
  lcmClasses = 2,           # number of latent classes
  metaMethod = "lp"
)
summary(meta_lcm_lp)

## ----qp-----------------------------------------------------------------------
meta_lcm_qp <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility,
  S          = -1,
  groupType  = "sfalcmcross",
  lcmClasses = 2,
  metaMethod = "qp"
)
summary(meta_lcm_qp)

## ----huang--------------------------------------------------------------------
meta_lcm_huang <- smfa(
  formula     = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data        = utility,
  S           = -1,
  groupType   = "sfalcmcross",
  lcmClasses  = 2,
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_lcm_huang)

## ----odonnell-----------------------------------------------------------------
meta_lcm_odonnell <- smfa(
  formula     = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data        = utility,
  S           = -1,
  groupType   = "sfalcmcross",
  lcmClasses  = 2,
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_lcm_odonnell)

## ----compare-classes----------------------------------------------------------
meta_lcm_2 <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility, S = -1,
  groupType  = "sfalcmcross", lcmClasses = 2, metaMethod = "lp"
)
meta_lcm_3 <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility, S = -1,
  groupType  = "sfalcmcross", lcmClasses = 3, metaMethod = "lp"
)
# Compare information criteria
ic(meta_lcm_2)
ic(meta_lcm_3)

## ----eff----------------------------------------------------------------------
eff_lcm <- efficiencies(meta_lcm_lp)
head(eff_lcm)

# Key LCM-specific columns:
# Group_c          — most likely class assignment
# PosteriorProb_c  — posterior probability of assigned class
# PosteriorProb_c1 — posterior probability of Class 1
# PosteriorProb_c2 — posterior probability of Class 2

## ----class-summary------------------------------------------------------------
# Proportion assigned to each class and mean posterior probability
with(eff_lcm, table(Group_c)) / nrow(eff_lcm) * 100   # % in each class

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

## ----data---------------------------------------------------------------------
library(smfa)

N <- 500; set.seed(12345)
z1 <- rnorm(N); z2 <- rnorm(N)
v1 <- rnorm(N); v2 <- rnorm(N)
g  <- rnorm(N)
e1 <- v1
e2 <- 0.7071 * (v1 + v2)
ds <- z1 + z2 + e1
d  <- ifelse(ds > 0, 1, 0)        # 1 = selected into the sample
group <- ifelse(g > 0, 1, 0)      # two technology groups
u  <- abs(rnorm(N))
x1 <- abs(rnorm(N)); x2 <- abs(rnorm(N))
y  <- abs(x1 + x2 + e2 - u)
dat <- as.data.frame(cbind(y = y, x1 = x1, x2 = x2,
                            z1 = z1, z2 = z2, d = d, group = group))

# About 50% of observations are selected
table(dat$d)
#>    0    1
#> 1013  987

## ----lp-----------------------------------------------------------------------
meta_sel_lp <- smfa(
  formula    = log(y) ~ log(x1) + log(x2),
  selectionF = d ~ z1 + z2,      # selection equation: d is the binary indicator
  data       = dat,
  group      = "group",
  S          = 1L,
  udist      = "hnormal",
  groupType  = "sfaselectioncross",
  modelType  = "greene10",        # Greene (2010) two-step ML correction
  lType      = "kronrod",         # integration method for the selection likelihood
  Nsub       = 20,               # number of sub-intervals for numerical integration
  uBound     = Inf,
  method     = "bfgs",
  itermax    = 2000,
  metaMethod = "lp"
)
summary(meta_sel_lp)

## ----qp-----------------------------------------------------------------------
meta_sel_qp <- smfa(
  formula    = log(y) ~ log(x1) + log(x2),
  selectionF = d ~ z1 + z2,
  data       = dat,
  group      = "group",
  S          = 1L,
  udist      = "hnormal",
  groupType  = "sfaselectioncross",
  modelType  = "greene10",
  lType      = "kronrod",
  Nsub       = 20,
  uBound     = Inf,
  method     = "bfgs",
  itermax    = 2000,
  metaMethod = "qp"
)
summary(meta_sel_qp)

## ----huang--------------------------------------------------------------------
meta_sel_huang <- smfa(
  formula     = log(y) ~ log(x1) + log(x2),
  selectionF  = d ~ z1 + z2,
  data        = dat,
  group       = "group",
  S           = 1L,
  udist       = "hnormal",
  groupType   = "sfaselectioncross",
  modelType   = "greene10",
  lType       = "kronrod",
  Nsub        = 100,
  uBound      = Inf,
  method      = "bfgs",
  itermax     = 2000,
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_sel_huang)

## ----odonnell-----------------------------------------------------------------
meta_sel_odonnell <- smfa(
  formula     = log(y) ~ log(x1) + log(x2),
  selectionF  = d ~ z1 + z2,
  data        = dat,
  group       = "group",
  S           = 1L,
  udist       = "hnormal",
  groupType   = "sfaselectioncross",
  modelType   = "greene10",
  lType       = "kronrod",
  Nsub        = 100,
  uBound      = Inf,
  method      = "bfgs",
  itermax     = 2000,
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_sel_odonnell)

## ----rho----------------------------------------------------------------------
# The rho parameter appears in the summary output:
# ----------------------------------------------------------------
#              Selection bias parameter
# ----------------------------------------------------------------
#           Coefficient Std. Error z value  Pr(>|z|)
# rho          0.89550    0.28696  3.1207  0.001804 **

# A significant rho indicates selection bias IS present and the
# correction is important.

## ----eff----------------------------------------------------------------------
eff_sel <- efficiencies(meta_sel_lp)

# Non-selected observations have NA efficiencies
sum(is.na(eff_sel$TE_group_BC))   # count of non-selected obs

# Subset for selected observations in group 1
sel_grp1 <- eff_sel[eff_sel$group == 1 & !is.na(eff_sel$TE_group_BC), ]
summary(sel_grp1[, c("TE_group_BC", "TE_meta_BC", "MTR_BC")])

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)

## ----fit----------------------------------------------------------------------
library(smfa)
data("ricephil", package = "sfaR")

ricephil$group <- cut(
  ricephil$AREA,
  breaks        = quantile(ricephil$AREA, probs = c(0, 1/3, 2/3, 1), na.rm = TRUE),
  labels        = c("small", "medium", "large"),
  include.lowest = TRUE
)

meta_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)

## ----summary------------------------------------------------------------------
summary(meta_lp)

## ----efficiencies-------------------------------------------------------------
eff <- efficiencies(meta_lp)
head(eff)

## ----subset-------------------------------------------------------------------
# All small farms
eff_small <- eff[eff$group == "small", ]

# Descriptive statistics
summary(eff_small[, c("TE_group_BC", "TE_meta_BC", "MTR_BC")])

# Group-level means
aggregate(cbind(TE_group_BC, TE_meta_BC, MTR_BC) ~ group, data = eff, FUN = mean)

## ----plot---------------------------------------------------------------------
# MTR distribution by group (base R)
boxplot(MTR_BC ~ group, data = eff,
        main = "Metatechnology Ratio by Farm Size",
        xlab = "Group", ylab = "MTR (BC)",
        col  = c("#e8f5ef", "#b2dfdb", "#80cbc4"))

# Histogram of TE_meta_BC
hist(eff$TE_meta_BC, breaks = 30,
     main = "Distribution of Metafrontier TE",
     xlab = "TE_meta_BC", col = "#2d8f65", border = "white")

## ----coef---------------------------------------------------------------------
# First fit a QP model
meta_qp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil, group = "group", S = 1, udist = "hnormal",
  groupType  = "sfacross", metaMethod = "qp"
)
coef(meta_qp)

## ----vcov---------------------------------------------------------------------
vcov(meta_qp)

## ----loglik-------------------------------------------------------------------
logLik(meta_lp)

## ----ic-----------------------------------------------------------------------
ic(meta_lp)
#>        AIC       BIC      HQIC
#> 1  184.579  253.710   212.113

## ----nobs---------------------------------------------------------------------
nobs(meta_lp)  # Total observations across all groups

## ----fitted-------------------------------------------------------------------
fv <- fitted(meta_lp)
head(fv)

## ----residuals----------------------------------------------------------------
res <- residuals(meta_lp)
head(res)

## ----compare------------------------------------------------------------------
meta_lp    <- smfa(log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
                               data = ricephil, group = "group", S = 1,
                               udist = "hnormal", groupType = "sfacross",
                               metaMethod = "lp")
meta_qp    <- smfa(log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
                               data = ricephil, group = "group", S = 1,
                               udist = "hnormal", groupType = "sfacross",
                               metaMethod = "qp")
meta_huang <- smfa(log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
                               data = ricephil, group = "group", S = 1,
                               udist = "hnormal", groupType = "sfacross",
                               metaMethod = "sfa", sfaApproach = "huang")

# Combined information criteria table
models <- list(LP = meta_lp, QP = meta_qp, Huang = meta_huang)
do.call(rbind, lapply(names(models), function(nm) {
  ic_vals <- ic(models[[nm]])
  data.frame(Model = nm, AIC = ic_vals[["AIC"]],
             BIC = ic_vals[["BIC"]], HQIC = ic_vals[["HQIC"]])
}))

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Coefficients extraction                                                      #
# Models:                                                                      #
#           -Stochastic metafrontier analysis                                  #
#           -Latent class stochastic metafrontier analysis                     #
#           -Sample selection correction for Stochastic Frontier Model         #
# Data: Cross sectional data & Pooled data                                     #
#------------------------------------------------------------------------------#

#' Extract coefficients of stochastic metafrontier models
#'
#' @description
#' From an object of class \code{'summary.smfa'},
#' \code{\link{coef}} extracts the coefficients,
#' their standard errors, z-values, and (asymptotic) P-values.
#'
#' From on object of class \code{'smfa'}, it extracts
#' only the estimated coefficients.
#'
#' @name coef
#'
#' @param object A stochastic metafrontier model returned by \code{\link{smfa}},
#' or an object of class \code{'summary.smfa'}.
#' @param ... Currently ignored.
#'
#' @return For objects of class \code{'summary.smfa'},
#' \code{\link{coef}} returns a matrix with four columns. Namely, the
#' estimated coefficients, their standard errors, z-values,
#' and (asymptotic) P-values.
#'
#' For objects of class \code{'smfa'}, \code{\link{coef}}
#' returns a numeric vector of the estimated coefficients.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data.
#'
#' @keywords methods coefficients
#'

# coefficients from smfa ----------
#' @rdname coef
#' @aliases coef.smfa
#' @importFrom stats coef
#' @export
coef.smfa <- function(object, ...) {
  if (!is.null(object$metaFrontierParam)) {
    return(object$metaFrontierParam)
  }
  # LP method: no metafrontier params; return a list of group params
  lapply(object$groupModels, function(m) m$mlParam[seq_len(m$nXvar)])
}

# coefficients from summary.smfa ----------
#' @rdname coef
#' @aliases coef.summary.smfa
#' @export
coef.summary.smfa <- function(object, ...) {
  object$metaRes
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Information Criteria extraction                                              #
# Models:                                                                      #
#           -Stochastic metafrontier analysis                                  #
#           -Latent class stochastic metafrontier analysis                     #
#           -Sample selection correction for stochastic metafrontier model     #
# Data: Cross sectional data & Pooled data                                     #
#------------------------------------------------------------------------------#

#' Extract information criteria of stochastic metafrontier models
#'
#' \code{\link{ic}} returns information criterion from stochastic
#' metafrontier models estimated with \code{\link{smfa}}.
#'
#' The different information criteria are computed as follows: \itemize{ \item
#' AIC: \eqn{-2 \log{LL} + 2 * K} \item BIC: \eqn{-2 \log{LL} + \log{N} * K}
#' \item HQIC: \eqn{-2 \log{LL} + 2 \log{\left[\log{N}\right]} * K} } where
#' \eqn{LL} is the maximum likelihood value, \eqn{K} the number of parameters
#' estimated and \eqn{N} the number of observations.
#'
#' @name ic
#'
#' @param object A stochastic metafrontier model returned
#' by \code{\link{smfa}}.
#' @param IC Character string. Information criterion measure. Three criteria
#' are available: \itemize{ \item \code{'AIC'} for Akaike information criterion
#' (default) \item \code{'BIC'} for Bayesian information criterion \item
#' \code{'HQIC'} for Hannan-Quinn information criterion }.
#' @param ... Currently ignored.
#'
#' @return \code{\link{ic}} returns a data frame with the values of the 
#' information criteria (AIC, BIC and HQIC) of the maximum likelihood coefficients. 
#' If the \code{IC} argument is provided, it returns only the requested criterion 
#' as a numeric value.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data.
#'
#' @keywords methods AIC BIC HQIC
#'
#' @rdname ic
#' @aliases ic.smfa
#' @importFrom sfaR ic
#' @export
ic.smfa <- function(object, IC = NULL, ...) {
  aic <- -2 * object$mlLoglik + 2 * object$nParm
  bic <- -2 * object$mlLoglik + log(object$Nobs) * object$nParm
  hqic <- -2 * object$mlLoglik + 2 * log(log(object$Nobs)) * object$nParm
  
  if (is.null(IC)) {
    res <- data.frame(AIC = aic, BIC = bic, HQIC = hqic)
    return(res)
  } else {
    if (!(IC %in% c("AIC", "BIC", "HQIC"))) {
      stop("Unknown information criteria: ", paste(IC), call. = FALSE)
    }
    obj <- switch(IC,
      AIC = aic,
      BIC = bic,
      HQIC = hqic
    )
    return(obj)
  }
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Summary of optimization objects                                              #
# Models:                                                                      #
#        -Stochastic metafrontier analysis and variants                        #
# Data:  Cross sectional & Pooled data                                         #
#------------------------------------------------------------------------------#

#' Summary of results for stochastic metafrontier models
#'
#' Create and print summary results for stochastic metafrontier models returned by
#' \code{\link{smfa}}.
#'
#' @param object An object of class \code{'smfa'} returned by the
#' function \code{\link{smfa}}.
#' @param ... Currently ignored.
#' @param x An object of class \code{'summary.smfa'}.
#' @param digits Numeric. Number of digits displayed in values.
#'
#' @name summary
#'
#' @return The \code{\link{summary}} method returns a list of class
#' \code{'summary.smfa'}
#' that contains the same elements as an object returned by \code{\link{smfa}}
#' with the following additional elements:
#'
#' \item{AIC}{Akaike information criterion.}
#'
#' \item{BIC}{Bayesian information criterion.}
#'
#' \item{HQIC}{Hannan-Quinn information criterion.}
#'
#' \item{metaRes}{Matrix of metafrontier estimates, their standard errors, z-values,
#' and asymptotic P-values.}
#'
#' \item{effStats}{A list of efficiency statistics including group means and
#' class membership probabilities.}
#'
#' \item{grpSummaries}{A list of summary objects for each group model.}
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function for cross-sectional or pooled data.
#'
#' @keywords methods summary
#'
#' @aliases summary.smfa
#' @export

summary.smfa <- function(object, ...) {
  # Information criteria
  object$AIC <- -2 * object$mlLoglik + 2 * object$nParm
  object$BIC <- -2 * object$mlLoglik + log(object$Nobs) * object$nParm
  object$HQIC <- -2 * object$mlLoglik + 2 * log(log(object$Nobs)) * object$nParm

  # Per-group summary objects (typed: summary.sfacross / sfalcmcross / sfaselectioncross)
  # For lcmNoGroup: summarise lcmObj once (all entries are the same model)
  if (isTRUE(object$lcmNoGroup)) {
    lcm_sm <- summary(object$lcmObj)
    object$grpSummaries <- setNames(
      rep(list(lcm_sm), object$nGroups),
      object$groups
    )
  } else {
    object$grpSummaries <- lapply(object$groupModels, summary)
  }

  # Metafrontier coefficient table (QP and SFA methods)
  if (!is.null(object$metaFrontierParam)) {
    if (!is.null(object$metaFrontierVcov)) {
      se <- sqrt(diag(object$metaFrontierVcov))
      zval <- object$metaFrontierParam / se
      pval <- 2 * pnorm(-abs(zval))
      metaRes <- cbind(
        Estimate = object$metaFrontierParam,
        `Std. Error` = se,
        `z value` = zval,
        `Pr(>|z|)` = pval
      )
    } else {
      metaRes <- matrix(
        object$metaFrontierParam,
        ncol = 1,
        dimnames = list(names(object$metaFrontierParam), "Estimate")
      )
    }
    object$metaRes <- metaRes
  }

  # Efficiency statistics (computed once for the summary panel)
  effStats <- tryCatch({
      eff <- efficiencies(object)
      grpVar <- object$dataTable$.mf_group

      buildRow <- function(g) {
        idx <- which(grpVar == g)
        nTE <- sum(!is.na(eff$TE_group_BC[idx]))
        nMTR <- sum(!is.na(eff$MTR_BC[idx]))
        c(
          N_group = length(idx),
          N_valid = nTE,
          TE_group_BC = if (nTE > 0) mean(eff$TE_group_BC[idx], na.rm = TRUE) else NA_real_,
          TE_group_JLMS = if (nTE > 0) mean(eff$TE_group_JLMS[idx], na.rm = TRUE) else NA_real_,
          TE_meta_BC = if (nMTR > 0) mean(eff$TE_meta_BC[idx], na.rm = TRUE) else NA_real_,
          TE_meta_JLMS = if (nMTR > 0) mean(eff$TE_meta_JLMS[idx], na.rm = TRUE) else NA_real_,
          MTR_BC = if (nMTR > 0) mean(eff$MTR_BC[idx], na.rm = TRUE) else NA_real_,
          MTR_JLMS = if (nMTR > 0) mean(eff$MTR_JLMS[idx], na.rm = TRUE) else NA_real_
        )
      }
      statMat <- do.call(rbind, lapply(object$groups, buildRow))
      rownames(statMat) <- object$groups

      # LCM: posterior class membership proportions
      postProb <- NULL
      if (object$groupType == "sfalcmcross") {
        # For lcmNoGroup: one pooled model -> compute once and show for all data
        # For grouped LCM: compute per group model
        if (isTRUE(object$lcmNoGroup)) {
          postProb <- list()
          lcm_m <- object$lcmObj
          nc <- lcm_m$nClasses
          eff_g <- tryCatch(efficiencies(lcm_m), error = function(e) NULL)
          if (!is.null(eff_g) && "Group_c" %in% names(eff_g)) {
            assign_tab <- table(eff_g$Group_c)
            assign_prop <- as.numeric(assign_tab) / sum(assign_tab)
            pp_cols <- grep(
              "^PosteriorProb_c[0-9]+$",
              names(eff_g),
              value = TRUE
            )
            mean_pp <- if (length(pp_cols) > 0) {
              colMeans(eff_g[, pp_cols, drop = FALSE], na.rm = TRUE)
            } else {
              rep(NA_real_, nc)
            }
            cls_labels <- paste0("Class ", seq_len(nc))
            pp_df <- data.frame(
              `% assigned` = formatC(
                assign_prop * 100,
                digits = 1,
                format = "f"
              ),
              `Mean post. prob.` = formatC(mean_pp, digits = 3, format = "f"),
              row.names = cls_labels,
              check.names = FALSE
            )
            # Store once - we'll display it once in the print method
            postProb[["pooled"]] <- pp_df
          }
        } else {
          postProb <- lapply(object$groups, function(g) {
            m <- object$groupModels[[g]]
            nc <- m$nClasses
            eff_g <- tryCatch(efficiencies(m), error = function(e) NULL)
            if (is.null(eff_g) || !("Group_c" %in% names(eff_g))) {
              return(NULL)
            }
            assign_tab <- table(eff_g$Group_c)
            assign_prop <- as.numeric(assign_tab) / sum(assign_tab)
            pp_cols <- grep(
              "^PosteriorProb_c[0-9]+$",
              names(eff_g),
              value = TRUE
            )
            mean_pp <- if (length(pp_cols) > 0) {
              colMeans(eff_g[, pp_cols, drop = FALSE], na.rm = TRUE)
            } else {
              rep(NA_real_, nc)
            }
            cls_labels <- paste0("Class ", seq_len(nc))
            data.frame(
              `% assigned` = formatC(
                assign_prop * 100,
                digits = 1,
                format = "f"
              ),
              `Mean post. prob.` = formatC(mean_pp, digits = 3, format = "f"),
              row.names = cls_labels,
              check.names = FALSE
            )
          })
          names(postProb) <- object$groups
        }
      }

      list(statMat = statMat, postProb = postProb)
    },
    error = function(e) NULL
  )

  object$effStats <- effStats
  class(object) <- "summary.smfa"
  return(object)
}


# print for summary.smfa ----------
#' @rdname summary
#' @aliases print.summary.smfa
#' @importFrom utils capture.output
#' @export
print.summary.smfa <- function(
  x,
  digits = max(3, getOption("digits") - 2),
  ...
) {
  lengthSum <- 60
  sep <- paste0(rep("-", lengthSum), collapse = "")
  sep2 <- paste0(rep("=", lengthSum), collapse = "")

  # Helper to print the tests
  .print_tests <- function(m) {
    if (is.null(m$df) || is.null(m$olsLoglik) || is.null(m$chisq) || is.null(m$CoelliM3Test)) {
      return()
    }
    cat("-----[ Tests vs. No Inefficiency ]-----\n")
    cat("Likelihood Ratio Test of Inefficiency\n")
    cat(
      "Deg. freedom for inefficiency model",
      paste0(
        rep(" ", lengthSum - 2 - nchar("Deg. freedom for inefficiency model") - nchar(formatC(m$df, digits = digits, format = "d"))),
        collapse = ""
      ),
      formatC(m$df, digits = digits, format = "d"),
      "\n"
    )
    cat(
      "Log Likelihood for OLS Log(H0) = ",
      paste0(
        rep(" ", lengthSum - 2 - nchar("Log Likelihood for OLS Log(H0) = ") - nchar(formatC(m$olsLoglik, digits = digits, format = "f"))),
        collapse = ""
      ),
      formatC(m$olsLoglik, digits = digits, format = "f"), "\n"
    )
    cat("LR statistic: \n")
    cat(
      "Chisq = 2*[LogL(H0)-LogL(H1)]  = ",
      paste0(
        rep(" ", lengthSum - 2 - nchar("Chisq = 2*[LogL(H0)-LogL(H1)]  = ") - nchar(formatC(m$chisq, digits = digits, format = "f"))),
        collapse = ""
      ),
      formatC(m$chisq, digits = digits, format = "f"), "\n"
    )
    cat(
      "Kodde-Palm C*:       95%:", formatC(.qchibarsq(0.95, df = m$df), digits = digits, format = "f"),
      paste0(
        rep(" ", lengthSum - 2 - nchar("Kodde-Palm C*:       95%:") - nchar(formatC(.qchibarsq(0.95, df = m$df), digits = digits, format = "f")) - nchar(formatC(.qchibarsq(0.99, df = m$df), digits = digits, format = "f")) - nchar("99%") - 3),
        collapse = ""
      ),
      "99%:", formatC(.qchibarsq(0.99, df = m$df), digits = digits, format = "f"), "\n"
    )
    cat("Coelli (1995) skewness test on OLS residuals\n")
    cat(
      "M3T: z                         = ",
      paste0(
        rep(" ", lengthSum - 2 - nchar("M3T: z                         = ") - nchar(formatC(m$CoelliM3Test[1], digits = digits, format = "f"))),
        collapse = ""
      ),
      formatC(m$CoelliM3Test[1], digits = digits, format = "f"), "\n"
    )
    cat(
      "M3T: p.value                   = ",
      paste0(
        rep(" ", lengthSum - 2 - nchar("M3T: p.value                   = ") - nchar(formatC(m$CoelliM3Test[2], digits = digits, format = "f"))),
        collapse = ""
      ),
      formatC(m$CoelliM3Test[2], digits = digits, format = "f"), "\n"
    )
  }

  # ---- Header ----
  cat(sep2, "\n")
  cat("Stochastic Metafrontier Analysis\n")
  cat("Metafrontier method:", mfauxdist(x$metaMethod, x$sfaApproach), "\n")
  cat(x$typeSfa, "\n")
  if (!is.null(x$sfaApproach) && x$metaMethod == "sfa") {
    cat("SFA approach       :", x$sfaApproach, "\n")
  }
  grp_appr <- switch(x$groupType,
    sfacross = "Stochastic Frontier Analysis",
    sfaselectioncross = "Sample Selection Stochastic Frontier Analysis",
    sfalcmcross = "Latent Class Stochastic Frontier Analysis",
    x$groupType
  )
  cat("Group approach     :", grp_appr, "\n")
  cat("Group estimator    :", x$groupType, "\n")
  if (!is.null(x$groupModels[[1]]$optType)) {
    cat("Group optim solver :", x$groupModels[[1]]$optType[[1]], "\n")
  }
  if (isTRUE(x$lcmNoGroup)) {
    cat("  (Pooled LCM - latent classes used as groups)\n")
  }
  cat(
    "Groups (",
    length(x$groups),
    "):",
    paste(x$groups, collapse = ", "),
    "\n"
  )
  cat("Total observations :", x$Nobs, "\n")
  cat("Distribution       :", x$udist, "\n")
  cat(sep2, "\n\n")

  # ---- Per-group sections ----
  # For lcmNoGroup: print the SINGLE pooled LCM model only once, then efficiency by class
  if (isTRUE(x$lcmNoGroup)) {
    lcm_m <- x$lcmObj
    lcm_sm <- x$grpSummaries[[1]] # same object for all classes
    mlR <- lcm_sm$mlRes
    nc <- lcm_m$nClasses
    nX_lcm <- lcm_m$nXvar
    nU_lcm <- lcm_m$nuZUvar
    nV_lcm <- lcm_m$nvZVvar
    nZH <- lcm_m$nZHvar

    cat(sep, "\n")
    cat(sprintf(
      "Pooled LCM (%d classes) on all data (N = %d)  Log-likelihood: %s\n",
      nc,
      nrow(lcm_m$dataTable),
      formatC(lcm_m$mlLoglik, digits = digits, format = "f")
    ))
    cat(sep, "\n")

    for (k in seq_len(nc)) {
      cat(sprintf("\n  -- Latent Class %d --\n", k))
      s_b <- (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + 1
      e_b <- (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm
      s_u <- (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm + 1
      e_u <- (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm + nU_lcm
      s_v <- (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm + nU_lcm + 1
      e_v <- (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm + nU_lcm + nV_lcm
      cat("  Frontier:\n")
      printCoefmat(
        mlR[s_b:e_b, , drop = FALSE],
        P.values = TRUE,
        digits = digits,
        signif.legend = FALSE,
        na.print = "NA"
      )
      cat("  Var(u):\n")
      printCoefmat(
        mlR[s_u:e_u, , drop = FALSE],
        P.values = TRUE,
        digits = digits,
        signif.legend = FALSE,
        na.print = "NA"
      )
      cat("  Var(v):\n")
      printCoefmat(
        mlR[s_v:e_v, , drop = FALSE],
        P.values = TRUE,
        digits = digits,
        signif.legend = FALSE,
        na.print = "NA"
      )
      # Per-class variance derived stats
      vs <- tryCatch({
          # Extract class-k sigma_u^2 from Zu intercept
          # Interleaved position
          Zu_int <- lcm_m$mlParam[
            (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm + 1
          ]
          Zv_int <- lcm_m$mlParam[
            (k - 1) * (nX_lcm + nU_lcm + nV_lcm) + nX_lcm + nU_lcm + 1
          ]
          su2_k <- exp(Zu_int)
          sv2_k <- exp(Zv_int)
          su_k <- sqrt(su2_k)
          sv_k <- sqrt(sv2_k)
          sig_k <- sqrt(su2_k + sv2_k)
          cat(sprintf(
            "  Sigma_u=%.4f  Sigma_v=%.4f  Sigma=%.4f  Gamma=%.4f  Lambda=%.4f\n",
            su_k,
            sv_k,
            sig_k,
            su2_k / (su2_k + sv2_k),
            su_k / sv_k
          ))
          invisible(NULL)
        },
        error = function(e) NULL
      )
    }
    # Class membership block
    s_th <- nc * (nX_lcm + nU_lcm + nV_lcm) + 1
    e_th <- nc * (nX_lcm + nU_lcm + nV_lcm) + (nc - 1) * nZH
    if (e_th >= s_th && nrow(mlR) >= e_th) {
      cat("\n  -- Class Membership (logit) --\n")
      printCoefmat(
        mlR[s_th:e_th, , drop = FALSE],
        P.values = TRUE,
        digits = digits,
        signif.legend = TRUE,
        na.print = "NA"
      )
    }
    .print_tests(lcm_sm)
    if (!is.null(lcm_m$optStatus)) {
      cat("Log likelihood status:", lcm_m$optStatus, "\n")
    }
    cat("\n")
  } else {
    # Normal per-group loop
    for (g in x$groups) {
      m <- x$groupModels[[g]]
      sm <- x$grpSummaries[[g]]
      Ng <- nrow(m$dataTable)
      mlR <- sm$mlRes
      cat(sep, "\n")
      cat(
        "Group:",
        g,
        sprintf(
          "(N = %d)  Log-likelihood: %s\n",
          Ng,
          formatC(m$mlLoglik, digits = digits, format = "f")
        )
      )
      cat(sep, "\n")

      if (x$groupType == "sfacross") {
        # ------- sfacross: delegate entirely to sfaR's own print method -------
        # This guarantees SEs, z-values, p-values, and derived variance statistics
        # (sigma_u, sigma_v, gamma, lambda, E[u], E[exp(-u)]) are rendered
        # exactly as sfaR would display them - no custom delta-method block.
        out_lines <- tryCatch(
          utils::capture.output(print(sm, digits = digits)),
          error = function(e) NULL
        )
        if (!is.null(out_lines)) {
          cat(paste(out_lines, collapse = "\n"), "\n")
        } else if (!is.null(mlR)) {
          printCoefmat(mlR,
            P.values = TRUE, digits = digits,
            signif.legend = TRUE, na.print = "NA"
          )
        }
      } else if (x$groupType == "sfalcmcross") {
        # ------- LCM: one block per class -------
        nc <- m$nClasses
        nX <- m$nXvar
        nU <- m$nuZUvar
        nV <- m$nvZVvar
        nZH <- m$nZHvar
        for (k in seq_len(nc)) {
          cat(sprintf("\n  -- Latent Class %d --\n", k))
          s_b <- (k - 1) * (nX + nU + nV) + 1
          e_b <- (k - 1) * (nX + nU + nV) + nX
          s_u <- (k - 1) * (nX + nU + nV) + nX + 1
          e_u <- (k - 1) * (nX + nU + nV) + nX + nU
          s_v <- (k - 1) * (nX + nU + nV) + nX + nU + 1
          e_v <- (k - 1) * (nX + nU + nV) + nX + nU + nV
          cat("  Frontier:\n")
          printCoefmat(
            mlR[s_b:e_b, , drop = FALSE],
            P.values = TRUE,
            digits = digits,
            signif.legend = FALSE,
            na.print = "NA"
          )
          cat("  Var(u):\n")
          printCoefmat(
            mlR[s_u:e_u, , drop = FALSE],
            P.values = TRUE,
            digits = digits,
            signif.legend = FALSE,
            na.print = "NA"
          )
          cat("  Var(v):\n")
          printCoefmat(
            mlR[s_v:e_v, , drop = FALSE],
            P.values = TRUE,
            digits = digits,
            signif.legend = FALSE,
            na.print = "NA"
          )
          # Per-class derived variance stats (point estimates only from mlParam)
          tryCatch({
              Zu_int <- m$mlParam[(k - 1) * (nX + nU + nV) + nX + 1]
              Zv_int <- m$mlParam[(k - 1) * (nX + nU + nV) + nX + nU + 1]
              su2_k <- exp(Zu_int)
              sv2_k <- exp(Zv_int)
              su_k <- sqrt(su2_k)
              sv_k <- sqrt(sv2_k)
              sig_k <- sqrt(su2_k + sv2_k)
              cat(sprintf(
                "  Sigma_u=%.4f  Sigma_v=%.4f  Sigma=%.4f  Gamma=%.4f  Lambda=%.4f\n",
                su_k,
                sv_k,
                sig_k,
                su2_k / (su2_k + sv2_k),
                su_k / sv_k
              ))
            },
            error = function(e) NULL
          )
        }
        s_th <- nc * (nX + nU + nV) + 1
        e_th <- nc * (nX + nU + nV) + (nc - 1) * nZH
        if (e_th >= s_th && nrow(mlR) >= e_th) {
          cat("\n  -- Class Membership (logit) --\n")
          printCoefmat(
            mlR[s_th:e_th, , drop = FALSE],
            P.values = TRUE,
            digits = digits,
            signif.legend = TRUE,
            na.print = "NA"
          )
        }
      } else if (x$groupType == "sfaselectioncross") {
        # ------- sfaselectioncross: delegate to sfaR's own print method -------
        out_lines <- tryCatch(
          utils::capture.output(print(sm, digits = digits)),
          error = function(e) NULL
        )
        if (!is.null(out_lines)) {
          cat(paste(out_lines, collapse = "\n"), "\n")
        } else if (!is.null(mlR)) {
          printCoefmat(mlR,
            P.values = TRUE, digits = digits,
            signif.legend = TRUE, na.print = "NA"
          )
        }
      }
      # For sfalcmcross, sfaR's print.summary.sfalcmcross does NOT include the
      # tests / optStatus block, so we emit it explicitly here.
      # For sfacross / sfaselectioncross, sfaR's print.summary.* already emits
      # these, so we skip them to avoid duplication.
      if (x$groupType == "sfalcmcross") {
        .print_tests(sm)
        if (!is.null(m$optStatus)) {
          cat("Log likelihood status:", m$optStatus, "\n")
        }
      }
      cat("\n")
    }
  } # end if/else lcmNoGroup

  # ---- Metafrontier coefficients ----
  cat(sep, "\n")
  cat("Metafrontier Coefficients (", x$metaMethod, "):\n", sep = "")
  if (!is.null(x$metaSfaObj) && !is.null(x$metaSfaObj$optType)) {
    cat("Meta-optim solver  :", x$metaSfaObj$optType[[1]], "\n")
  }
  if (!is.null(x$metaRes)) {
    printCoefmat(
      x$metaRes,
      P.values = ncol(x$metaRes) == 4,
      digits = digits,
      signif.legend = TRUE,
      na.print = "NA"
    )
    # Metafrontier variance stats (SFA method only)
    if (x$metaMethod == "sfa" && !is.null(x$metaSfaObj)) {
      # Delegate metafrontier SFA display to sfaR's own print method so that
      # variance statistics (sigma_u, sigma_v, gamma, lambda, E[u]) are shown
      # exactly as sfaR would display them, with correct Hessian-based SEs.
      meta_sm <- suppressWarnings(tryCatch(summary(x$metaSfaObj), error = function(e) NULL))
      if (!is.null(meta_sm)) {
        cat("\n  Meta-frontier model details:\n")
        out_meta <- tryCatch(
          utils::capture.output(print(meta_sm, digits = digits)),
          error = function(e) NULL
        )
        if (!is.null(out_meta)) {
          cat(paste(out_meta, collapse = "\n"), "\n")
        } else {
          .print_tests(meta_sm)
        }
      }
      if (!is.null(x$metaSfaObj$optStatus)) {
        cat("Log likelihood status:", x$metaSfaObj$optStatus, "\n")
      }
    }
  } else {
    cat("  (LP: deterministic envelope - no estimated parameters)\n")
  }
  cat("\n")

  # ---- Efficiency statistics panel ----
  if (!is.null(x$effStats)) {
    cat(sep, "\n")
    cat("Efficiency Statistics (group means):\n")
    cat(sep, "\n")
    sm_eff <- x$effStats$statMat
    out <- data.frame(
      N_obs = formatC(sm_eff[, "N_group"], format = "d"),
      N_valid = formatC(sm_eff[, "N_valid"], format = "d"),
      TE_group_BC = formatC(sm_eff[, "TE_group_BC"], digits = digits, format = "f"),
      TE_group_JLMS = formatC(sm_eff[, "TE_group_JLMS"], digits = digits, format = "f"),
      TE_meta_BC = formatC(sm_eff[, "TE_meta_BC"], digits = digits, format = "f"),
      TE_meta_JLMS = formatC(sm_eff[, "TE_meta_JLMS"], digits = digits, format = "f"),
      MTR_BC = formatC(sm_eff[, "MTR_BC"], digits = digits, format = "f"),
      MTR_JLMS = formatC(sm_eff[, "MTR_JLMS"], digits = digits, format = "f"),
      row.names = rownames(sm_eff),
      stringsAsFactors = FALSE
    )
    print(out)
    ov <- colMeans(
      sm_eff[, c("TE_group_BC", "TE_group_JLMS", "TE_meta_BC", "TE_meta_JLMS", "MTR_BC", "MTR_JLMS"), drop = FALSE],
      na.rm = TRUE
    )
    # colMeans returns NaN (not NA) when all values in a column are NA — fix that
    ov[is.nan(ov)] <- NA_real_
    fmt_val <- function(v) if (is.na(v)) "NA" else sprintf("%.4f", v)
    cat(sprintf(
      "\nOverall:\nTE_group_BC=%s  TE_group_JLMS=%s\nTE_meta_BC=%s   TE_meta_JLMS=%s\nMTR_BC=%s     MTR_JLMS=%s\n",
      fmt_val(ov["TE_group_BC"]), fmt_val(ov["TE_group_JLMS"]),
      fmt_val(ov["TE_meta_BC"]), fmt_val(ov["TE_meta_JLMS"]),
      fmt_val(ov["MTR_BC"]), fmt_val(ov["MTR_JLMS"])
    ))

    # LCM: posterior class proportions
    pp_list <- x$effStats$postProb
    if (!is.null(pp_list) && any(!sapply(pp_list, is.null))) {
      cat("\n")
      cat(sep, "\n")
      if (isTRUE(x$lcmNoGroup)) {
        cat("Posterior Class Membership (pooled LCM):\n")
        cat(sep, "\n")
        pp <- pp_list[["pooled"]]
        if (!is.null(pp) && is.data.frame(pp)) print(pp)
      } else {
        cat("Posterior Class Membership (by group):\n")
        cat(sep, "\n")
        for (g in x$groups) {
          pp <- pp_list[[g]]
          if (!is.null(pp) && is.data.frame(pp)) {
            cat("  Group:", g, "\n")
            print(pp)
            cat("\n")
          }
        }
      }
    }
  }

  # ---- Information criteria ----
  cat(sep, "\n")
  cat("Total Log-likelihood:", round(x$mlLoglik, digits), "\n")
  cat(
    "AIC:",
    round(x$AIC, digits),
    "  BIC:",
    round(x$BIC, digits),
    "  HQIC:",
    round(x$HQIC, digits),
    "\n"
  )
  if (!is.null(x$tests)) {
    cat(sep, "\n")
    cat("Tests vs. No Inefficiency:\n")
    printCoefmat(
      x$tests,
      P.values = TRUE,
      has.Pvalue = TRUE,
      digits = digits,
      signif.legend = TRUE,
      na.print = "NA"
    )
  }
  cat(sep, "\n")
  if (!is.null(x$optStatus)) {
    cat("Log likelihood status:", x$optStatus, "\n")
  }
  cat(x$mlDate, "\n")
  invisible(x)
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Fitted values of models                                                      #
# Models:                                                                      #
#           -Stochastic metafrontier analysis                                  #
#           -Latent class stochastic metafrontier analysis                     #
#           -Sample selection correction for stochastic metafrontier model     #
# Data: Cross sectional data & Pooled data                                     #
#------------------------------------------------------------------------------#

#' Extract fitted values of stochastic metafrontier models
#'
#' \code{\link{fitted}} returns the fitted frontier values from stochastic
#' metafrontier models estimated with \code{\link{smfa}}.
#'
#' @param object A stochastic metafrontier model returned
#' by \code{\link{smfa}}.
#' @param ... Currently ignored.
#'
#' @name fitted
#'
#' @return A vector of fitted values is returned.
#'
#' @note The fitted values are ordered in the same way as the corresponding
#' observations in the dataset used for the estimation.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data.
#'
#' @keywords methods fitted
#'
# fitted values for smfa (returns metafrontier fitted values) ----------
#' @rdname fitted
#' @aliases fitted.smfa
#' @importFrom stats fitted
#' @export
fitted.smfa <- function(object, ...) {
  object$dataTable$.mf_yhat_meta
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Residuals of model (v - S * u)                                               #
# Models:                                                                      #
#           -Stochastic metafrontier analysis                                  #
#           -Latent class stochastic metafrontier analysis                     #
#           -Sample selection correction for stochastic metafrontier model     #
# Data: Cross sectional data & Pooled data                                     #
#------------------------------------------------------------------------------#

#' Extract residuals of stochastic metafrontier models
#'
#' This function returns the residuals' values from stochastic metafrontier models
#' estimated with \code{\link{smfa}}.
#'
#' @param object A stochastic metafrontier model returned
#' by \code{\link{smfa}}.
#' @param \dots Currently ignored.
#'
#' @name residuals
#'
#' @return \code{\link{residuals}} returns a vector of residuals values.
#'
#' @note The residuals values are ordered in the same way as the corresponding
#' observations in the dataset used for the estimation.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data.
#'
#' @keywords methods residuals
#'
# residuals for smfa (residuals vs. metafrontier) ----------
#' @rdname residuals
#' @aliases residuals.smfa
#' @importFrom stats residuals
#' @export
residuals.smfa <- function(object, ...) {
  # Residual = observed Y - metafrontier fitted value
  Yvar <- object$dataTable[[all.vars(object$formula)[1]]]
  if (is.null(Yvar)) {
    # Reconstruct Y from formula and data
    Yvar <- model.response(model.frame(
      object$formula,
      data = object$dataTable,
      na.action = na.pass
    ))
  }
  as.numeric(Yvar) - object$dataTable$.mf_yhat_meta
}

################################################################################
#                                                                              #
# R functions for the smfa package                                             #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Model: Stochastic Metafrontier Analysis                                      #
# Data: Cross-sectional or pooled data with group variable                     #
#------------------------------------------------------------------------------#

#' Stochastic metafrontier estimation
#'
#' @description
#' \code{\link{smfa}} estimates a stochastic metafrontier model
#' for cross-sectional or pooled data. The function follows the theoretical
#' frameworks of Battese, Rao, and O'Donnell (2004) and O'Donnell, Rao, and
#' Battese (2008), and additionally implements the two-stage stochastic approach
#' of Huang, Huang, and Liu (2014). Three types of group-level frontier models
#' are supported: standard stochastic frontier analysis
#' (\code{\link[sfaR]{sfacross}}), sample selection stochastic frontier
#' analysis (\code{\link[sfaR]{sfaselectioncross}}), and latent class stochastic
#' frontier analysis (\code{\link[sfaR]{sfalcmcross}}).
#'
#' @aliases smfa print.smfa
#'
#' @param formula A symbolic description of the frontier model to be estimated,
#'   based on the generic function \code{\link[stats]{formula}}. For
#'   \code{groupType = "sfaselectioncross"}, this argument specifies the
#'   frontier (outcome) equation and must be a standard formula whose left-hand
#'   side is the output (or cost) variable and whose right-hand side contains
#'   the frontier regressors (see also \code{selectionF}).
#' @param muhet A one-part formula to account for heterogeneity in the mean of
#'   the pre-truncated normal distribution. Applicable only when
#'   \code{groupType = "sfacross"} and \code{udist = "tnormal"}. The variables
#'   specified model the conditional mean
#'   \eqn{\mu_i = \bm{\omega}'\mathbf{Z}_{\mu}} of the truncated normal
#'   inefficiency distribution (see section \sQuote{Details}).
#' @param uhet A one-part formula to account for heteroscedasticity in the
#'   one-sided error variance. Applicable for all three model types. The
#'   variance of the inefficiency term is modelled as
#'   \eqn{\sigma^2_u = \exp(\bm{\delta}'\mathbf{Z}_u)}, where
#'   \eqn{\mathbf{Z}_u} are the inefficiency drivers and \eqn{\bm{\delta}}
#'   the associated coefficients (see section \sQuote{Details}).
#' @param vhet A one-part formula to account for heteroscedasticity in the
#'   two-sided error variance. Applicable for all three model types. The
#'   variance of the noise term is modelled as
#'   \eqn{\sigma^2_v = \exp(\bm{\phi}'\mathbf{Z}_v)}, where
#'   \eqn{\mathbf{Z}_v} are the heteroscedasticity variables and
#'   \eqn{\bm{\phi}} the coefficients (see section \sQuote{Details}).
#' @param thet A one-part formula to account for technological heterogeneity
#'   in the construction of the latent classes. Applicable only when
#'   \code{groupType = "sfalcmcross"}. The variables specified enter the logit
#'   formulation that determines the prior class membership probabilities
#'   \eqn{\pi(i,j)} (see section \sQuote{Details}).
#' @param logDepVar Logical. Informs whether the dependent variable is logged
#'   (\code{TRUE}) or not (\code{FALSE}). Default \code{TRUE}. Must match the
#'   transformation applied to the left-hand side of \code{formula}.
#' @param data A data frame containing all variables referenced in
#'   \code{formula}, \code{selectionF}, \code{muhet}, \code{uhet}, \code{vhet},
#'   \code{thet}, and \code{group}.
#' @param subset An optional vector specifying a subset of observations to be
#'   used in the estimation process.
#' @param weights An optional vector of weights to be used for weighted
#'   log-likelihood estimation. Should be \code{NULL} or a numeric vector with
#'   strictly positive values. When \code{NULL} (default), all observations
#'   receive equal weight.
#' @param wscale Logical. When \code{weights} is not \code{NULL}, a scaling
#'   transformation is applied such that the weights sum to the sample size:
#'   \deqn{w_{\mathrm{new}} = n \times
#'     \frac{w_{\mathrm{old}}}{\sum w_{\mathrm{old}}}}
#'   Default \code{TRUE}. When \code{FALSE}, the raw weights are used without
#'   scaling.
#' @param group Character string. The name of the column in \code{data}
#'   identifying the technology group of each observation. The column is coerced
#'   to a factor internally and must have at least two unique values. When
#'   \code{groupType = "sfalcmcross"} and \code{group} is \code{NULL}, a single
#'   pooled latent class model is estimated and class assignments serve as groups
#'   (see section \sQuote{Details}).
#' @param S Integer. Frontier orientation.
#'   \itemize{
#'     \item \code{S = 1} (default): production or profit frontier,
#'       \eqn{\varepsilon_i = v_i - u_i}.
#'     \item \code{S = -1}: cost frontier,
#'       \eqn{\varepsilon_i = v_i + u_i}.
#'   }
#' @param udist Character string. Distribution for the one-sided error term
#'   \eqn{u_i \ge 0}. The following distributions are available for
#'   \code{groupType = "sfacross"}:
#'   \itemize{
#'     \item \code{"hnormal"} (default): half-normal distribution
#'       (Aigner \emph{et al.}, 1977; Meeusen and van den Broeck, 1977).
#'     \item \code{"exponential"}: exponential distribution.
#'     \item \code{"tnormal"}: truncated normal distribution (Stevenson, 1980).
#'     \item \code{"rayleigh"}: Rayleigh distribution (Hajargasht, 2015).
#'     \item \code{"uniform"}: uniform distribution (Li, 1996; Nguyen, 2010).
#'     \item \code{"gamma"}: Gamma distribution, estimated by maximum simulated
#'       likelihood (Greene, 2003).
#'     \item \code{"lognormal"}: log-normal distribution, estimated by maximum
#'       simulated likelihood (Migon and Medici, 2001; Wang and Ye, 2020).
#'     \item \code{"weibull"}: Weibull distribution, estimated by maximum
#'       simulated likelihood (Tsionas, 2007).
#'     \item \code{"genexponential"}: generalised exponential distribution
#'       (Papadopoulos, 2020).
#'     \item \code{"tslaplace"}: truncated skewed Laplace distribution
#'       (Wang, 2012).
#'   }
#'   For \code{groupType = "sfaselectioncross"} and \code{"sfalcmcross"}, only
#'   \code{"hnormal"} is currently supported.
#' @param start Numeric vector. Optional starting values for the maximum
#'   likelihood (ML) or maximum simulated likelihood (MSL) estimation of the
#'   group-level frontier models. When \code{NULL} (default), starting values
#'   are computed automatically. For \code{groupType = "sfacross"}, they are
#'   derived from OLS residuals. For \code{groupType = "sfalcmcross"}, they
#'   depend on \code{whichStart}.
#' @param scaling Logical. Applicable only when \code{groupType = "sfacross"}
#'   and \code{udist = "tnormal"}. When \code{TRUE}, the scaling property model
#'   (Wang and Schmidt, 2002) is estimated, whereby
#'   \eqn{u_i = h(\mathbf{Z}_u, \bm{\delta}) u^*_i} and \eqn{u^*_i} follows
#'   a truncated normal distribution \eqn{N^+(\tau, \exp(c_u))}. Default
#'   \code{FALSE}.
#' @param modelType Character string. Applicable only when
#'   \code{groupType = "sfaselectioncross"}. Specifies the model used to correct
#'   for selection bias. Currently, only \code{"greene10"} (default) is
#'   supported, corresponding to the two-step approach of Greene (2010): a
#'   probit model is estimated for the selection equation, and its inverse Mills
#'   ratio is included as a correction term in the stochastic frontier second
#'   step.
#' @param groupType Character string. Type of frontier model estimated for each
#'   technology group. Three options are available:
#'   \itemize{
#'     \item \code{"sfacross"} (default): standard cross-sectional stochastic
#'       frontier analysis (\code{\link[sfaR]{sfacross}}). Groups are defined
#'       by the \code{group} variable. All 10 distributions for \code{udist}
#'       are supported, along with heteroscedasticity in both error components
#'       (\code{uhet}, \code{vhet}), heterogeneity in the truncated mean
#'       (\code{muhet}), and the scaling property.
#'     \item \code{"sfaselectioncross"}: sample selection stochastic frontier
#'       analysis (\code{\link[sfaR]{sfaselectioncross}}). Corrects for sample
#'       selection bias via the generalised Heckman approach (Greene, 2010).
#'       Requires \code{selectionF}. Only observations for which the selection
#'       indicator equals one enter the frontier and metafrontier; efficiency
#'       estimates for non-selected observations are \code{NA}. Only
#'       \code{udist = "hnormal"} is supported.
#'     \item \code{"sfalcmcross"}: latent class stochastic frontier analysis
#'       (\code{\link[sfaR]{sfalcmcross}}). Estimates a finite mixture of
#'       frontier models with the number of classes determined by
#'       \code{lcmClasses}. When \code{group} is supplied, a separate latent
#'       class model is estimated per group-stratum and combined for the
#'       metafrontier. When \code{group} is omitted, a single pooled model is
#'       estimated and class assignments serve as technology groups. Supports
#'       \code{thet} for class-membership covariates and \code{uhet},
#'       \code{vhet} for within-class heteroscedasticity. Only
#'       \code{udist = "hnormal"} is supported.
#'   }
#' @param metaMethod Character string. Method for estimating the global
#'   metafrontier that envelopes all group frontiers. Three options are
#'   available:
#'   \itemize{
#'     \item \code{"lp"} (default): deterministic linear programming envelope.
#'       Finds the parameter vector \eqn{\bm{\beta}^*} minimising
#'       \eqn{\sum_i |\ln \hat{f}(x_i, \bm{\beta}^*) - \ln \hat{f}(x_i,
#'       \hat{\bm{\beta}}_{(g)})|}
#'       subject to \eqn{\ln \hat{f}(x_i, \bm{\beta}^*) \ge \ln \hat{f}(x_i,
#'       \hat{\bm{\beta}}_{(g)})} for all observations and all groups
#'       (Battese \emph{et al.}, 2004).
#'     \item \code{"qp"}: deterministic quadratic programming envelope.
#'       Minimises the sum of squared deviations under the same envelope
#'       constraint.
#'     \item \code{"sfa"}: stochastic metafrontier estimated by a second-stage
#'       pooled SFA. The specific construction of the dependent variable is
#'       determined by \code{sfaApproach}.
#'   }
#' @param sfaApproach Character string. Applicable only when
#'   \code{metaMethod = "sfa"}. Determines how the second-stage SFA is
#'   constructed:
#'   \itemize{
#'     \item \code{"ordonnell"} (default): The LP envelope of the group frontier
#'       predicted values is re-estimated with a stochastic frontier, following
#'       O'Donnell, Rao, and Battese (2008). The second-stage SFA directly
#'       targets the global technology envelope.
#'     \item \code{"huang"}: the group-specific fitted frontier value
#'       \eqn{\ln \hat{y}^g_i} for each observation is used as the dependent
#'       variable in a pooled cross-sectional SFA
#'       (Huang, Huang, and Liu, 2014). The technology gap \eqn{U_i \ge 0} and
#'       second-stage noise \eqn{V_i} are estimated directly by the SFA
#'       procedure.
#'     \item \code{"ordonnell"}: the column-wise maximum of all group-fitted
#'       frontier values (the deterministic LP envelope) is used as the dependent
#'       variable in the second-stage SFA (O'Donnell, Rao, and Battese, 2008).
#'   }
#' @param selectionF A two-sided formula specifying the sample selection
#'   equation, e.g., \code{selected ~ z1 + z2}. The left-hand side must be a
#'   binary (0/1) indicator already present in \code{data}: \code{1} means the
#'   observation participates in the frontier and metafrontier; \code{0} means
#'   it is excluded (efficiency estimates will be \code{NA}). Alternatively, a
#'   named list of formulas, one per group level, may be supplied to allow
#'   group-specific selection equations. Required when
#'   \code{groupType = "sfaselectioncross"}; ignored otherwise.
#' @param lcmClasses Integer. Number of latent classes to be estimated per group
#'   when \code{groupType = "sfalcmcross"}. Must be between \code{2} and
#'   \code{5} (default \code{2}). The optimal number of classes can be selected
#'   based on information criteria (see \code{\link[sfaR]{ic}}).
#' @param whichStart Integer. Strategy for obtaining starting values in the
#'   latent class model (\code{groupType = "sfalcmcross"}):
#'   \itemize{
#'     \item \code{1}: starting values are obtained from the method of moments.
#'     \item \code{2} (default): the model is initialised by first solving a
#'       homoscedastic pooled cross-sectional SFA using the algorithm specified
#'       by \code{initAlg} for at most \code{initIter} iterations.
#'   }
#' @param initAlg Character string. Optimisation algorithm used during the
#'   initialisation of the latent class model when \code{whichStart = 2}.
#'   Only algorithms from the \code{maxLik} package are supported:
#'   \itemize{
#'     \item \code{"nm"} (default): Nelder-Mead (see
#'       \code{\link[maxLik]{maxNM}}).
#'     \item \code{"bfgs"}: Broyden-Fletcher-Goldfarb-Shanno (see
#'       \code{\link[maxLik]{maxBFGS}}).
#'     \item \code{"bhhh"}: Berndt-Hall-Hall-Hausman (see
#'       \code{\link[maxLik]{maxBHHH}}).
#'     \item \code{"nr"}: Newton-Raphson (see \code{\link[maxLik]{maxNR}}).
#'     \item \code{"cg"}: Conjugate Gradient (see
#'       \code{\link[maxLik]{maxCG}}).
#'     \item \code{"sann"}: Simulated Annealing (see
#'       \code{\link[maxLik]{maxSANN}}).
#'   }
#' @param initIter Integer. Maximum number of iterations for the initialisation
#'   algorithm when \code{whichStart = 2} and
#'   \code{groupType = "sfalcmcross"}. Default \code{100}.
#' @param lType Character string. Specifies how the likelihood is evaluated for
#'   the selection model (\code{groupType = "sfaselectioncross"}). Five options
#'   are available:
#'   \itemize{
#'     \item \code{"ghermite"} (default): Gauss-Hermite quadrature (see
#'       \code{\link[fastGHQuad]{gaussHermiteData}}).
#'     \item \code{"kronrod"}: Gauss-Kronrod quadrature (see
#'       \code{\link[stats]{integrate}}).
#'     \item \code{"hcubature"}: adaptive integration over hypercubes (see
#'       \code{\link[cubature]{hcubature}}).
#'     \item \code{"pcubature"}: p-adaptive cubature (see
#'       \code{\link[cubature]{pcubature}}).
#'     \item \code{"msl"}: maximum simulated likelihood (controlled by
#'       \code{simType}, \code{Nsim}, \code{prime}, \code{burn},
#'       \code{antithetics}, and \code{seed}).
#'   }
#' @param Nsub Integer. Number of quadrature nodes or integration subdivisions
#'   when \code{lType} is \code{"ghermite"}, \code{"kronrod"},
#'   \code{"hcubature"}, or \code{"pcubature"}. Applicable only when
#'   \code{groupType = "sfaselectioncross"}. Default \code{100}.
#' @param uBound Numeric. Upper bound for the numerical integration of the
#'   inefficiency component when \code{lType} is \code{"kronrod"},
#'   \code{"hcubature"}, or \code{"pcubature"}. For Gauss-Hermite the bound is
#'   automatically infinite. Applicable only when
#'   \code{groupType = "sfaselectioncross"}. Default \code{Inf}.
#' @param intol Numeric. Integration tolerance for the quadrature approaches
#'   \code{"kronrod"}, \code{"hcubature"}, and \code{"pcubature"}. Applicable
#'   only when \code{groupType = "sfaselectioncross"}. Default \code{1e-6}.
#' @param method Character string. Optimisation algorithm for the main ML/MSL
#'   estimation of each group-level frontier model. Default \code{"bfgs"}.
#'   Eleven algorithms are available:
#'   \itemize{
#'     \item \code{"bfgs"}: Broyden-Fletcher-Goldfarb-Shanno (see
#'       \code{\link[maxLik]{maxBFGS}}).
#'     \item \code{"bhhh"}: Berndt-Hall-Hall-Hausman (see
#'       \code{\link[maxLik]{maxBHHH}}).
#'     \item \code{"nr"}: Newton-Raphson (see \code{\link[maxLik]{maxNR}}).
#'     \item \code{"nm"}: Nelder-Mead (see \code{\link[maxLik]{maxNM}}).
#'     \item \code{"cg"}: Conjugate Gradient (see
#'       \code{\link[maxLik]{maxCG}}).
#'     \item \code{"sann"}: Simulated Annealing (see
#'       \code{\link[maxLik]{maxSANN}}).
#'     \item \code{"ucminf"}: quasi-Newton optimisation with BFGS updating of
#'       the inverse Hessian and soft line search (see
#'       \code{\link[ucminf]{ucminf}}).
#'     \item \code{"mla"}: Marquardt-Levenberg algorithm (see
#'       \code{\link[marqLevAlg]{mla}}).
#'     \item \code{"sr1"}: Symmetric Rank 1 trust-region method (see
#'       \code{\link[trustOptim]{trust.optim}}).
#'     \item \code{"sparse"}: trust-region method with sparse Hessian (see
#'       \code{\link[trustOptim]{trust.optim}}).
#'     \item \code{"nlminb"}: PORT routines optimisation (see
#'       \code{\link[stats]{nlminb}}).
#'   }
#' @param hessianType Integer. Specifies which Hessian is returned for the
#'   group-level frontier estimation. The accepted values match those of the
#'   underlying \code{sfaR} function for each \code{groupType}:
#'   \itemize{
#'     \item For \code{groupType = "sfacross"}: if \code{1} (default), the
#'       analytic Hessian is returned; if \code{2}, the BHHH Hessian
#'       \eqn{\mathbf{G}'\mathbf{G}} is estimated.
#'     \item For \code{groupType = "sfalcmcross"}: if \code{1} (default),
#'       the analytic Hessian is returned; if \code{2}, the BHHH Hessian is
#'       estimated.
#'     \item For \code{groupType = "sfaselectioncross"}: if \code{1}, the
#'       analytic Hessian is returned; if \code{2} (default), the BHHH
#'       Hessian \eqn{\mathbf{G}'\mathbf{G}} is estimated. The BHHH default
#'       reflects the two-step nature of the selection estimator.
#'   }
#'   When \code{NULL} (the package default), each group-level model uses the
#'   natural default of the corresponding \code{sfaR} function, ensuring that
#'   standard errors computed by \code{smfa} are identical to those
#'   from a standalone \code{sfaR} call on the same group subset.
#' @param simType Character string. Simulation method for maximum simulated
#'   likelihood (MSL). Applicable to \code{groupType = "sfacross"} when
#'   \code{udist} is \code{"gamma"}, \code{"lognormal"}, or \code{"weibull"},
#'   and to \code{groupType = "sfaselectioncross"} when \code{lType = "msl"}:
#'   \itemize{
#'     \item \code{"halton"} (default): Halton quasi-random sequences.
#'     \item \code{"ghalton"}: Generalised-Halton sequences.
#'     \item \code{"sobol"}: Sobol low-discrepancy sequences.
#'     \item \code{"uniform"}: pseudo-random uniform draws.
#'   }
#' @param Nsim Integer. Number of simulation draws for MSL. Default \code{100}.
#' @param prime Integer. Prime number used to construct Halton or
#'   Generalised-Halton sequences. Default \code{2}.
#' @param burn Integer. Number of leading draws discarded from the Halton
#'   sequence to reduce serial correlation. Default \code{10}.
#' @param antithetics Logical. If \code{TRUE}, antithetic draws are added: the
#'   first \code{Nsim/2} draws are taken, and the remaining \code{Nsim/2} are
#'   \eqn{1 - \text{draw}}. Default \code{FALSE}.
#' @param seed Integer. Random seed for simulation draws, ensuring
#'   reproducibility of MSL estimates. Default \code{12345}.
#' @param itermax Integer. Maximum number of iterations for the main
#'   optimisation. Default \code{2000}. For \code{method = "sann"}, it is
#'   recommended to increase this substantially (e.g., \code{itermax = 20000}).
#' @param printInfo Logical. If \code{TRUE}, optimisation progress is printed
#'   during estimation of each group-level model. Default \code{FALSE}.
#' @param tol Numeric. Convergence tolerance. The algorithm is considered
#'   converged when the change in the log-likelihood between successive
#'   iterations is smaller than \code{tol} in absolute value. Default
#'   \code{1e-12}.
#' @param gradtol Numeric. Gradient convergence tolerance. The algorithm is
#'   considered converged when the Euclidean norm of the gradient is smaller
#'   than \code{gradtol}. Default \code{1e-6}.
#' @param stepmax Numeric. Maximum step length used by the \code{"ucminf"}
#'   algorithm. Default \code{0.1}.
#' @param qac Character string. Quadratic Approximation Correction for the
#'   \code{"bhhh"} and \code{"nr"} algorithms when the Hessian is not negative
#'   definite:
#'   \itemize{
#'     \item \code{"marquardt"} (default): step length is decreased while
#'       also shifting closer to the gradient direction.
#'     \item \code{"stephalving"}: step length is halved, preserving the
#'       current direction.
#'   }
#'   See \code{\link[maxLik]{maxBHHH}} and \code{\link[maxLik]{maxNR}} for
#'   details.
#' @param x An object of class \code{"smfa"}, as returned by
#'   \code{smfa}, for use with the \code{print} method.
#' @param ... Additional arguments passed through to the second-stage SFA
#'   call when \code{metaMethod = "sfa"}.
#'
#' @return \code{smfa} returns an object of class
#'   \code{"smfa"}, which is a list containing:
#'   \item{call}{The matched call.}
#'   \item{groupModels}{A named list of fitted group-level frontier objects,
#'     one per technology group. Each element is of class \code{"sfacross"},
#'     \code{"sfaselectioncross"}, or \code{"sfalcmcross"}, depending on
#'     \code{groupType}.}
#'   \item{metaSfaObj}{The fitted metafrontier object. For
#'     \code{metaMethod = "sfa"}, an object of class \code{"sfacross"}
#'     from the second-stage SFA. The dependent variable column in
#'     \code{metaSfaObj$dataTable} is named according to the approach used:
#'     \code{"lp_envelope"} when \code{sfaApproach = "ordonnell"} (the
#'     column-wise maximum of all group-evaluated frontier values is the
#'     dependent variable) and \code{"group_fitted_values"} when
#'     \code{sfaApproach = "huang"} (each observation's own-group fitted
#'     frontier value is the dependent variable). For \code{metaMethod = "lp"}
#'     or \code{"qp"}, a list containing the optimisation result and the
#'     estimated envelope coefficients.}
#'   \item{metaRes}{Estimated metafrontier coefficients (with standard errors,
#'     z-values, and p-values for \code{metaMethod = "sfa"}, or the plain
#'     coefficient vector for deterministic envelopes).}
#'   \item{formula}{The \code{formula} supplied to the call.}
#'   \item{metaMethod}{The metafrontier estimation method used.}
#'   \item{sfaApproach}{The second-stage SFA approach; \code{NA} when
#'     \code{metaMethod} is not \code{"sfa"}.}
#'   \item{groupType}{The type of group-level frontier model estimated.}
#'   \item{group}{The name of the grouping variable.}
#'   \item{groups}{Character vector of unique group labels.}
#'   \item{S}{The frontier orientation (\code{1} or \code{-1}).}
#'   \item{dataTable}{The data used in estimation, augmented with
#'     \code{.mf_yhat_group} (group-specific fitted frontier values) and
#'     \code{.mf_yhat_meta} (metafrontier fitted values).}
#'   \item{lcmNoGroup}{Logical. \code{TRUE} when \code{groupType = "sfalcmcross"}
#'     and \code{group} was not supplied.}
#'   \item{lcmObj}{When \code{lcmNoGroup = TRUE}, the pooled
#'     \code{sfalcmcross} object.}
#'
#' @details
#' \subsection{Standard stochastic frontier (\code{groupType = "sfacross"})}{
#'   The stochastic frontier model is defined as:
#'   \deqn{y_i = \alpha + \mathbf{x}_i'\bm{\beta} + v_i - Su_i}
#'   where \eqn{y} is the output (cost, revenue, or profit), \eqn{\mathbf{x}}
#'   is the vector of frontier regressors, \eqn{u_i \ge 0} is the one-sided
#'   inefficiency term with variance \eqn{\sigma^2_u}, and \eqn{v_i} is the
#'   symmetric noise term with variance \eqn{\sigma^2_v}.
#'
#'   Estimation is by ML for all distributions except \code{"gamma"},
#'   \code{"lognormal"}, and \code{"weibull"}, for which MSL is used with
#'   Halton, Generalised-Halton, Sobol, or uniform draws. Antithetic draws are
#'   available for the uniform case.
#'
#'   To account for heteroscedasticity, the variances are modelled as
#'   \eqn{\sigma^2_u = \exp(\bm{\delta}'\mathbf{Z}_u)} and
#'   \eqn{\sigma^2_v = \exp(\bm{\phi}'\mathbf{Z}_v)}. For the truncated normal
#'   distribution, heterogeneity in the pre-truncation mean is modelled as
#'   \eqn{\mu_i = \bm{\omega}'\mathbf{Z}_{\mu}}. The scaling property (Wang and
#'   Schmidt, 2002) can also be imposed for the truncated normal.
#' }
#'
#' \subsection{Sample selection frontier (\code{groupType = "sfaselectioncross"})}{
#'   This model extends the Heckman (1979) selection framework to the
#'   stochastic frontier setting (Greene, 2010; Dakpo \emph{et al.}, 2021).
#'   The selection and frontier equations are:
#'   \deqn{y_{1i}^* = \mathbf{Z}_{si}'\bm{\gamma} + w_i, \quad
#'     w_i \sim \mathcal{N}(0,1)}
#'   \deqn{y_{2i}^* = \mathbf{x}_i'\bm{\beta} + v_i - Su_i}
#'   where \eqn{y_{1i} = \mathbf{1}(y_{1i}^* > 0)} is the binary selection
#'   indicator and \eqn{y_{2i} = y_{2i}^*} is observed only when
#'   \eqn{y_{1i} = 1}. Selection bias arises from
#'   \eqn{\rho = \mathrm{Corr}(w_i, v_i) \ne 0}. Only selected observations
#'   enter the frontier and metafrontier estimation; efficiency estimates for
#'   non-selected observations are \code{NA}.
#' }
#'
#' \subsection{Latent class frontier (\code{groupType = "sfalcmcross"})}{
#'   The latent class model (Orea and Kumbhakar, 2004) estimates a finite
#'   mixture of \eqn{J} frontier models:
#'   \deqn{y_i = \alpha_j + \mathbf{x}_i'\bm{\beta}_j + v_{i|j} - Su_{i|j}}
#'   The prior class probability follows a logit specification:
#'   \deqn{\pi(i,j) = \frac{\exp(\bm{\theta}_j'\mathbf{Z}_{hi})}
#'     {\sum_{m=1}^{J}\exp(\bm{\theta}_m'\mathbf{Z}_{hi})}}
#'   Class assignment is based on the maximum posterior probability computed via
#'   Bayes' rule. When \code{group} is omitted, a single pooled model is
#'   estimated and class assignments serve as technology groups.
#' }
#'
#' \subsection{Metafrontier estimation}{
#'   The global metafrontier \eqn{f(x_i, \bm{\beta}^*)} envelopes all
#'   group frontiers. With LP (Battese \emph{et al.}, 2004),
#'   \eqn{\bm{\beta}^*} minimises
#'   \eqn{\sum_i |\ln \hat{f}(x_i, \bm{\beta}^*) - \ln \hat{f}(x_i,
#'   \hat{\bm{\beta}}_{(g)})|}
#'   subject to \eqn{\ln \hat{f}(x_i, \bm{\beta}^*) \ge \ln \hat{f}(x_i,
#'   \hat{\bm{\beta}}_{(g)})}. QP minimises the squared analogue. The
#'   stochastic approaches (Huang \emph{et al.}, 2014; O'Donnell \emph{et al.},
#'   2008) treat the technology gap \eqn{U_i} as a one-sided error in a
#'   second-stage SFA. Group and metafrontier efficiencies are:
#'   \deqn{TE_i^g = \exp(-u_i^g), \quad
#'     MTR_i = \exp(-U_i), \quad
#'     TE_i^* = TE_i^g \times MTR_i}
#'   Both Jondrow \emph{et al.} (1982) and Battese and Coelli (1988) estimators
#'   are provided for each measure. See \code{\link{efficiencies}} for details.
#' }
#'
#' @references
#' Aigner, D. J., Lovell, C. A. K., and Schmidt, P. 1977. Formulation and
#' estimation of stochastic frontier production function models.
#' \emph{Journal of Econometrics}, \bold{6}(1), 21--37.
#' \doi{10.1016/0304-4076(77)90052-5}
#'
#' Battese, G. E., and Coelli, T. J. 1988. Prediction of firm-level technical
#' efficiencies with a generalized frontier production function and panel data.
#' \emph{Journal of Econometrics}, \bold{38}(3), 387--399.
#' \doi{10.1016/0304-4076(88)90053-X}
#'
#' Battese, G. E., Rao, D. S. P., and O'Donnell, C. J. 2004. A metafrontier
#' production function for estimation of technical efficiencies and technology
#' gaps for firms operating under different technologies.
#' \emph{Journal of Productivity Analysis}, \bold{21}(1), 91--103.
#' \doi{10.1023/B:PROD.0000012454.06094.29}
#'
#' Greene, W. 2003. Simulated likelihood estimation of the normal-gamma
#' stochastic frontier function. \emph{Journal of Productivity Analysis},
#' \bold{19}(2-3), 179--190.
#' \doi{10.1023/A:1022853416499}
#'
#' Greene, W. 2010. A stochastic frontier model with correction for sample
#' selection. \emph{Journal of Productivity Analysis}, \bold{34}(1), 15--24.
#' \doi{10.1007/s11123-009-0159-1}
#'
#' Hajargasht, G. 2015. Stochastic frontiers with a Rayleigh distribution.
#' \emph{Journal of Productivity Analysis}, \bold{44}(2), 199--208.
#' \doi{10.1007/s11123-014-0417-8}
#'
#' Heckman, J. J. 1979. Sample selection bias as a specification error.
#' \emph{Econometrica}, \bold{47}(1), 153--161.
#' \doi{10.2307/1912352}
#'
#' Huang, C. J., Huang, T.-H., and Liu, N.-H. 2014. A new approach to
#' estimating the metafrontier production function based on a stochastic
#' frontier framework. \emph{Journal of Productivity Analysis},
#' \bold{42}(3), 241--254. \doi{10.1007/s11123-014-0402-2}
#'
#' Jondrow, J., Lovell, C. A. K., Materov, I. S., and Schmidt, P. 1982. On
#' the estimation of technical inefficiency in the stochastic frontier
#' production function model. \emph{Journal of Econometrics},
#' \bold{19}(2-3), 233--238.
#' \doi{10.1016/0304-4076(82)90004-5}
#'
#' Li, Q. 1996. Estimating a stochastic production frontier when the adjusted
#' error is symmetric. \emph{Economics Letters}, \bold{52}(3), 221--228.
#' \doi{10.1016/S0165-1765(96)00857-9}
#'
#' Meeusen, W., and van den Broeck, J. 1977. Efficiency estimation from
#' Cobb-Douglas production functions with composed error.
#' \emph{International Economic Review}, \bold{18}(2), 435--444.
#' \doi{10.2307/2525757}
#'
#' Migon, H. S., and Medici, E. 2001. Bayesian inference for generalised
#' exponential models. Working paper, Universidade Federal do Rio de Janeiro.
#'
#' Nguyen, N. B. 2010. Estimation of technical efficiency in stochastic
#' frontier analysis. PhD thesis, Bowling Green State University.
#'
#' O'Donnell, C. J., Rao, D. S. P., and Battese, G. E. 2008. Metafrontier
#' frameworks for the study of firm-level efficiencies and technology ratios.
#' \emph{Empirical Economics}, \bold{34}(2), 231--255.
#' \doi{10.1007/s00181-007-0119-4}
#'
#' Orea, L., and Kumbhakar, S. C. 2004. Efficiency measurement using a latent
#' class stochastic frontier model. \emph{Empirical Economics}, \bold{29}(1),
#' 169--183. \doi{10.1007/s00181-003-0184-2}
#'
#' Dakpo, K. H., Jeanneaux, P., and Latruffe, L. 2016. Modelling
#' pollution-generating technologies in performance benchmarking: Recent
#' developments, limits and future prospects in the nonparametric framework.
#' \emph{European Journal of Operational Research}, \bold{250}(2), 347--359.
#' \doi{10.1016/j.ejor.2015.07.024}
#'
#' Papadopoulos, A. 2015. The half-normal specification for the two-tier
#' stochastic frontier model. \emph{Journal of Productivity Analysis},
#' \bold{43}(2), 225--230. \doi{10.1007/s11123-014-0389-8}
#'
#' Stevenson, R. E. 1980. Likelihood functions for generalised stochastic
#' frontier estimation. \emph{Journal of Econometrics}, \bold{13}(1), 57--66.
#' \doi{10.1016/0304-4076(80)90042-1}
#'
#' Dakpo, K. H., Latruffe, L., Desjeux, Y., and Jeanneaux, P. 2021. Latent
#' class modelling for a robust assessment of productivity: Application to
#' French grazing livestock farms. \emph{Journal of Agricultural Economics},
#' \bold{72}(3), 760--781. \doi{10.1111/1477-9552.12422}
#'
#' Dakpo, K. H., Latruffe, L., Desjeux, Y., and Jeanneaux, P. 2022. Modeling
#' heterogeneous technologies in the presence of sample selection: The case of
#' dairy farms and the adoption of agri-environmental schemes in France.
#' \emph{Agricultural Economics}, \bold{53}(3), 422--438.
#' \doi{10.1111/agec.12683}
#'
#' Tsionas, E. G. 2007. Efficiency measurement with the Weibull stochastic
#' frontier. \emph{Oxford Bulletin of Economics and Statistics},
#' \bold{69}(5), 693--706. \doi{10.1111/j.1468-0084.2007.00475.x}
#'
#' Wang, H.-J. 2012. Stochastic frontier models. In \emph{A Companion to
#' Theoretical Econometrics}, ed. B. H. Baltagi, Blackwell, Oxford.
#'
#' Wang, H.-J., and Schmidt, P. 2002. One-step and two-step estimation of the
#' effects of exogenous variables on technical efficiency levels.
#' \emph{Journal of Productivity Analysis}, \bold{18}(2), 129--144.
#' \doi{10.1023/A:1016565719882}
#'
#' Dakpo, K. H., Desjeux, Y., and Latruffe, L. 2023. sfaR: Stochastic Frontier
#' Analysis using R. R package version 1.0.1.
#' \url{https://CRAN.R-project.org/package=sfaR}
#'
#' @seealso \code{\link[sfaR]{sfacross}}, \code{\link[sfaR]{sfaselectioncross}},
#'   \code{\link[sfaR]{sfalcmcross}}, \code{\link{efficiencies}},
#'   \code{\link{summary.smfa}}, \code{\link[sfaR]{ic}}
#'
#' @keywords models optimize metafrontier
#'
#' @examples
#' ###########################################################################
#' ## -------- SECTION 1: Standard SFA Group Frontier ----------------------##
#' ## Using the rice production dataset (ricephil) from Battese et al.      ##
#' ## Groups are formed based on farm area terciles (small/medium/large).   ##
#' ###########################################################################
#'
#' data("ricephil", package = "sfaR")
#' ricephil$group <- cut(ricephil$AREA,
#'   breaks = quantile(ricephil$AREA, probs = c(0, 1 / 3, 2 / 3, 1), na.rm = TRUE),
#'   labels = c("small", "medium", "large"),
#'   include.lowest = TRUE
#' )
#'
#' ## 1a. sfacross groups + LP metafrontier
#' ##     Deterministic envelope via linear programming (Battese et al., 2004).
#' meta_sfacross_lp <- smfa(
#'   formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
#'   data       = ricephil,
#'   group      = "group",
#'   S          = 1,
#'   udist      = "hnormal",
#'   groupType  = "sfacross",
#'   metaMethod = "lp"
#' )
#' summary(meta_sfacross_lp)
#' # Retrieve individual efficiency and metatechnology ratio estimates:
#' ef_lp <- efficiencies(meta_sfacross_lp)
#' head(ef_lp)
#'
#' ## 1b. sfacross groups + QP metafrontier
#' ##     Deterministic envelope via quadratic programming.
#' meta_sfacross_qp <- smfa(
#'   formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
#'   data       = ricephil,
#'   group      = "group",
#'   S          = 1,
#'   udist      = "hnormal",
#'   groupType  = "sfacross",
#'   metaMethod = "qp"
#' )
#' summary(meta_sfacross_qp)
#'
#' \donttest{
#' ## 1c. sfacross groups + Two-stage SFA metafrontier (Huang et al., 2014)
#' ##     The group-specific fitted frontier values serve as the dependent
#' ##     variable in the second-stage SFA, yielding a stochastic technology gap.
#' meta_sfacross_huang <- smfa(
#'   formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
#'   data        = ricephil,
#'   group       = "group",
#'   S           = 1,
#'   udist       = "hnormal",
#'   groupType   = "sfacross",
#'   metaMethod  = "sfa",
#'   sfaApproach = "huang"
#' )
#' summary(meta_sfacross_huang)
#' ef_huang <- efficiencies(meta_sfacross_huang)
#' head(ef_huang)
#'
#' ## 1d. sfacross groups + O'Donnell et al. (2008) stochastic metafrontier
#' ##     The LP deterministic envelope is used as the second-stage dependent
#' ##     variable: the metafrontier is estimated stochastically around the
#' ##     envelope.
#' meta_sfacross_odonnell <- smfa(
#'   formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
#'   data        = ricephil,
#'   group       = "group",
#'   S           = 1,
#'   udist       = "hnormal",
#'   groupType   = "sfacross",
#'   metaMethod  = "sfa",
#'   sfaApproach = "ordonnell"
#' )
#' summary(meta_sfacross_odonnell)
#' }
#'
#' ###########################################################################
#' ## -------- SECTION 2: Latent Class (LCM) Group Frontier ---------------##
#' ## No observed group variable: a pooled sfalcmcross model assigns       ##
#' ## observations to 2 latent technology classes; these classes become the ##
#' ## technology groups for the metafrontier.                               ##
#' ###########################################################################
#'
#' data("utility", package = "sfaR")
#'
#' ## 2a. sfalcmcross (pooled, 2 classes) + LP metafrontier
#' meta_lcm_lp <- smfa(
#'   formula    = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
#'   data       = utility,
#'   S          = -1,
#'   groupType  = "sfalcmcross",
#'   lcmClasses = 2,
#'   metaMethod = "lp"
#' )
#' summary(meta_lcm_lp)
#' ef_lcm_lp <- efficiencies(meta_lcm_lp)
#' head(ef_lcm_lp)
#'
#' \donttest{
#' ## 2b. sfalcmcross (pooled, 2 classes) + QP metafrontier
#' meta_lcm_qp <- smfa(
#'   formula    = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
#'   data       = utility,
#'   S          = -1,
#'   groupType  = "sfalcmcross",
#'   lcmClasses = 2,
#'   metaMethod = "qp"
#' )
#' summary(meta_lcm_qp)
#'
#' ## 2c. sfalcmcross (pooled, 2 classes) + Two-stage SFA metafrontier
#' ##     (Huang et al., 2014)
#' meta_lcm_huang <- smfa(
#'   formula     = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
#'   data        = utility,
#'   S           = -1,
#'   groupType   = "sfalcmcross",
#'   lcmClasses  = 2,
#'   metaMethod  = "sfa",
#'   sfaApproach = "huang"
#' )
#' summary(meta_lcm_huang)
#' ef_lcm_huang <- efficiencies(meta_lcm_huang)
#' head(ef_lcm_huang)
#'
#' ## 2d. sfalcmcross (pooled, 2 classes) + O'Donnell et al. (2008)
#' meta_lcm_odonnell <- smfa(
#'   formula     = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
#'   data        = utility,
#'   S           = -1,
#'   groupType   = "sfalcmcross",
#'   lcmClasses  = 2,
#'   metaMethod  = "sfa",
#'   sfaApproach = "ordonnell"
#' )
#' summary(meta_lcm_odonnell)
#' }
#'
#' ###########################################################################
#' ## -------- SECTION 3: Sample Selection SFA Group Frontier -------------##
#' ###########################################################################
#'
#' ## 3a. Small toy example for automatic testing (< 5s)
#' N <- 100
#' set.seed(12345)
#' z1 <- rnorm(N); v1 <- rnorm(N); g <- rnorm(N)
#' ds <- z1 + v1; d <- ifelse(ds > 0, 1, 0)
#' group <- ifelse(g > 0, 1, 0)
#' x1 <- rnorm(N); y <- x1 + rnorm(N) - abs(rnorm(N))
#' dat <- data.frame(y = y, x1 = x1, z1 = z1, d = d, group = group)
#'
#' meta_toy <- smfa(
#'   formula    = y ~ x1,
#'   selectionF = d ~ z1,
#'   data       = dat,
#'   group      = "group",
#'   groupType  = "sfaselectioncross",
#'   lType      = "ghermite",
#'   Nsub       = 10,
#'   itermax    = 100,
#'   metaMethod = "lp"
#' )
#' summary(meta_toy)
#'
#' \donttest{
#' ## 3b. More complex selection models
#' ## Simulated dataset with a Heckman selection mechanism.
#'
#' N <- 2000
#' set.seed(12345)
#' z1 <- rnorm(N); z2 <- rnorm(N); v1 <- rnorm(N); v2 <- rnorm(N); g <- rnorm(N)
#' e1 <- v1; e2 <- 0.7071 * (v1 + v2)
#' ds <- z1 + z2 + e1; d <- ifelse(ds > 0, 1, 0)
#' group <- ifelse(g > 0, 1, 0)
#' u <- abs(rnorm(N)); x1 <- rnorm(N); x2 <- rnorm(N)
#' y <- x1 + x2 + e2 - u
#' dat <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1, z2 = z2, d = d, group = group)
#'
#' meta_sel_lp <- smfa(
#'   formula    = y ~ x1 + x2,
#'   selectionF = d ~ z1 + z2,
#'   data       = dat,
#'   group      = "group",
#'   S          = 1L,
#'   udist      = "hnormal",
#'   groupType  = "sfaselectioncross",
#'   modelType  = "greene10",
#'   lType      = "kronrod",
#'   Nsub       = 100,
#'   metaMethod = "lp"
#' )
#' summary(meta_sel_lp)
#' }


#'
#' @importFrom sfaR sfacross sfalcmcross sfaselectioncross
#' @importFrom stats as.formula lm model.frame model.matrix model.response na.pass pnorm printCoefmat setNames terms update
#' @export
smfa <- function(
  formula,
  muhet,
  uhet,
  vhet,
  thet,
  logDepVar = TRUE,
  data,
  subset,
  weights,
  wscale = TRUE,
  group = NULL,
  S = 1L,
  udist = "hnormal",
  start = NULL,
  scaling = FALSE,
  modelType = "greene10",
  groupType = "sfacross",
  metaMethod = "lp",
  sfaApproach = "ordonnell",
  selectionF = NULL,
  lcmClasses = 2L,
  whichStart = 2L,
  initAlg = "nm",
  initIter = 100L,
  lType = "ghermite",
  Nsub = 100L,
  uBound = Inf,
  intol = 1e-6,
  method = "bfgs",
  hessianType = NULL,
  simType = "halton",
  Nsim = 100L,
  prime = 2L,
  burn = 10L,
  antithetics = FALSE,
  seed = 12345L,
  itermax = 2000L,
  printInfo = FALSE,
  tol = 1e-12,
  gradtol = 1e-06,
  stepmax = 0.1,
  qac = "marquardt",
  ...
) {
  cl <- match.call()

  # ---------- Reject dollar-sign formula notation ----------
  # Using df$var inside a formula (e.g. log(ricephil$PROD) ~ log(ricephil$AREA))
  # bypasses the 'data=' argument when data is subset per group, causing
  # model.frame to use the full dataset and producing length mismatches.
  .check_no_dollar <- function(fml, nm) {
    if (!missing(fml) && !is.null(fml)) {
      txt <- deparse(fml)
      if (any(grepl("$", txt, fixed = TRUE))) {
        stop(
          "Do not use '$' notation in '",
          nm,
          "'. ",
          "Use bare variable names with 'data = <your data frame>' instead.\n",
          "  Bad : log(ricephil$PROD) ~ log(ricephil$AREA)\n",
          "  Good: log(PROD) ~ log(AREA)  with  data = ricephil",
          call. = FALSE
        )
      }
    }
  }
  .check_no_dollar(formula, "formula")
  if (!missing(muhet)) {
    .check_no_dollar(muhet, "muhet")
  }
  if (!missing(uhet)) {
    .check_no_dollar(uhet, "uhet")
  }
  if (!missing(vhet)) {
    .check_no_dollar(vhet, "vhet")
  }
  if (!missing(thet)) {
    .check_no_dollar(thet, "thet")
  }
  if (!is.null(selectionF) && inherits(selectionF, "formula")) {
    .check_no_dollar(selectionF, "selectionF")
  }

  # ---------- Validate groupType ----------
  groupType <- tolower(groupType)
  if (!(groupType %in% c("sfacross", "sfaselectioncross", "sfalcmcross"))) {
    stop(
      "argument 'groupType' must be 'sfacross', 'sfaselectioncross', or 'sfalcmcross'",
      call. = FALSE
    )
  }

  # ---------- Validate metaMethod ----------
  metaMethod <- tolower(metaMethod)
  if (!(metaMethod %in% c("lp", "qp", "sfa"))) {
    stop("argument 'metaMethod' must be 'lp', 'qp', or 'sfa'", call. = FALSE)
  }

  # ---------- Validate sfaApproach ----------
  sfaApproach <- tolower(sfaApproach)
  if (!(sfaApproach %in% c("ordonnell", "huang"))) {
    stop("argument 'sfaApproach' must be 'ordonnell' or 'huang'", call. = FALSE)
  }

  # ---------- Validate / normalise group ----------
  # For sfalcmcross the group variable is OPTIONAL: if omitted, a pooled
  # sfalcmcross is fitted on all data and latent classes serve as groups.
  lcmNoGroup <- (groupType == "sfalcmcross" && is.null(group))

  if (!lcmNoGroup) {
    if (is.null(group) || !is.character(group) || length(group) != 1) {
      stop(
        "argument 'group' must be a single character string naming a column in 'data'",
        call. = FALSE
      )
    }
    if (!group %in% names(data)) {
      stop("'group' variable '", group, "' not found in 'data'", call. = FALSE)
    }
  }

  # Apply subset if provided
  if (!missing(subset)) {
    data <- data[subset, , drop = FALSE]
  }

  # ---- LCM NO-GROUP early path ----
  # Fit a single pooled sfalcmcross; use posterior class assignments as groups.
  if (lcmNoGroup) {
    return(.smfa_lcm_nogroup(
      formula = formula,
      data = data,
      S = S,
      udist = udist,
      start = start,
      lcmClasses = lcmClasses,
      whichStart = whichStart,
      initAlg = initAlg,
      initIter = initIter,
      metaMethod = metaMethod,
      sfaApproach = sfaApproach,
      method = method,
      hessianType = hessianType,
      itermax = itermax,
      printInfo = printInfo,
      tol = tol,
      gradtol = gradtol,
      stepmax = stepmax,
      qac = qac,
      logDepVar = logDepVar,
      wscale = wscale,
      muhet_missing = missing(muhet),
      uhet_missing = missing(uhet),
      vhet_missing = missing(vhet),
      thet_missing = missing(thet),
      uhet = if (!missing(uhet)) uhet else NULL,
      vhet = if (!missing(vhet)) vhet else NULL,
      thet = if (!missing(thet)) thet else NULL,
      weights = if (!missing(weights)) weights else NULL,
      cl = cl,
      ...
    ))
  }

  # Ensure group is a factor (normal grouped path)
  data[[group]] <- as.factor(data[[group]])
  groupLevels <- levels(data[[group]])
  nGroups <- length(groupLevels)
  if (nGroups < 2) {
    stop(
      "at least 2 groups are required for metafrontier analysis",
      call. = FALSE
    )
  }

  # ---------- Validate S ----------
  if (length(S) != 1 || !(S %in% c(-1L, 1L))) {
    stop("argument 'S' must equal either 1 or -1", call. = FALSE)
  }
  typeSfa <- if (S == 1L) {
    "Stochastic Production/Profit Frontier, e = v - u"
  } else {
    "Stochastic Cost Frontier, e = v + u"
  }

  # ---------- selectionF validation for sfaselectioncross ----------
  if (groupType == "sfaselectioncross") {
    if (is.null(selectionF)) {
      stop(
        "'selectionF' must be provided when groupType = 'sfaselectioncross'",
        call. = FALSE
      )
    }
    # selectionF must be a two-sided formula whose LHS is the selection indicator,
    # e.g. selected ~ EDYRS + AGE  - consistent with sfaselectioncross's own API.
    # The LHS variable name is used to extract the selectDum column.
    .validate_selF <- function(fml, gname) {
      if (!inherits(fml, "formula")) {
        stop(
          "'selectionF' for group '",
          gname,
          "' must be a formula",
          call. = FALSE
        )
      }
      lhs_vars <- all.vars(update(fml, . ~ 0))
      if (length(lhs_vars) < 1L) {
        stop(
          "'selectionF' must be a two-sided formula with the selection indicator ",
          "as the LHS, e.g. selected ~ EDYRS + AGE",
          call. = FALSE
        )
      }
    }
    if (inherits(selectionF, "formula")) {
      .validate_selF(selectionF, "all groups")
      selecFList <- setNames(rep(list(selectionF), nGroups), groupLevels)
    } else if (is.list(selectionF)) {
      missing_g <- setdiff(groupLevels, names(selectionF))
      if (length(missing_g) > 0) {
        stop(
          "'selectionF' list is missing entries for groups: ",
          paste(missing_g, collapse = ", "),
          call. = FALSE
        )
      }
      selecFList <- selectionF[groupLevels]
      lapply(seq_along(selecFList), function(k) {
        .validate_selF(selecFList[[k]], names(selecFList)[k])
      })
    } else {
      stop(
        "'selectionF' must be a two-sided formula (e.g. selected ~ z1 + z2) ",
        "or a named list of such formulas",
        call. = FALSE
      )
    }
  }

  # ---------- Build sfaR model arguments ----------
  # Common arguments accepted by all three sfaR functions
  sfaBaseArgs <- list(
    logDepVar = logDepVar,
    S = S,
    udist = udist,
    start = start,
    method = method,
    itermax = itermax,
    printInfo = printInfo,
    tol = tol,
    gradtol = gradtol,
    stepmax = stepmax,
    qac = qac
  )
  # hessianType: only include if the user explicitly passed it;
  # otherwise each underlying sfaR function uses its own natural default:
  #   sfacross / sfalcmcross -> hessianType = 1L (analytic Hessian)
  #   sfaselectioncross      -> hessianType = 2L (BHHH / outer-product)
  # Forcing a single default here causes SE differences vs. standalone sfaR calls.
  if (!is.null(hessianType)) {
    sfaBaseArgs$hessianType <- hessianType
  }
  # Add weights only if provided (all three accept it as optional)
  if (!missing(weights)) sfaBaseArgs$weights <- weights
  sfaBaseArgs$wscale <- wscale

  if (groupType == "sfacross") {
    # sfacross-specific: formula, scaling, muhet, uhet, vhet, simType/Nsim/prime/burn/antithetics/seed
    sfaBaseArgs$formula <- formula
    sfaBaseArgs$scaling <- scaling
    sfaBaseArgs$simType <- simType
    sfaBaseArgs$Nsim <- Nsim
    sfaBaseArgs$prime <- prime
    sfaBaseArgs$burn <- burn
    sfaBaseArgs$antithetics <- antithetics
    sfaBaseArgs$seed <- seed
    if (!missing(muhet)) sfaBaseArgs$muhet <- muhet
    if (!missing(uhet)) sfaBaseArgs$uhet <- uhet
    if (!missing(vhet)) sfaBaseArgs$vhet <- vhet
  } else if (groupType == "sfalcmcross") {
    # sfalcmcross-specific: formula, lcmClasses, whichStart, initAlg, initIter, uhet, vhet, thet
    # sfalcmcross does NOT accept: seed, simType, Nsim, prime, burn, antithetics, scaling, muhet
    sfaBaseArgs$formula <- formula
    sfaBaseArgs$lcmClasses <- lcmClasses
    sfaBaseArgs$whichStart <- whichStart
    sfaBaseArgs$initAlg <- initAlg
    sfaBaseArgs$initIter <- initIter
    if (!missing(uhet)) sfaBaseArgs$uhet <- uhet
    if (!missing(vhet)) sfaBaseArgs$vhet <- vhet
    if (!missing(thet)) sfaBaseArgs$thet <- thet
  } else if (groupType == "sfaselectioncross") {
    # sfaselectioncross-specific: uses frontierF (NOT formula), selectionF, modelType,
    # lType, Nsub, uBound, intol, simType, Nsim, prime, burn, antithetics, seed
    # Does NOT accept: formula, scaling, muhet, thet, whichStart, initAlg, initIter, lcmClasses
    sfaBaseArgs$frontierF <- formula
    sfaBaseArgs$modelType <- modelType
    sfaBaseArgs$lType <- lType
    sfaBaseArgs$Nsub <- Nsub
    sfaBaseArgs$uBound <- uBound
    sfaBaseArgs$intol <- intol
    sfaBaseArgs$simType <- simType
    sfaBaseArgs$Nsim <- Nsim
    sfaBaseArgs$prime <- prime
    sfaBaseArgs$burn <- burn
    sfaBaseArgs$antithetics <- antithetics
    sfaBaseArgs$seed <- seed
    if (!missing(uhet)) sfaBaseArgs$uhet <- uhet
    if (!missing(vhet)) sfaBaseArgs$vhet <- vhet
  }

  # ---------- Step 1: Fit group-specific frontier models ----------
  if (printInfo) {
    cat(
      "Estimating group-specific stochastic frontiers (",
      groupType,
      ") ...\n",
      sep = ""
    )
  }
  groupModels <- vector("list", nGroups)
  names(groupModels) <- groupLevels

  for (g in groupLevels) {
    if (printInfo) cat("  Group:", g, "\n")
    subData <- data[data[[group]] == g, , drop = FALSE]
    if (nrow(subData) == 0) {
      stop("Group '", g, "' has zero observations", call. = FALSE)
    }
    grpArgs <- c(sfaBaseArgs, list(data = subData))

    if (groupType == "sfaselectioncross") {
      grpArgs$selectionF <- selecFList[[g]]
    }

    estimFun <- switch(groupType,
      sfacross = sfacross,
      sfaselectioncross = sfaselectioncross,
      sfalcmcross = sfalcmcross
    )

    groupModels[[g]] <- tryCatch(
      do.call(estimFun, grpArgs),
      error = function(e) {
        stop(
          "Estimation failed for group '",
          g,
          "': ",
          e$message,
          call. = FALSE
        )
      }
    )
  }
  if (printInfo) cat("Group frontiers estimated.\n")

  # ---------- Collect full dataset and fitted values ----------
  dataFull <- data
  dataFull$.mf_rowid <- seq_len(nrow(dataFull))
  dataFull$.mf_group <- dataFull[[group]]
  dataFull$.mf_yhat_group <- NA_real_
  dataFull$.mf_logL_OBS <- NA_real_

  # Selection mask (only relevant for sfaselectioncross)
  if (groupType == "sfaselectioncross") {
    dataFull$.mf_selected <- NA_integer_
  }

  for (g in groupLevels) {
    idx <- which(dataFull[[group]] == g)
    dt_g <- groupModels[[g]]$dataTable

    if (groupType == "sfaselectioncross") {
      # Use the LHS of selectionF to identify the selection indicator - exactly
      # as chalfnormeff_ss does via all.vars(object$selectionF)[1]
      selColName <- all.vars(selecFList[[g]])[1]
      selDum <- dt_g[[selColName]]
      if (is.null(selDum)) {
        stop(
          "Selection indicator column '",
          selColName,
          "' not found in ",
          "dataTable for group '",
          g,
          "'",
          call. = FALSE
        )
      }
      dataFull$.mf_selected[idx] <- as.integer(selDum)
      sel_idx <- idx[selDum == 1]
      dataFull$.mf_yhat_group[sel_idx] <- dt_g$mlFitted[selDum == 1]
      dataFull$.mf_logL_OBS[sel_idx] <- dt_g$logL_OBS[selDum == 1]
    } else if (groupType == "sfalcmcross") {
      best_fit <- extract_lcm_fitted(groupModels[[g]])
      dataFull$.mf_yhat_group[idx] <- best_fit
      dataFull$.mf_logL_OBS[idx] <- dt_g$logL_OBS
    } else {
      # sfacross
      dataFull$.mf_yhat_group[idx] <- dt_g$mlFitted
      dataFull$.mf_logL_OBS[idx] <- dt_g$logL_OBS
    }
  }

  # ---------- Build Xvar matrix for meta-stage ----------
  # Use formula from one of the group models
  if (groupType == "sfaselectioncross") {
    firstModel <- groupModels[[1]]
    formulaFull <- firstModel$frontierF
    # Only use selected obs for metafrontier
    metaIdx <- which(!is.na(dataFull$.mf_yhat_group))
    dataMeta <- dataFull[metaIdx, , drop = FALSE]
    mcFull <- model.frame(formulaFull, data = dataMeta, na.action = na.pass)
    validMeta <- rowSums(is.na(mcFull)) == 0
    Xvar_full <- model.matrix(terms(formulaFull, rhs = 1), mcFull)[
      validMeta, ,
      drop = FALSE
    ]
  } else {
    firstModel <- groupModels[[1]]
    formulaFull <- firstModel$formula
    mcFull <- model.frame(formulaFull, data = dataFull, na.action = na.pass)
    validFull <- rowSums(
      is.na(mcFull) | Reduce("|", lapply(mcFull, is.infinite))
    ) ==
      0
    Xvar_full <- model.matrix(terms(formulaFull, rhs = 1), mcFull)[
      validFull, ,
      drop = FALSE
    ]
    metaIdx <- seq_len(nrow(dataFull))[validFull]
  }

  nXvar <- ncol(Xvar_full)

  # ---------- Build N x G matrix of group betas evaluated at all obs ----------
  # (needed for LP, QP, and ordonnell SFA; not needed for Huang)
  if (
    metaMethod %in%
      c("lp", "qp") ||
      (metaMethod == "sfa" && sfaApproach == "ordonnell")
  ) {
    groupFrontierAll <- matrix(NA_real_, nrow = length(metaIdx), ncol = nGroups)
    colnames(groupFrontierAll) <- groupLevels

    for (g in groupLevels) {
      if (groupType == "sfalcmcross") {
        # For LCM: for obs in group g, use their best-posterior-class fitted
        # value directly from dataTable (already computed by sfalcmcross).
        # For obs NOT in group g, evaluate group g's class-1 beta at their X.
        lcm_nXvar <- groupModels[[g]]$nXvar
        lcm_nuZU <- groupModels[[g]]$nuZUvar
        lcm_nvZV <- groupModels[[g]]$nvZVvar
        stride <- lcm_nXvar + lcm_nuZU + lcm_nvZV
        dt_g <- groupModels[[g]]$dataTable

        # Positions of obs in group g within metaIdx
        grp_pos_in_meta <- which(dataFull[[group]][metaIdx] == g)
        non_pos_in_meta <- which(dataFull[[group]][metaIdx] != g)

        # Own-group obs: use best-class fitted value from dataTable
        # extract_lcm_fitted() returns N_g-length vector ordered as dt_g rows
        if (length(grp_pos_in_meta) > 0) {
          best_fit_g <- extract_lcm_fitted(groupModels[[g]])
          groupFrontierAll[grp_pos_in_meta, g] <- best_fit_g
        }

        # Non-own-group obs: evaluate class-1 beta from group g at their X
        if (length(non_pos_in_meta) > 0) {
          beta1_g <- groupModels[[g]]$mlParam[seq_len(lcm_nXvar)]
          groupFrontierAll[non_pos_in_meta, g] <-
            as.numeric(Xvar_full[non_pos_in_meta, , drop = FALSE] %*% beta1_g)
        }
      } else if (groupType == "sfaselectioncross") {
        # frontier beta: mlParam[1:nXvar_F] as in sfaselectioncross.R
        betaG <- groupModels[[g]]$mlParam[seq_len(groupModels[[g]]$nXvar)]
        groupFrontierAll[, g] <- as.numeric(Xvar_full %*% betaG)
      } else {
        # sfacross: mlParam[1:nXvar]
        betaG <- groupModels[[g]]$mlParam[seq_len(nXvar)]
        groupFrontierAll[, g] <- as.numeric(Xvar_full %*% betaG)
      }
    }
  }

  # ---------- Step 2: Estimate metafrontier ----------
  if (printInfo) {
    cat(
      "Estimating metafrontier using method:",
      mfauxdist(metaMethod, sfaApproach),
      "\n"
    )
  }

  metaFrontierParam <- NULL
  metaFrontierVcov <- NULL
  metaSfaObj <- NULL
  yhat_meta_local <- NULL # meta fitted values on metaIdx rows

  if (metaMethod == "lp") {
    yhat_meta_local <- mf_lp(groupFrontierAll)
    metaFrontierParam <- NULL
    metaFrontierVcov <- NULL
  } else if (metaMethod == "qp") {
    qpRes <- mf_qp(Xvar = Xvar_full, groupFrontierMat = groupFrontierAll)
    metaFrontierParam <- qpRes$beta
    names(metaFrontierParam) <- colnames(Xvar_full)
    metaFrontierVcov <- qpRes$vcov
    yhat_meta_local <- qpRes$yhat
  } else {
    # sfa
    if (sfaApproach == "huang") {
      yhat_group_meta <- dataFull$.mf_yhat_group[metaIdx]
      sfaMetaRes <- mf_huang(
        Xvar = Xvar_full,
        yhat_group = yhat_group_meta,
        S = S,
        method = method,
        udist = udist,
        itermax = itermax,
        tol = tol,
        gradtol = gradtol,
        stepmax = stepmax,
        qac = qac,
        ...
      )
    } else {
      sfaMetaRes <- mf_sfa_ordonnell(
        Xvar = Xvar_full,
        groupFrontierMat = groupFrontierAll,
        S = S,
        method = method,
        udist = udist,
        itermax = itermax,
        tol = tol,
        gradtol = gradtol,
        stepmax = stepmax,
        qac = qac,
        ...
      )
    }
    metaFrontierParam <- sfaMetaRes$beta
    names(metaFrontierParam) <- colnames(Xvar_full)
    metaFrontierVcov <- sfaMetaRes$vcov
    metaSfaObj <- sfaMetaRes$sfaObj
    yhat_meta_local <- sfaMetaRes$yhat
  }

  # Store meta fitted values in full data table
  dataFull$.mf_yhat_meta <- NA_real_
  dataFull$.mf_yhat_meta[metaIdx] <- yhat_meta_local
  dataFull$.mf_gap <- NA_real_
  dataFull$.mf_gap[metaIdx] <-
    S * (dataFull$.mf_yhat_meta[metaIdx] - dataFull$.mf_yhat_group[metaIdx])

  # ---------- Compute total log-likelihood and nParm ----------
  groupLogLiks <- sapply(groupModels, function(m) m$mlLoglik)
  groupNParms <- sapply(groupModels, function(m) m$nParm)
  totalLogLik <- sum(groupLogLiks)
  totalNParm <- sum(groupNParms)

  if (metaMethod == "sfa" && !is.null(metaSfaObj)) {
    totalLogLik <- totalLogLik + metaSfaObj$mlLoglik
    totalNParm <- totalNParm + metaSfaObj$nParm
  } else if (metaMethod == "qp") {
    totalNParm <- totalNParm + length(metaFrontierParam)
  }

  # ---------- Assemble return object ----------
  mlDate <- format(Sys.time(), "Model was estimated on : %b %a %d, %Y at %H:%M")

  returnObj <- list()
  returnObj$call <- cl
  returnObj$formula <- formula
  returnObj$group <- group
  returnObj$groups <- groupLevels
  returnObj$nGroups <- nGroups
  returnObj$S <- S
  returnObj$typeSfa <- typeSfa
  returnObj$udist <- udist
  returnObj$groupType <- groupType
  returnObj$metaMethod <- metaMethod
  returnObj$sfaApproach <- sfaApproach
  returnObj$lcmNoGroup <- FALSE
  returnObj$logDepVar <- logDepVar
  returnObj$Nobs <- nrow(dataFull)
  returnObj$NobsMeta <- length(metaIdx)
  returnObj$nXvar <- nXvar
  returnObj$groupModels <- groupModels
  if (exists("groupFrontierAll")) {
    returnObj$groupFrontierAll <- groupFrontierAll
  }
  returnObj$metaFrontierParam <- metaFrontierParam
  returnObj$metaFrontierVcov <- metaFrontierVcov
  returnObj$metaSfaObj <- metaSfaObj
  returnObj$mlLoglik <- totalLogLik
  returnObj$nParm <- totalNParm
  returnObj$dataTable <- dataFull
  returnObj$mlDate <- mlDate

  class(returnObj) <- "smfa"
  return(returnObj)
}

# print for smfa ----------
#' @rdname smfa
#' @export
print.smfa <- function(x, ...) {
  cat("Call:\n")
  cat(deparse(x$call), "\n\n")
  cat("Stochastic Metafrontier Analysis\n")
  grp_appr <- switch(x$groupType,
    sfacross = "Stochastic Frontier Analysis",
    sfaselectioncross = "Sample Selection Stochastic Frontier Analysis",
    sfalcmcross = "Latent Class Stochastic Frontier Analysis",
    x$groupType
  )
  cat("Group approach   :", grp_appr, "\n")
  cat("Group estimator  :", x$groupType, "\n")
  if (isTRUE(x$lcmNoGroup)) {
    cat("  (Pooled LCM - latent classes used as groups)\n")
  }
  cat("Metafrontier method:", mfauxdist(x$metaMethod, x$sfaApproach), "\n")
  cat("Groups (", x$nGroups, "):", paste(x$groups, collapse = ", "), "\n")
  cat("Total observations:", x$Nobs)
  if (!is.null(x$NobsMeta) && x$NobsMeta < x$Nobs) {
    cat(" (", x$NobsMeta, "in metafrontier)", sep = "")
  }
  cat("\n")
  cat("Distribution:", x$udist, "\n")
  cat(x$typeSfa, "\n\n")
  if (!is.null(x$metaFrontierParam)) {
    cat("Metafrontier coefficients:\n")
    print.default(format(x$metaFrontierParam), print.gap = 2, quote = FALSE)
  } else {
    cat("Metafrontier: deterministic envelope (LP) - no estimated parameters\n")
  }
  if (isTRUE(x$lcmNoGroup)) {
    cat("\nLog-likelihood (pooled LCM):", round(x$mlLoglik, 4), "\n")
  } else {
    cat("\nGroup log-likelihoods:\n")
    for (g in x$groups) {
      cat(sprintf("  %-20s: %.4f\n", g, x$groupModels[[g]]$mlLoglik))
    }
    cat(sprintf("  %-20s: %.4f\n", "Total", x$mlLoglik))
  }
  invisible(x)
}

# ---------------------------------------------------------------------------
# Internal helper: pooled LCM metafrontier (no explicit group variable)
# ---------------------------------------------------------------------------
.smfa_lcm_nogroup <- function(
  formula,
  data,
  S,
  udist,
  lcmClasses,
  whichStart,
  initAlg,
  initIter,
  metaMethod,
  sfaApproach,
  method,
  hessianType,
  itermax,
  printInfo,
  tol,
  gradtol,
  stepmax,
  qac,
  logDepVar,
  muhet_missing,
  uhet_missing,
  vhet_missing,
  thet_missing,
  uhet = NULL,
  vhet = NULL,
  thet = NULL,
  cl,
  ...
) {
  typeSfa <- if (S == 1L) {
    "Stochastic Production/Profit Frontier, e = v - u"
  } else {
    "Stochastic Cost Frontier, e = v + u"
  }

  # ---- Step 1: Fit single pooled sfalcmcross ----
  if (printInfo) {
    cat(
      "Fitting pooled sfalcmcross (",
      lcmClasses,
      " classes) on all data ...\n",
      sep = ""
    )
  }
  lcmArgs <- list(
    formula    = formula,
    data       = data,
    S          = S,
    udist      = udist,
    logDepVar  = logDepVar,
    lcmClasses = lcmClasses,
    whichStart = whichStart,
    initAlg    = initAlg,
    initIter   = initIter,
    method     = method,
    itermax    = itermax,
    printInfo  = printInfo,
    tol        = tol,
    gradtol    = gradtol,
    stepmax    = stepmax,
    qac        = qac
  )
  # hessianType: only include if the user explicitly passed it;
  # otherwise sfalcmcross uses its own natural default (1 = analytic Hessian).
  if (!is.null(hessianType)) {
    lcmArgs$hessianType <- hessianType
  }
  if (!uhet_missing && !is.null(uhet)) {
    lcmArgs$uhet <- uhet
  }
  if (!vhet_missing && !is.null(vhet)) {
    lcmArgs$vhet <- vhet
  }
  if (!thet_missing && !is.null(thet)) {
    lcmArgs$thet <- thet
  }

  lcmObj <- tryCatch(
    do.call(sfalcmcross, lcmArgs),
    error = function(e) stop("sfalcmcross failed: ", e$message, call. = FALSE)
  )
  if (printInfo) cat("Pooled LCM estimated.\n")

  # ---- Step 2: Extract class assignments and TEs ----
  effLcm <- efficiencies(lcmObj)
  Group_c <- effLcm$Group_c # integer 1..nc per obs
  teGroup_BC <- effLcm$teBC_c # best-class BC efficiency

  nc <- lcmClasses
  groupNames <- paste0("Class_", seq_len(nc))
  nObs <- nrow(data)

  # Build .mf_group column (factor with class labels)
  mf_group <- factor(paste0("Class_", Group_c), levels = groupNames)

  # ---- Step 3: Build Xvar matrix for meta-stage ----
  mcFull <- model.frame(formula, data = data, na.action = na.pass)
  validFull <- rowSums(
    is.na(mcFull) |
      Reduce("|", lapply(mcFull, is.infinite))
  ) ==
    0
  Xvar_full <- model.matrix(terms(formula, rhs = 1), mcFull)[
    validFull, ,
    drop = FALSE
  ]
  metaIdx <- seq_len(nObs)[validFull]
  nXvar <- ncol(Xvar_full)

  # ---- Step 4: Extract class-k betas and build groupFrontierAll ----
  # sfalcmcross mlParam layout: [beta_1 ... beta_nc | Zu_1 ... Zu_nc | Zv_1 ... Zv_nc | theta]
  nX <- lcmObj$nXvar
  nU <- lcmObj$nuZUvar
  nV <- lcmObj$nvZVvar

  # Best-class fitted values (y-hat for each obs using its assigned class frontier)
  # Use mlFitted_ck columns if available, otherwise reconstruct.
  dt_lcm <- lcmObj$dataTable
  yhat_group <- rep(NA_real_, nObs)

  # Try to use mlFitted_ck columns directly
  fit_cols <- paste0("mlFitted_c", seq_len(nc))
  has_fit <- all(fit_cols %in% names(dt_lcm))
  if (has_fit) {
    for (k in seq_len(nc)) {
      idx_k <- which(Group_c == k)
      if (length(idx_k) > 0) {
        yhat_group[idx_k] <- dt_lcm[[fit_cols[k]]][idx_k]
      }
    }
  } else {
    # Reconstruct from class betas
    for (k in seq_len(nc)) {
      # Interleaved layout: [beta_1, Zu_1, Zv_1, beta_2, Zu_2, Zv_2, ...]
      s_idx <- (k - 1) * (nX + nU + nV) + 1
      beta_k <- lcmObj$mlParam[s_idx:(s_idx + nX - 1)]
      yhat_group <- yhat_group # keep NA for invalid
      idx_k <- which(mf_group[validFull] == groupNames[k])
      if (length(idx_k) > 0) {
        yhat_group[metaIdx[idx_k]] <- as.numeric(Xvar_full[idx_k, ] %*% beta_k)
      }
    }
  }

  # groupFrontierAll: N_meta x nc matrix - class k beta evaluated at all obs
  if (
    metaMethod %in%
      c("lp", "qp") ||
      (metaMethod == "sfa" && sfaApproach == "ordonnell")
  ) {
    groupFrontierAll <- matrix(
      NA_real_,
      nrow = length(metaIdx),
      ncol = nc,
      dimnames = list(NULL, groupNames)
    )
    for (k in seq_len(nc)) {
      s_idx <- (k - 1) * (nX + nU + nV) + 1
      beta_k <- lcmObj$mlParam[s_idx:(s_idx + nX - 1)]
      groupFrontierAll[, k] <- as.numeric(Xvar_full %*% beta_k)
    }
  }

  # ---- Step 5: Estimate metafrontier ----
  if (printInfo) {
    cat(
      "Estimating metafrontier using method:",
      mfauxdist(metaMethod, sfaApproach),
      "\n"
    )
  }
  metaFrontierParam <- NULL
  metaFrontierVcov <- NULL
  metaSfaObj <- NULL
  yhat_meta_local <- NULL

  if (metaMethod == "lp") {
    yhat_meta_local <- mf_lp(groupFrontierAll)
  } else if (metaMethod == "qp") {
    qpRes <- mf_qp(Xvar = Xvar_full, groupFrontierMat = groupFrontierAll)
    metaFrontierParam <- setNames(qpRes$beta, colnames(Xvar_full))
    metaFrontierVcov <- qpRes$vcov
    yhat_meta_local <- qpRes$yhat
  } else {
    # sfa
    if (sfaApproach == "huang") {
      sfaMetaRes <- mf_huang(
        Xvar = Xvar_full,
        yhat_group = yhat_group[metaIdx],
        S = S,
        method = method,
        udist = udist,
        itermax = itermax,
        tol = tol,
        gradtol = gradtol,
        stepmax = stepmax,
        qac = qac,
        ...
      )
    } else {
      sfaMetaRes <- mf_sfa_ordonnell(
        Xvar = Xvar_full,
        groupFrontierMat = groupFrontierAll,
        S = S,
        method = method,
        udist = udist,
        itermax = itermax,
        tol = tol,
        gradtol = gradtol,
        stepmax = stepmax,
        qac = qac,
        ...
      )
    }
    metaFrontierParam <- setNames(sfaMetaRes$beta, colnames(Xvar_full))
    metaFrontierVcov <- sfaMetaRes$vcov
    metaSfaObj <- sfaMetaRes$sfaObj
    yhat_meta_local <- sfaMetaRes$yhat
  }

  # ---- Step 6: Assemble dataTable ----
  dataFull <- data
  dataFull$.mf_rowid <- seq_len(nObs)
  dataFull$.mf_group <- mf_group # Class_1 / Class_2 / ...
  dataFull$.mf_yhat_group <- yhat_group
  dataFull$.mf_yhat_meta <- NA_real_
  dataFull$.mf_yhat_meta[metaIdx] <- yhat_meta_local
  dataFull$.mf_gap <- NA_real_
  dataFull$.mf_gap[metaIdx] <-
    S * (dataFull$.mf_yhat_meta[metaIdx] - dataFull$.mf_yhat_group[metaIdx])
  dataFull$.mf_logL_OBS <- dt_lcm$logL_OBS

  # ---- Step 7: Assemble return object ----
  mlDate <- format(Sys.time(), "Model was estimated on : %b %a %d, %Y at %H:%M")
  totalNParm <- lcmObj$nParm
  totalLogLik <- lcmObj$mlLoglik
  if (metaMethod == "sfa" && !is.null(metaSfaObj)) {
    totalLogLik <- totalLogLik + metaSfaObj$mlLoglik
    totalNParm <- totalNParm + metaSfaObj$nParm
  } else if (metaMethod == "qp") {
    totalNParm <- totalNParm + length(metaFrontierParam)
  }

  obj <- list(
    call = cl,
    formula = formula,
    group = ".mf_group",
    groups = groupNames,
    nGroups = nc,
    S = S,
    typeSfa = typeSfa,
    udist = udist,
    groupType = "sfalcmcross",
    metaMethod = metaMethod,
    sfaApproach = sfaApproach,
    lcmNoGroup = TRUE,
    lcmObj = lcmObj, # single pooled LCM model
    groupModels = setNames(
      # mirror same object per class for compatibility
      rep(list(lcmObj), nc),
      groupNames
    ),
    logDepVar = logDepVar,
    Nobs = nObs,
    NobsMeta = length(metaIdx),
    nXvar = nXvar,
    metaFrontierParam = metaFrontierParam,
    metaFrontierVcov = metaFrontierVcov,
    metaSfaObj = metaSfaObj,
    mlLoglik = totalLogLik,
    nParm = totalNParm,
    dataTable = dataFull,
    mlDate = mlDate
  )
  if (exists("groupFrontierAll")) {
    obj$groupFrontierAll <- groupFrontierAll
  }
  class(obj) <- "smfa"
  return(obj)
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Efficiency/Inefficiency estimation                                           #
# Models:                                                                      #
#        -Stochastic metafrontier analysis and its variants                    #
# Data:  Cross sectional & Pooled data                                         #
#------------------------------------------------------------------------------#

#' Compute efficiency estimates and metatechnology ratios from stochastic metafrontier models
#'
#' @description
#' \code{efficiencies} returns all efficiency estimates and metatechnology ratio
#' (MTR) measures for objects of class \code{"smfa"} returned by
#' \code{\link{smfa}}. The function supports models estimated via
#' linear programming (LP), quadratic programming (QP), and stochastic
#' second-stage SFA (\code{"sfa"}), and for each observation it computes the
#' group-specific technical efficiency, the metafrontier technical efficiency,
#' and the metatechnology ratio (MTR), using both the Jondrow, Lovell, Materov,
#' and Schmidt (1982) (JLMS) and the Battese and Coelli (1988) (BC) estimators.
#' Additional model-specific columns are returned depending on \code{groupType}.
#'
#' @name efficiencies
#'
#' @param object An object of class \code{"smfa"} returned by
#'   \code{\link{smfa}}.
#' @param level A number strictly between 0 and 0.9999 specifying the nominal
#'   coverage for (in-)efficiency confidence intervals. Default \code{0.95}.
#'   This argument is passed to the underlying \code{efficiencies} method of the
#'   group-level model (class \code{"sfacross"}, \code{"sfalcmcross"}, or
#'   \code{"sfaselectioncross"}).
#' @param newData Optional data frame for out-of-sample prediction of efficiency
#'   estimates. When \code{NULL} (default), efficiencies are computed for the
#'   observations used in the estimation.
#' @param ... Further arguments (currently ignored).
#'
#' @return A data frame with one row per observation (in the original row order),
#'   containing the following columns. The exact set of columns depends on
#'   \code{groupType}:
#'
#'   \strong{Columns present for all model types:}
#'   \describe{
#'     \item{\code{id}}{
#'       Observation identifier. Contains the row name of each observation as
#'       it appeared in the data supplied to \code{\link{smfa}}.
#'       When the data frame has no explicit row names, sequential integers
#'       (\code{"1"}, \code{"2"}, \ldots) are used. This column is always the
#'       first column of the returned data frame.}
#'     \item{\code{<group>} or \code{Group_c}}{
#'       The technology group identifier for each observation. For
#'       \code{groupType = "sfacross"} and \code{"sfaselectioncross"}, this
#'       column takes the name of the user-supplied \code{group} variable and
#'       contains the group label to which each observation belongs. For
#'       \code{groupType = "sfalcmcross"}, it is named \code{Group_c} and
#'       contains the integer index of the latent class assigned by the maximum
#'       posterior probability criterion.}
#'     \item{\code{u_g}}{
#'       Group-specific conditional mean of the inefficiency term, computed as
#'       \eqn{E[u_i \mid \varepsilon_i]}. This is the JLMS (Jondrow, Lovell,
#'       Materov, and Schmidt, 1982) point estimate of the inefficiency at the
#'       group-frontier level. For \code{groupType = "sfaselectioncross"},
#'       \code{u_g} is \code{NA} for observations not selected into the sample
#'       (selection indicator = 0).}
#'     \item{\code{TE_group_JLMS}}{
#'       Group-specific technical efficiency using the Jondrow, Lovell,
#'       Materov, and Schmidt (1982) estimator:
#'       \eqn{TE^g_i = \exp(-E[u_i \mid \varepsilon_i])}. For
#'       \code{groupType = "sfaselectioncross"}, \code{NA} for non-selected
#'       observations.}
#'     \item{\code{TE_group_BC}}{
#'       Group-specific technical efficiency using the Battese and Coelli
#'       (1988) estimator:
#'       \eqn{TE^g_i = E[\exp(-u_i) \mid \varepsilon_i]}. For
#'       \code{groupType = "sfaselectioncross"}, \code{NA} for non-selected
#'       observations.}
#'     \item{\code{TE_group_BC_reciprocal}}{
#'       Reciprocal of the Battese and Coelli (1988) group technical efficiency:
#'       \eqn{1 / TE^{g,BC}_i}. For production frontiers this equals the
#'       cost-efficiency index implied by the BC estimator. Present for all three
#'       model types. For \code{groupType = "sfaselectioncross"}, \code{NA} for
#'       non-selected observations.}
#'     \item{\code{u_meta}}{
#'       Metafrontier inefficiency, measuring the technology-gap component
#'       \eqn{U_i \ge 0} that separates the group frontier from the global
#'       metafrontier. Computed from the second-stage SFA when
#'       \code{metaMethod = "sfa"}, or derived from the LP/QP gap as
#'       \eqn{U_i = \max\{S \cdot (\ln \hat{y}^*_i - \ln \hat{y}^g_i), 0\}}
#'       when \code{metaMethod = "lp"} or \code{"qp"}.}
#'     \item{\code{TE_meta_JLMS}}{
#'       Metafrontier technical efficiency based on the JLMS group efficiency:
#'       \eqn{TE^*_{JLMS,i} = TE^g_{JLMS,i} \times MTR_{JLMS,i}}.}
#'     \item{\code{TE_meta_BC}}{
#'       Metafrontier technical efficiency based on the Battese and Coelli
#'       (1988) group efficiency:
#'       \eqn{TE^*_{BC,i} = TE^g_{BC,i} \times MTR_{BC,i}}.}
#'     \item{\code{MTR_JLMS}}{
#'       Metatechnology ratio computed using the JLMS group efficiency:
#'       \eqn{MTR_{JLMS,i} = TE^*_{JLMS,i} / TE^g_{JLMS,i} = \exp(-U_i)}.
#'       Values range from 0 to 1. A value of 1 indicates that the group
#'       frontier for this observation coincides with the metafrontier.}
#'     \item{\code{MTR_BC}}{
#'       Metatechnology ratio computed using the Battese and Coelli (1988)
#'       group efficiency:
#'       \eqn{MTR_{BC,i} = TE^*_{BC,i} / TE^g_{BC,i} = \exp(-U_i)}.}
#'   }
#'
#'   \strong{Additional columns for \code{groupType = "sfacross"} only:}
#'   \describe{
#'     \item{\code{uLB_g}, \code{uUB_g}}{
#'       Lower and upper bounds of the \code{level} confidence interval for the
#'       conditional mean inefficiency \code{u_g}, constructed using the
#'       asymptotic distribution of the conditional estimator. Available for
#'       distributions with closed-form expressions for the confidence bounds,
#'       such as \code{udist = "hnormal"} and \code{udist = "tnormal"}.}
#'     \item{\code{m_g}}{
#'       Mode of the conditional distribution of the one-sided error term
#'       \eqn{u_i \mid \varepsilon_i}. This is an alternative point estimate of
#'       inefficiency. Available for distributions for which the conditional mode
#'       has a closed-form expression.}
#'     \item{\code{TE_group_mode}}{
#'       Group-specific technical efficiency evaluated at the conditional mode:
#'       \eqn{TE^g_{\mathrm{mode},i} = \exp(-m_i)}.}
#'     \item{\code{teBCLB_g}, \code{teBCUB_g}}{
#'       Lower and upper bounds of the \code{level} confidence interval for the
#'       Battese and Coelli (1988) group technical efficiency \code{TE_group_BC}.
#'       Constructed from the corresponding bounds on the conditional
#'       distribution of \eqn{\exp(-u_i \mid \varepsilon_i)}.}
#'   }
#'
#'   \strong{Additional columns for \code{groupType = "sfalcmcross"} only:}
#'   \describe{
#'     \item{\code{PosteriorProb_c}}{
#'       Posterior probability that observation \eqn{i} belongs to its assigned
#'       class (the one with the highest posterior probability). Computed via
#'       Bayes' rule as
#'       \eqn{P(j \mid y_i, x_i) \propto \pi(i,j) \, P(i \mid j)},
#'       where \eqn{\pi(i,j)} is the prior class probability and \eqn{P(i \mid j)}
#'       is the class-conditional likelihood.}
#'     \item{\code{PosteriorProb_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Posterior probability of belonging to latent class \eqn{J}, computed
#'       via Bayes' rule for each class separately. One column is produced for
#'       each estimated class.}
#'     \item{\code{PriorProb_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Prior (unconditional) probability of belonging to latent class \eqn{J},
#'       given by the logistic specification
#'       \eqn{\pi(i,J) = \exp(\bm{\theta}_J'\mathbf{Z}_{hi}) /
#'       \sum_m \exp(\bm{\theta}_m'\mathbf{Z}_{hi})}.}
#'     \item{\code{u_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Conditional mean of the inefficiency term for class \eqn{J}:
#'       \eqn{E[u_{i \mid J} \mid \varepsilon_{i \mid J}]}.}
#'     \item{\code{teBC_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Battese and Coelli (1988) technical efficiency for class \eqn{J}:
#'       \eqn{E[\exp(-u_{i \mid J}) \mid \varepsilon_{i \mid J}]}.}
#'     \item{\code{teBC_reciprocal_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Reciprocal of the class-\eqn{J} Battese and Coelli (1988) efficiency:
#'       \eqn{1/TE^{BC}_{i \mid J}}.}
#'     \item{\code{ineff_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Inefficiency estimate for the observation restricted to class \eqn{J}
#'       (i.e. the value assigned to the class to which the observation
#'       \emph{does} belong; \code{NA} for other classes).}
#'     \item{\code{effBC_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Battese and Coelli (1988) efficiency for the observation's assigned
#'       class; \code{NA} for non-assigned classes.}
#'     \item{\code{ReffBC_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
#'       Reciprocal Battese and Coelli (1988) efficiency for the observation's
#'       assigned class; \code{NA} for non-assigned classes.}
#'   }
#'
#' @details
#' \subsection{Group-specific efficiency estimates}{
#'   For each group, the group-level frontier model is estimated by maximising
#'   the log-likelihood using the distribution specified by \code{udist} in
#'   \code{\link{smfa}}. Given the estimated composite error
#'   \eqn{\varepsilon_i = v_i - Su_i}, the conditional distribution of
#'   \eqn{u_i \mid \varepsilon_i} is used to compute:
#'   \itemize{
#'     \item the JLMS estimator (Jondrow, Lovell, Materov, and Schmidt, 1982):
#'       \eqn{\hat{u}_i = E[u_i \mid \varepsilon_i]}, and
#'       \eqn{TE^g_{JLMS,i} = \exp(-\hat{u}_i)};
#'     \item the BC estimator (Battese and Coelli, 1988):
#'       \eqn{TE^g_{BC,i} = E[\exp(-u_i) \mid \varepsilon_i]};
#'     \item the mode estimator: \eqn{m_i = \mathrm{mode}[u_i \mid \varepsilon_i]},
#'       and \eqn{TE^g_{\mathrm{mode},i} = \exp(-m_i)};
#'     \item confidence bounds on \eqn{u_i} and \eqn{TE^g_{BC,i}} at the
#'       nominal level \code{level}.
#'   }
#'   For \code{groupType = "sfaselectioncross"}, all estimates are \code{NA}
#'   for observations not selected into the sample (binary selection indicator
#'   equal to 0). For \code{groupType = "sfalcmcross"}, the composite
#'   efficiencies \code{u_g}, \code{TE_group_JLMS}, and \code{TE_group_BC}
#'   are computed using the posterior-probability-weighted class assignments.
#' }
#'
#' \subsection{Metatechnology ratio and metafrontier efficiency}{
#'   The MTR measures how far the group frontier lies below the metafrontier
#'   for each observation. Let \eqn{\ln \hat{y}^g_i} be the group-specific
#'   fitted frontier value and \eqn{\ln \hat{y}^*_i} the metafrontier fitted
#'   value.
#'   \itemize{
#'     \item For \code{metaMethod = "lp"} or \code{"qp"} (Battese, Rao, and
#'       O'Donnell, 2004):
#'       \deqn{MTR_i = \exp\!\bigl(
#'           -\max\!\bigl\{S \cdot (\ln \hat{y}^*_i - \ln \hat{y}^g_i),\, 0\bigr\}
#'         \bigr)}
#'       where \eqn{S = 1} for production/profit frontiers and \eqn{S = -1}
#'       for cost frontiers. The technology gap
#'       \eqn{U_i = \max\{S \cdot (\ln \hat{y}^*_i - \ln \hat{y}^g_i), 0\}}
#'       is stored in \code{u_meta}.
#'     \item For \code{metaMethod = "sfa"} with \code{sfaApproach = "huang"}
#'       (Huang, Huang, and Liu, 2014):
#'       \deqn{MTR_i = TE^*_i = \exp(-U_i)}
#'       where \eqn{U_i} is the one-sided error term from the second-stage SFA.
#'     \item For \code{metaMethod = "sfa"} with
#'       \code{sfaApproach = "ordonnell"} (O'Donnell, Rao, and Battese, 2008):
#'       \eqn{MTR_i = TE^{*,\mathrm{sfa}}_i / TE^g_i}, where
#'       \eqn{TE^{*,\mathrm{sfa}}_i} is the technical efficiency from the
#'       second-stage SFA fitted on the LP envelope values.
#'   }
#'   The metafrontier technical efficiency is then:
#'   \deqn{TE^*_i = TE^g_i \times MTR_i}
#'   computed separately for the JLMS and BC group efficiency estimators.
#'   Both \code{MTR_JLMS} and \code{MTR_BC} are reported, distinguishing
#'   which group-level efficiency estimator was used as the basis.
#' }
#'
#' @references
#' Battese, G. E., and Coelli, T. J. 1988. Prediction of firm-level technical
#' efficiencies with a generalized frontier production function and panel data.
#' \emph{Journal of Econometrics}, \bold{38}(3), 387--399.
#' \doi{10.1016/0304-4076(88)90053-X}
#'
#' Battese, G. E., Rao, D. S. P., and O'Donnell, C. J. 2004. A metafrontier
#' production function for estimation of technical efficiencies and technology
#' gaps for firms operating under different technologies.
#' \emph{Journal of Productivity Analysis}, \bold{21}(1), 91--103.
#' \doi{10.1023/B:PROD.0000012454.06094.29}
#'
#' Huang, C. J., Huang, T.-H., and Liu, N.-H. 2014. A new approach to
#' estimating the metafrontier production function based on a stochastic
#' frontier framework. \emph{Journal of Productivity Analysis},
#' \bold{42}(3), 241--254. \doi{10.1007/s11123-014-0402-2}
#'
#' Jondrow, J., Lovell, C. A. K., Materov, I. S., and Schmidt, P. 1982. On
#' the estimation of technical inefficiency in the stochastic frontier
#' production function model. \emph{Journal of Econometrics},
#' \bold{19}(2-3), 233--238.
#' \doi{10.1016/0304-4076(82)90004-5}
#'
#' O'Donnell, C. J., Rao, D. S. P., and Battese, G. E. 2008. Metafrontier
#' frameworks for the study of firm-level efficiencies and technology ratios.
#' \emph{Empirical Economics}, \bold{34}(2), 231--255.
#' \doi{10.1007/s00181-007-0119-4}
#'
#' Orea, L., and Kumbhakar, S. C. 2004. Efficiency measurement using a latent
#' class stochastic frontier model. \emph{Empirical Economics}, \bold{29}(1),
#' 169--183. \doi{10.1007/s00181-003-0184-2}
#'
#' Dakpo, K. H., Desjeux, Y., and Latruffe, L. 2023. sfaR: Stochastic Frontier
#' Analysis using R. R package version 1.0.1.
#' \url{https://CRAN.R-project.org/package=sfaR}
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier
#' analysis model fitting function using cross-sectional or pooled data;
#' \code{\link[sfaR]{efficiencies}}, for the underlying group-level efficiency
#' extractor.
#'
#' @aliases efficiencies.smfa
#' @importFrom sfaR efficiencies
#' @export
efficiencies.smfa <- function(
  object,
  level = 0.95,
  newData = NULL,
  ...
) {
  if (level < 0 || level > 0.9999) {
    stop("'level' must be between 0 and 0.9999", call. = FALSE)
  }
  if (!is.null(newData)) {
    warning(
      "'newData' is not supported for smfa objects; ignored.",
      call. = FALSE
    )
  }

  dataFull <- object$dataTable
  groupVar <- object$group
  groups <- object$groups
  S <- object$S
  N <- nrow(dataFull)
  groupType <- if (!is.null(object$groupType)) object$groupType else "sfacross"

  # ---------------------------------------------------------------------------
  # Helper: safely pull column with fallback to N-length NA vector
  # ---------------------------------------------------------------------------
  .g <- function(df, col, n) {
    if (!is.null(df) && col %in% names(df)) df[[col]] else rep(NA_real_, n)
  }
  .gc <- function(df, col, n) {
    if (!is.null(df) && col %in% names(df)) as.character(df[[col]]) else rep(NA_character_, n)
  }

  # ---------------------------------------------------------------------------
  # Containers for the CORE columns (used in MTR computation)
  # ---------------------------------------------------------------------------
  u_g <- rep(NA_real_, N)
  teGroup_JLMS <- rep(NA_real_, N)
  teGroup_BC <- rep(NA_real_, N)

  # Group identifier column
  if (groupType == "sfalcmcross") {
    group_col <- rep(NA_character_, N)
  } else {
    group_col <- as.character(dataFull[[groupVar]])
  }

  # ---------------------------------------------------------------------------
  # Extra sfaR columns that vary by model type — stored as named lists of
  # N-length vectors, assembled into the result data frame at the end.
  # ---------------------------------------------------------------------------
  extra_cols <- list()

  # ---- sfacross / sfaselectioncross optional extras ----
  uLB_g <- rep(NA_real_, N)
  uUB_g <- rep(NA_real_, N)
  m_g <- rep(NA_real_, N)
  teMO_g <- rep(NA_real_, N)
  teBCLB_g <- rep(NA_real_, N)
  teBCUB_g <- rep(NA_real_, N)
  teBC_recip_g <- rep(NA_real_, N)

  # ---- sfalcmcross extras: will be built dynamically ----
  lcm_extra_list <- NULL # will become a data frame with all LCM columns

  # ===========================================================================
  # ---- LCM NO-GROUP path: single pooled sfalcmcross ----
  # ===========================================================================
  if (isTRUE(object$lcmNoGroup) && !is.null(object$lcmObj)) {
    effLcm <- efficiencies(object$lcmObj, level = level)
    group_col <- .gc(effLcm, "Group_c", N)
    u_g <- .g(effLcm, "u_c", N)
    teGroup_JLMS <- .g(effLcm, "teJLMS_c", N)
    teGroup_BC <- .g(effLcm, "teBC_c", N)
    teBC_recip_g <- .g(effLcm, "teBC_reciprocal_c", N)
    # Preserve ALL remaining LCM columns (class-specific, posterior probs, etc.)
    lcm_cols_to_keep <- setdiff(
      names(effLcm),
      c("Group_c", "u_c", "teJLMS_c", "teBC_c", "teBC_reciprocal_c")
    )
    if (length(lcm_cols_to_keep) > 0) {
      lcm_extra_list <- effLcm[, lcm_cols_to_keep, drop = FALSE]
    }
  } else {
    # ===========================================================================
    # ---- Normal grouped path ----
    # ===========================================================================
    # For sfalcmcross we aggregate the LCM extra columns across groups
    lcm_extra_all <- vector("list", length(groups))
    names(lcm_extra_all) <- groups

    for (g in groups) {
      idx <- which(dataFull[[groupVar]] == g)

      effG <- tryCatch({
          m_g_obj <- object$groupModels[[g]]
          efficiencies(m_g_obj, level = level)
        },
        error = function(e) {
          warning(sprintf("Group '%s' efficiencies failed: %s", g, e$message), call. = FALSE)
          NULL
        }
      )

      if (is.null(effG)) next
      ng <- length(idx) # number of obs in this group

      if (groupType == "sfalcmcross") {
        # sfaR sfalcmcross columns:
        # Group_c, PosteriorProb_c, u_c, teJLMS_c, teBC_c, teBC_reciprocal_c,
        # PosteriorProb_c1, PriorProb_c1, u_c1, teBC_c1, teBC_reciprocal_c1, ...(per class)
        # ineff_c1, effBC_c1, ReffBC_c1, ...
        group_col[idx] <- .gc(effG, "Group_c", ng)
        u_g[idx] <- .g(effG, "u_c", ng)
        teGroup_JLMS[idx] <- .g(effG, "teJLMS_c", ng)
        teGroup_BC[idx] <- .g(effG, "teBC_c", ng)
        teBC_recip_g[idx] <- .g(effG, "teBC_reciprocal_c", ng)
        # Extra LCM-specific columns
        lcm_extra_cols <- setdiff(
          names(effG),
          c("Group_c", "u_c", "teJLMS_c", "teBC_c", "teBC_reciprocal_c")
        )
        if (length(lcm_extra_cols) > 0) {
          lcm_extra_all[[g]] <- list(idx = idx, df = effG[, lcm_extra_cols, drop = FALSE])
        }
      } else if (groupType == "sfaselectioncross") {
        # sfaR sfaselectioncross: u, teJLMS, teBC, teBC_reciprocal (NA for non-selected)
        stopifnot(nrow(effG) == ng)
        u_g[idx] <- .g(effG, "u", ng)
        teGroup_JLMS[idx] <- .g(effG, "teJLMS", ng)
        teGroup_BC[idx] <- .g(effG, "teBC", ng)
        teBC_recip_g[idx] <- .g(effG, "teBC_reciprocal", ng)
      } else {
        # sfacross: u, uLB, uUB, teJLMS, m, teMO, teBC, teBCLB, teBCUB, teBC_reciprocal
        u_g[idx] <- .g(effG, "u", ng)
        uLB_g[idx] <- .g(effG, "uLB", ng)
        uUB_g[idx] <- .g(effG, "uUB", ng)
        teGroup_JLMS[idx] <- .g(effG, "teJLMS", ng)
        m_g[idx] <- .g(effG, "m", ng)
        teMO_g[idx] <- .g(effG, "teMO", ng)
        teGroup_BC[idx] <- .g(effG, "teBC", ng)
        teBCLB_g[idx] <- .g(effG, "teBCLB", ng)
        teBCUB_g[idx] <- .g(effG, "teBCUB", ng)
        teBC_recip_g[idx] <- .g(effG, "teBC_reciprocal", ng)
      }
    } # end for g

    # For sfalcmcross: assemble extra columns across groups into one data frame
    if (groupType == "sfalcmcross") {
      all_lcm_cols <- unique(unlist(lapply(lcm_extra_all, function(x) {
        if (!is.null(x)) names(x$df) else NULL
      })))
      if (length(all_lcm_cols) > 0) {
        # Initialise with NAs
        lcm_extra_list <- setNames(
          as.data.frame(matrix(NA_real_, nrow = N, ncol = length(all_lcm_cols))),
          all_lcm_cols
        )
        for (g in groups) {
          x <- lcm_extra_all[[g]]
          if (!is.null(x)) {
            for (col in names(x$df)) {
              lcm_extra_list[[col]][x$idx] <- x$df[[col]]
            }
          }
        }
      }
    }
  } # end else (normal grouped path)

  # ===========================================================================
  # ---- Compute metafrontier efficiency and MTR ----
  # ===========================================================================
  yhat_group <- dataFull$.mf_yhat_group
  yhat_meta <- dataFull$.mf_yhat_meta
  metaMethod <- object$metaMethod
  sfaApproach <- if (!is.null(object$sfaApproach)) object$sfaApproach else "ordonnell"

  if (metaMethod %in% c("lp", "qp")) {
    mtrRes <- compute_mtr(
      yhat_group   = yhat_group,
      yhat_meta    = yhat_meta,
      teGroup_BC   = teGroup_BC,
      teGroup_JLMS = teGroup_JLMS,
      uGroup       = u_g,
      S            = S,
      metaMethod   = metaMethod,
      sfaApproach  = sfaApproach
    )
    teMeta_BC <- mtrRes$teMeta_BC
    teMeta_JLMS <- mtrRes$teMeta_JLMS
    mtr_BC <- mtrRes$mtr_BC
    mtr_JLMS <- mtrRes$mtr_JLMS
    u_meta <- mtrRes$u_meta
  } else {
    # SFA metafrontier: pull ALL efficiency variables from second-stage SFA
    effMeta <- efficiencies(object$metaSfaObj, level = level)
    effMeta_sfa <- data.frame(
      teBC   = rep(NA_real_, N),
      teJLMS = rep(NA_real_, N),
      u      = rep(NA_real_, N)
    )
    metaRows <- which(!is.na(yhat_meta))
    n_meta <- min(length(metaRows), nrow(effMeta))
    indices <- metaRows[seq_len(n_meta)]
    if (!is.null(effMeta[["teBC"]])) effMeta_sfa$teBC[indices] <- effMeta[["teBC"]][seq_len(n_meta)]
    if (!is.null(effMeta[["teJLMS"]])) effMeta_sfa$teJLMS[indices] <- effMeta[["teJLMS"]][seq_len(n_meta)]
    if (!is.null(effMeta[["u"]])) effMeta_sfa$u[indices] <- effMeta[["u"]][seq_len(n_meta)]

    mtrRes <- compute_mtr(
      yhat_group = yhat_group,
      yhat_meta = yhat_meta,
      teGroup_BC = teGroup_BC,
      teGroup_JLMS = teGroup_JLMS,
      uGroup = u_g,
      S = S,
      metaMethod = metaMethod,
      sfaApproach = sfaApproach,
      effMeta_sfa = effMeta_sfa
    )
    teMeta_BC <- mtrRes$teMeta_BC
    teMeta_JLMS <- mtrRes$teMeta_JLMS
    mtr_BC <- mtrRes$mtr_BC
    mtr_JLMS <- mtrRes$mtr_JLMS
    u_meta <- mtrRes$u_meta
  }

  # 0. Observation id (mirrors sfaR's 'IdObs'; shown first for easy lookup)
  id_col <- rownames(dataFull)
  if (is.null(id_col)) id_col <- as.character(seq_len(N))

  # 1. Group identifier
  if (groupType == "sfalcmcross") {
    res <- data.frame(id = id_col, Group_c = group_col, stringsAsFactors = FALSE)
  } else {
    res <- data.frame(id = id_col, group_col = group_col, stringsAsFactors = FALSE)
    names(res)[2] <- groupVar
  }

  # 2. Core group-level columns (always present)
  res$u_g <- u_g
  res$TE_group_JLMS <- teGroup_JLMS
  res$TE_group_BC <- teGroup_BC

  # 3. Optional sfacross/sfaselectioncross columns (present when model produces them)
  if (groupType %in% c("sfacross", "sfaselectioncross")) {
    res$TE_group_BC_reciprocal <- teBC_recip_g
  }
  if (groupType == "sfacross") {
    # Confidence bounds and mode-based estimates — only attach if non-trivial
    if (any(!is.na(uLB_g))) res$uLB_g <- uLB_g
    if (any(!is.na(uUB_g))) res$uUB_g <- uUB_g
    if (any(!is.na(m_g))) res$m_g <- m_g
    if (any(!is.na(teMO_g))) res$TE_group_mode <- teMO_g
    if (any(!is.na(teBCLB_g))) res$teBCLB_g <- teBCLB_g
    if (any(!is.na(teBCUB_g))) res$teBCUB_g <- teBCUB_g
  }

  # 4. sfalcmcross class-specific columns (passed through verbatim from sfaR)
  if (groupType == "sfalcmcross") {
    res$TE_group_BC_reciprocal <- teBC_recip_g
    if (!is.null(lcm_extra_list) && ncol(lcm_extra_list) > 0) {
      res <- cbind(res, lcm_extra_list)
    }
  }

  # 5. Metafrontier columns
  res$u_meta <- u_meta
  res$TE_meta_JLMS <- teMeta_JLMS
  res$TE_meta_BC <- teMeta_BC
  res$MTR_JLMS <- mtr_JLMS
  res$MTR_BC <- mtr_BC

  rownames(res) <- rownames(dataFull)
  return(res)
}

################################################################################
#                                                                              #
# R internal helper functions for stochastic metafrontier analysis             #
# sfaR package                                                                 #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Metafrontier estimation helpers                                              #
#------------------------------------------------------------------------------#
.pchibarsq <- function(p, df = 1, mix = 0.5, lower.tail = TRUE, log.p = FALSE) {
  df <- rep(df, length.out = length(p))
  mix <- rep(mix, length.out = length(p))
  c1 <- ifelse(df == 1, if (lower.tail) {
    1
  } else {
    0
  }, pchisq(p, df - 1, lower.tail = lower.tail))
  c2 <- pchisq(p, df, lower.tail = lower.tail)
  r <- mix * c1 + (1 - mix) * c2
  if (log.p) {
    log(r)
  } else {
    r
  }
}

#' @importFrom stats pchisq qchisq uniroot
#' @noRd
.qchibarsq <- function(q, df = 1, mix = 0.5) {
  n <- max(length(q), length(df), length(mix))
  df <- rep(df, length.out = n)
  mix <- rep(mix, length.out = n)
  q <- rep(q, length.out = n)
  tmpf2 <- function(q, df, mix) {
    if (df > 1) {
      tmpf <- function(x) {
        .pchibarsq(x, df, mix) - q
      }
      uniroot(tmpf, lower = qchisq(q, df - 1), upper = qchisq(
        q,
        df
      ))$root
    } else {
      newq <- (q - mix) / (1 - mix)
      ifelse(newq < 0, 0, qchisq(newq, df = 1))
    }
  }
  mapply(tmpf2, q, df, mix)
}


# Pretty-print name for the metafrontier method -----------
#' @param metaMethod character string for metafrontier method
#' @param sfaApproach character string for SFA approach ("ordonnell" or "huang")
#' @noRd
mfauxdist <- function(metaMethod, sfaApproach = "ordonnell") {
  base <- switch(metaMethod,
    lp = "Linear Programming (LP) Metafrontier",
    qp = "Quadratic Programming (QP) Metafrontier",
    sfa = if (sfaApproach == "huang") {
      "SFA Metafrontier [Huang et al. (2014), two-stage]"
    } else {
      "SFA Metafrontier [O'Donnell et al. (2008), envelope]"
    }
  )
  base
}

# LP-based metafrontier: column-maximum of evaluated group frontier values -----
# The metafrontier predicted value for obs i: max over all group betas at X_i
#' @param groupFrontierMat N x G matrix of group frontier predicted values (all
#'   groups evaluated at each obs's X)
#' @noRd
mf_lp <- function(groupFrontierMat) {
  apply(groupFrontierMat, 1, max, na.rm = TRUE)
}

# QP-based metafrontier: OLS of envelope on X ----------------------------------
#' @param Xvar matrix of explanatory variables (N x K)
#' @param groupFrontierMat N x G matrix of group frontier predicted values
#' @noRd
mf_qp <- function(Xvar, groupFrontierMat) {
  yMeta <- apply(groupFrontierMat, 1, max, na.rm = TRUE)
  if (colnames(Xvar)[1] == "(Intercept)") {
    fit <- lm(yMeta ~ ., data = as.data.frame(Xvar[, -1, drop = FALSE]))
  } else {
    fit <- lm(yMeta ~ -1 + ., data = as.data.frame(Xvar))
  }
  list(
    beta  = coef(fit),
    vcov  = vcov(fit),
    yhat  = fitted(fit),
    sigma = summary(fit)$sigma,
    fit   = fit
  )
}

# O'Donnell SFA metafrontier: sfacross on envelope of group-evaluated values ---
#' @param Xvar matrix (N x K)
#' @param groupFrontierMat N x G matrix of group frontier values (all betas
#'   evaluated at all obs)
#' @param S 1 or -1
#' @param method optimization method
#' @param udist distribution string
#' @param ... passed to sfacross
#' @noRd
mf_sfa_ordonnell <- function(Xvar, groupFrontierMat, S, method, udist, ...) {
  yMeta <- apply(groupFrontierMat, 1, max, na.rm = TRUE)
  mf_sfa_fit(
    Xvar = Xvar, yMeta = yMeta, S = S, method = method,
    udist = udist, depname = "lp_envelope", ...
  )
}

# Huang (2014) SFA metafrontier: sfacross on pooled own-group fitted values ----
# DV = yhat_{g(i)} for each obs, i.e. the fitted value from the obs's own group.
#' @param Xvar matrix (N x K) - same X used in group models
#' @param yhat_group N-vector of group fitted values (each obs from its own group)
#' @param S 1 or -1
#' @param method optimization method
#' @param udist distribution string
#' @param ... passed to sfacross
#' @noRd
mf_huang <- function(Xvar, yhat_group, S, method, udist, ...) {
  mf_sfa_fit(
    Xvar = Xvar, yMeta = yhat_group, S = S, method = method,
    udist = udist, depname = "group_fitted_values", ...
  )
}

# Shared second-stage sfacross fitting routine ---------------------------------
#' @noRd
mf_sfa_fit <- function(Xvar, yMeta, S, method, udist, depname = ".yMeta", ...) {
  # Use safe syntactic names internally to avoid issues when original Xvar
  # column names contain expressions like "log(df$var)", spaces, or operators.
  K <- ncol(Xvar)
  orig_names <- colnames(Xvar)
  safe_names <- paste0(".X", seq_len(K))

  hasIntercept <- orig_names[1L] == "(Intercept)"

  df_meta <- setNames(as.data.frame(Xvar), safe_names)
  df_meta[[depname]] <- yMeta

  if (hasIntercept) {
    rhs_parts <- safe_names[-1L]
    fStr <- if (length(rhs_parts) > 0L) {
      paste(depname, "~", paste(rhs_parts, collapse = " + "))
    } else {
      paste(depname, "~ 1")
    }
  } else {
    fStr <- paste(depname, "~ -1 +", paste(safe_names, collapse = " + "))
  }
  fml <- as.formula(fStr)

  sfaObj <- tryCatch(
    sfacross(
      formula = fml, udist = udist, S = S, data = df_meta,
      method = method, ...
    ),
    error = function(e) {
      stop("SFA metafrontier estimation failed: ", e$message, call. = FALSE)
    }
  )

  # Return coefficients indexed back to original names
  beta_idx <- seq_len(K)
  betas <- sfaObj$mlParam[beta_idx]
  names(betas) <- orig_names
  vcov_mat <- sfaObj$invHessian[beta_idx, beta_idx, drop = FALSE]
  dimnames(vcov_mat) <- list(orig_names, orig_names)

  list(
    beta   = betas,
    vcov   = vcov_mat,
    yhat   = sfaObj$dataTable$mlFitted,
    sfaObj = sfaObj
  )
}

# Extract best-class fitted value from sfalcmcross object ----------------------
# For each observation, selects the fitted value from the class with the highest
# posterior probability (using the same posterior logic as efficiencies.sfalcmcross).
#' @param lcmObj a sfalcmcross model object
#' @noRd
extract_lcm_fitted <- function(lcmObj) {
  dt <- lcmObj$dataTable
  nClass <- lcmObj$nClasses

  # efficiencies.sfalcmcross returns Group_c (best-posterior class index, 1-based)
  effRes <- efficiencies(lcmObj)

  if ("Group_c" %in% names(effRes)) {
    bestClass <- effRes$Group_c
    best_fitted <- vapply(seq_len(nrow(dt)), function(i) {
      colnm <- paste0("mlFitted_c", bestClass[i])
      if (colnm %in% names(dt)) dt[[colnm]][i] else NA_real_
    }, numeric(1))
  } else {
    # Fallback: class-1 fitted values
    best_fitted <- if ("mlFitted_c1" %in% names(dt)) dt[["mlFitted_c1"]] else NA_real_
  }
  best_fitted
}

# Compute metatechnology ratio (MTR) and metafrontier efficiencies ----------------
#' @param yhat_group N-vector of group frontier fitted values
#' @param yhat_meta N-vector of metafrontier fitted values
#' @param teGroup_BC N-vector of group efficiency (teBC from group SFA)
#' @param teGroup_JLMS N-vector of group efficiency (teJLMS from group SFA)
#' @param uGroup N-vector of group inefficiency
#' @param S integer 1 or -1
#' @param metaMethod character "lp", "qp", or "sfa"
#' @param sfaApproach character "ordonnell" or "huang" (used when metaMethod="sfa")
#' @param effMeta_sfa optional data.frame of efficiency estimates from second-stage SFA
#' @noRd
compute_mtr <- function(yhat_group, yhat_meta, teGroup_BC, teGroup_JLMS, uGroup, S,
                        metaMethod = "lp", sfaApproach = "ordonnell",
                        effMeta_sfa = NULL) {
  if (metaMethod %in% c("lp", "qp")) {
    gap <- S * (yhat_meta - yhat_group)
    mtr <- exp(-pmax(gap, 0))
    teMeta_BC <- teGroup_BC * mtr
    teMeta_JLMS <- teGroup_JLMS * mtr
    u_meta <- uGroup - log(mtr)
    list(teMeta_BC = teMeta_BC, teMeta_JLMS = teMeta_JLMS, mtr_BC = mtr, mtr_JLMS = mtr, u_meta = u_meta)
  } else if (sfaApproach == "huang") {
    mtr_BC <- effMeta_sfa$teBC
    mtr_JLMS <- effMeta_sfa$teJLMS
    u_mtr <- effMeta_sfa$u
    teMeta_BC <- teGroup_BC * mtr_BC
    teMeta_JLMS <- teGroup_JLMS * mtr_JLMS
    u_meta <- uGroup + u_mtr
    list(teMeta_BC = teMeta_BC, teMeta_JLMS = teMeta_JLMS, mtr_BC = mtr_BC, mtr_JLMS = mtr_JLMS, u_meta = u_meta)
  } else {
    # ordonnell
    teMeta_BC <- effMeta_sfa$teBC
    teMeta_JLMS <- effMeta_sfa$teJLMS
    u_meta <- effMeta_sfa$u
    mtr_BC <- teMeta_BC / teGroup_BC
    mtr_JLMS <- teMeta_JLMS / teGroup_JLMS

    n_gt1 <- sum(!is.na(mtr_BC) & mtr_BC > 1)
    if (n_gt1 > 0) {
      warning(
        sprintf("%d MTR value(s) > 1 detected in O'Donnell SFA approach. ", n_gt1),
        "This typically occurs when the second-stage SFA estimates near-zero ",
        "inefficiency (sigma_u -> 0), causing TE_meta ~= 1 and MTR = TE_meta/TE_group > 1. ",
        "Consider using metaMethod='lp' or sfaApproach='huang' instead.",
        call. = FALSE
      )
    }
    list(teMeta_BC = teMeta_BC, teMeta_JLMS = teMeta_JLMS, mtr_BC = mtr_BC, mtr_JLMS = mtr_JLMS, u_meta = u_meta)
  }
}

smfaStartupMessage <- function() {
  art_lines <- c(
    "                        .d888         ",
    "                       d88P\"          ",
    "                       888            ",
    ".d8888b  88888b.d88b.  888888 8888b.  ",
    "88K      888 \"888 \"88b 888       \"88b ",
    " Y8888b. 888  888  888 888   .d888888  ",
    "     X88 888  888  888 888   888  888 ",
    " 88888P' 888  888  888 888   \"Y888888 "
  )
  base_msg <- paste0(
    "\n\n* Please cite the 'smfa' package as:\n",
    "Owili, S. O. (2026). smfa: Stochastic Metafrontier Analysis. R package version 1.0.0.\n\n",
    "See also: citation(\"smfa\")\n\n",
    "* For any questions, suggestions, or comments on the 'smfa' package, you can contact the authors directly or visit:\n",
    "  https://github.com/SulmanOlieko/smfa/issues\n"
  )

  console_width <- getOption("width")
  art_width <- max(nchar(art_lines))

  if (console_width < art_width) {
    # 1. Truncate all lines to fit the console width
    trimmed_art <- substr(art_lines, 1, console_width - 5)

    # 2. Add the ASCII ellipsis strictly to the last row
    ellipsis <- "      88 88 88"
    e_len <- nchar(ellipsis)
    cut_point <- max(0, console_width - e_len)

    last_line_index <- length(trimmed_art)
    trimmed_art[last_line_index] <- paste0(
      substr(art_lines[last_line_index], 1, cut_point),
      ellipsis
    )

    # 3. Set the version alignment target
    align_width <- console_width
  } else {
    trimmed_art <- art_lines
    align_width <- art_width
  }

  art_msg <- paste(trimmed_art, collapse = "\n")

  # 4. Pad the version text so it hits the exact right edge
  version_msg <- paste0(
    "\n",
    sprintf(paste0("%", align_width, "s"), "version 1.0.0")
  )

  return(paste0(art_msg, version_msg, base_msg))
}

.onAttach <- function(lib, pkg) {
  # We still skip printing during the CRAN check to keep the test logs clean
  in_chk <- Sys.getenv("_R_CHECK_PACKAGE_NAME_") != ""

  if (!in_chk) {
    msg <- smfaStartupMessage()
  } else {
    msg <- paste0(
      "\n* Please cite the 'smfa' package as:\n",
      "Owili, SO. (2026). smfa: Metafrontier Analysis in R. R package version 1.0.0.\n\n",
      "See also: citation(\"smfa\")\n\n",
      "* For any questions, suggestions, or comments on the 'smfa' package, you can contact the authors directly or visit:\n",
      "  https://github.com/SulmanOlieko/smfa/issues\n"
    )
  }

  packageStartupMessage(msg)
  invisible()
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Variance - Covariance Matrix of estimates                                    #
# Models:                                                                      #
#           -Stochastic Frontier Analysis                                      #
#           -Latent Class Stochastic Frontier Analysis                         #
#           -Sample selection correction for Stochastic Frontier Model         #
# Data: Cross sectional data & Pooled data                                     #
#------------------------------------------------------------------------------#

#' Extract total number of observations used in frontier models
#'
#' This function extracts the total number of 'observations' from a
#' fitted point frontier model.
#'
#' `nobs` gives the number of observations actually
#' used by the estimation procedure.
#'
#' @name nobs
#'
#' @param object a `smfa`
#' object for which the number of total observations is to be extracted.
#' @param \dots Currently ignored.
#'
#' @return A single number, normally an integer.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data
#' @keywords attribute
#'
# Extract number of observations for smfa ----------
#' @rdname nobs
#' @aliases nobs.smfa
#' @importFrom stats nobs
#' @export
nobs.smfa <- function(object, ...) {
  return(object$Nobs)
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Log-likelihood extraction                                                    #
# Models:                                                                      #
#           -Stochastic Metafrontier Analysis                                  #
# Data:     Cross sectional & Pooled data                                      #
#------------------------------------------------------------------------------#

#' Extract log-likelihood value of stochastic metafrontier models
#'
#' \code{\link{logLik}} extracts the log-likelihood value(s) from stochastic
#' metafrontier models estimated with \code{\link{smfa}}.
#'
#' @param object A stochastic metafrontier model returned
#' by \code{\link{smfa}}.
#' @param individual Logical. If \code{FALSE} (default), the sum of all
#' observations' log-likelihood values is returned. If \code{TRUE}, a vector of
#' each observation's log-likelihood value is returned.
#' @param ... Currently ignored.
#'
#' @name logLik
#'
#' @return \code{\link{logLik}} returns either an object of class
#' \code{'logLik'}, which is the log-likelihood value with the total number of
#' observations (\code{nobs}) and the number of free parameters (\code{df}) as
#' attributes, when \code{individual = FALSE}, or a list of elements, containing
#' the log-likelihood of each observation (\code{logLik}), the total number of
#' observations (\code{Nobs}) and the number of free parameters (\code{df}),
#' when \code{individual = TRUE}.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data.
#'
#' @keywords methods likelihood
#'
#' @aliases logLik.smfa
#' @importFrom stats logLik
#' @export
logLik.smfa <- function(object, individual = FALSE, ...) {
  if (length(individual) != 1 || !is.logical(individual[1])) {
    stop("argument 'individual' must be a single logical value", call. = FALSE)
  }
  if (individual) {
    LL <- list()
    LL[["logLik"]] <- object$dataTable$.mf_logL_OBS
    LL[["Nobs"]] <- object$Nobs
    LL[["df"]] <- object$nParm
    return(LL)
  } else {
    LL <- object$mlLoglik
    attributes(LL)$nobs <- object$Nobs
    attributes(LL)$df <- object$nParm
    class(LL) <- "logLik"
    return(LL)
  }
}

################################################################################
#                                                                              #
# R functions for the smfa package                                     #
#                                                                              #
################################################################################

#------------------------------------------------------------------------------#
# Variance - Covariance Matrix of estimates                                    #
# Models:                                                                      #
#           -Stochastic metafrontier analysis                                  #
#           -Latent class stochastic metafrontier analysis                     #
#           -Sample selection correction for stochastic metafrontier model     #
# Data: Cross sectional data & Pooled data                                     #
#------------------------------------------------------------------------------#

#' Compute variance-covariance matrix of stochastic metafrontier models
#'
#' \code{\link{vcov}} computes the variance-covariance matrix of the maximum
#' likelihood (ML) coefficients from stochastic metafrontier models estimated with
#' \code{\link{smfa}}.
#'
#' @details The variance-covariance matrix is obtained by the inversion of the
#' negative Hessian matrix. Depending on the distribution and the
#' \code{'hessianType'} option, the analytical/numeric Hessian or the bhhh
#' Hessian is evaluated.
#'
#' @param object A stochastic metafrontier model returned
#' by \code{\link{smfa}}.
#' @param ... Currently ignored
#'
#' @name vcov
#'
#' @return The variance-covariance matrix of the maximum likelihood
#' coefficients is returned.
#'
#' @seealso \code{\link{smfa}}, for the stochastic metafrontier analysis model
#' fitting function using cross-sectional or pooled data.
#'
#' @keywords methods vcov
#'
# variance covariance matrix for smfa ----------
#' @rdname vcov
#' @aliases vcov.smfa
#' @importFrom stats vcov
#' @export
vcov.smfa <- function(object, ...) {
  if (!is.null(object$metaFrontierVcov)) {
    return(object$metaFrontierVcov)
  }
  # LP method: no metafrontier vcov; return a named list of group vcov's
  lapply(object$groupModels, function(m) m$invHessian)
}

---
title: "Latent Class SFA Metafrontier (groupType = \"sfalcmcross\")"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Latent Class SFA Metafrontier (groupType = "sfalcmcross")}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)
```

## Overview

In many applications, the technology groups that firms belong to are **unobserved** — we
cannot directly observe which firms operate under which technology type. The latent class
model (LCM) addresses this by:

1. Fitting a **pooled latent class SFA** on the entire dataset using the `sfaR` implementation based on Dakpo et al. (2021), simultaneously estimating
   class-specific frontier parameters and class membership probabilities for each firm.
2. Assigning each firm to a class based on its highest **posterior class probability**.
3. Using these assignments as the technology groups for the metafrontier.

This approach is appropriate when:
- No observed group variable is available.
- Technology heterogeneity is suspected but not directly measurable.
- A priori group boundaries are unclear or arbitrary.

## Data Preparation

We use the `utility` dataset from `sfaR`, which contains 791 observations from US
electricity utilities. We estimate a cost frontier with no explicit group variable.

```{r data}
library(smfa)
data("utility", package = "sfaR")
```

## Method 1: LCM + LP Metafrontier

Fit a 2-class latent class pooled SFA, then estimate the LP deterministic envelope over
the inferred class frontiers.

```{r lp}
meta_lcm_lp <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility,
  S          = -1,          # cost frontier (S = -1)
  groupType  = "sfalcmcross",
  lcmClasses = 2,           # number of latent classes
  metaMethod = "lp"
)
summary(meta_lcm_lp)
```

> **Note:** The `group` argument is not needed when `groupType = "sfalcmcross"` — the
> latent classes are identified automatically by the LCM. The `lcmClasses` argument
> controls the number of classes.

## Method 2: LCM + QP Metafrontier

```{r qp}
meta_lcm_qp <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility,
  S          = -1,
  groupType  = "sfalcmcross",
  lcmClasses = 2,
  metaMethod = "qp"
)
summary(meta_lcm_qp)
```

## Method 3: LCM + SFA (Huang)

```{r huang}
meta_lcm_huang <- smfa(
  formula     = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data        = utility,
  S           = -1,
  groupType   = "sfalcmcross",
  lcmClasses  = 2,
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_lcm_huang)
```

## Method 4: LCM + SFA (O'Donnell)

```{r odonnell}
meta_lcm_odonnell <- smfa(
  formula     = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data        = utility,
  S           = -1,
  groupType   = "sfalcmcross",
  lcmClasses  = 2,
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_lcm_odonnell)
```

## Choosing the Number of Classes

The number of latent classes (`lcmClasses`) should be guided by economic theory and
information criteria. You can compare models with different numbers of classes:

```{r compare-classes}
meta_lcm_2 <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility, S = -1,
  groupType  = "sfalcmcross", lcmClasses = 2, metaMethod = "lp"
)
meta_lcm_3 <- smfa(
  formula    = log(tc/wf) ~ log(y) + log(wl/wf) + log(wk/wf),
  data       = utility, S = -1,
  groupType  = "sfalcmcross", lcmClasses = 3, metaMethod = "lp"
)
# Compare information criteria
ic(meta_lcm_2)
ic(meta_lcm_3)
```

Prefer the model with the lower AIC/BIC.

## Extracting Efficiencies and Posterior Probabilities

For LCM models, `efficiencies()` returns extra columns for posterior class membership
probabilities, which can be used for robustness checks or classification:

```{r eff}
eff_lcm <- efficiencies(meta_lcm_lp)
head(eff_lcm)

# Key LCM-specific columns:
# Group_c          — most likely class assignment
# PosteriorProb_c  — posterior probability of assigned class
# PosteriorProb_c1 — posterior probability of Class 1
# PosteriorProb_c2 — posterior probability of Class 2
```

### Class membership summary

```{r class-summary}
# Proportion assigned to each class and mean posterior probability
with(eff_lcm, table(Group_c)) / nrow(eff_lcm) * 100   # % in each class
```

## Key Difference from `sfacross`

| Feature | `sfacross` | `sfalcmcross` |
|---------|-----------|--------------|
| Group variable | Required | Not required |
| Group estimation | Separate SFA per group | Pooled LCM simultaneously |
| Output | Group-level SFA summaries | Pooled LCM summary with class-specific parameters |
| Extra efficiency columns | Confidence bounds | Posterior class probabilities |

---
title: "Standard SFA Metafrontier (groupType = \"sfacross\")"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Standard SFA Metafrontier (groupType = "sfacross")}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)
```

## Overview

When observed group labels are available (e.g., farm size, region, ownership type),
`groupType = "sfacross"` fits a separate cross-sectional stochastic frontier for each
group using `sfaR::sfacross()`. The group-specific results are then used to estimate the
common metafrontier using any of four methods.

## Data Preparation

We use the `ricephil` dataset from `sfaR`, which contains 344 Filipino rice farms.
We create three technology groups based on farm area terciles.

```{r data}
library(smfa)
data("ricephil", package = "sfaR")

ricephil$group <- cut(
  ricephil$AREA,
  breaks        = quantile(ricephil$AREA, probs = c(0, 1/3, 2/3, 1), na.rm = TRUE),
  labels        = c("small", "medium", "large"),
  include.lowest = TRUE
)

table(ricephil$group)
#>  small medium  large
#>    125    104    115
```

## Method 1: LP Metafrontier

The **linear programming (LP)** envelope minimises the sum of absolute deviations from
group frontier predictions while satisfying a convexity constraint. No stochastic
parameters are estimated for the metafrontier itself.

```{r lp}
meta_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)
summary(meta_lp)
```

> **Note:** Since the LP metafrontier is estimated via linear programming, no estimated
> parameters are returned for the metafrontier level. The LP envelope is fully determined
> by the group frontier predictions.

## Method 2: QP Metafrontier

The **quadratic programming (QP)** envelope minimises the sum of *squared* deviations
from group frontier predictions. Unlike LP, QP produces a smooth envelope that is
differentiable everywhere, and it returns estimated coefficients with standard errors.

```{r qp}
meta_qp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "qp"
)
summary(meta_qp)
```

## Method 3: Stochastic Metafrontier — Huang et al. (2014)

The **two-stage stochastic metafrontier** of Huang, Huang & Liu (2014) uses the
group-specific *fitted frontier values* as the dependent variable in a second-stage
pooled SFA. The technology gap U and noise V are estimated stochastically, which
naturally bounds the metatechnology ratio MTR ∈ (0, 1].

```{r huang}
meta_huang <- smfa(
  formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data        = ricephil,
  group       = "group",
  S           = 1,
  udist       = "hnormal",
  groupType   = "sfacross",
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_huang)
```

## Method 4: Stochastic Envelope — O'Donnell et al. (2008)

The **O'Donnell et al. (2008)** approach uses the LP deterministic envelope as the
dependent variable in the second-stage SFA, rather than the group-specific fitted values.
This mixed deterministic–stochastic approach embeds the envelope within an SFA framework.

> **Warning:** MTR values > 1 can arise with this method when the second-stage SFA
> estimates near-zero inefficiency. If this occurs, consider using `metaMethod = "lp"` or
> `sfaApproach = "huang"` instead.

```{r odonnell}
meta_ordonnell <- smfa(
  formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data        = ricephil,
  group       = "group",
  S           = 1,
  udist       = "hnormal",
  groupType   = "sfacross",
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_ordonnell)
```

## Comparing Methods

All four methods use identical group-level estimates. The differences arise only in how the
metafrontier is computed:

| Method | Metafrontier Coefficients Returned | MTR Bounded ≤ 1? |
|--------|------------------------------------|------------------|
| LP | No (envelope rule) | Yes |
| QP | Yes (with SE) | Yes |
| SFA (huang) | Yes (with SE) | Yes |
| SFA (ordonnell) | Yes (with SE) | Not guaranteed |

## Extracting Efficiencies

All models return firm-level efficiency estimates via `efficiencies()`:

```{r eff}
eff <- efficiencies(meta_lp)
head(eff)

# Subset for a specific group
eff_small <- eff[eff$group == "small", ]
summary(eff_small[, c("TE_group_BC", "TE_meta_BC", "MTR_BC")])
```

## Other Extractors

```{r other}
coef(meta_qp)          # metafrontier coefficients
vcov(meta_qp)          # variance-covariance matrix
logLik(meta_lp)        # log-likelihood
ic(meta_lp)            # AIC, BIC, HQIC
nobs(meta_lp)          # number of observations
fitted(meta_lp)        # fitted values
residuals(meta_lp)     # residuals
```

---
title: "Getting Started with smfa"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with smfa}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)
```

## What is smfa?

`smfa` is an R package for implementing **metafrontier analysis** — a framework for productivity and performance benchmarking of firms (or farms, utilities, hospitals, etc.) that operate under *different* technologies. This is also referred to as "heterogeneous technology" analysis.

The classic **stochastic frontier analysis (SFA)** estimates a single frontier for all firms. Metafrontier analysis extends this by:

1. Estimating **group-specific frontiers** for each technology sub-group.
2. Estimating a common **metafrontier** that envelops all group frontiers.
3. Computing **metatechnology ratios (MTR)**, which quantify how far each group's frontier lies below the best-practice metafrontier.

This allows researchers to disentangle:

- **Technical efficiency** relative to the group's own frontier (TE_group).
- **Metafrontier efficiency** relative to the common best-practice frontier (TE_meta).
- **Metatechnology ratio** (MTR = TE_meta / TE_group), which captures how technologically advanced a group's production possibilities are.

## Conceptual Framework

$$\text{MTR}_i = \frac{\text{TE\_meta}_i}{\text{TE\_group}_i}$$

A **MTR close to 1** means the group's technology is near the metafrontier (advanced technology). A **MTR far below 1** means the group operates under a less advanced technology.

## Methods Supported

`smfa` supports four metafrontier estimation methods (`metaMethod`):

| Method | Description |
|--------|-------------|
| `"lp"` | Linear programming deterministic envelope — Battese, Rao & O'Donnell (2004) |
| `"qp"` | Quadratic programming deterministic envelope |
| `"sfa"` (`sfaApproach = "huang"`) | Two-stage stochastic metafrontier — Huang, Huang & Liu (2014) |
| `"sfa"` (`sfaApproach = "ordonnell"`) | Two-stage SFA on LP envelope — O'Donnell, Rao & Battese (2008) |

And three group frontier types (`groupType`):

| Group Type | When to Use |
|------------|-------------|
| `"sfacross"` | Technology groups are *observed* (e.g., a group variable exists) |
| `"sfalcmcross"` | Technology groups are *unobserved* — latent class model identifies them (Dakpo et al. 2021) |
| `"sfaselectioncross"` | Sample selection bias is present (Dakpo et al. 2022) |

## Installation

```{r install, eval = FALSE}
# Install smfa from CRAN
install.packages("smfa")

# Or install the development version from GitHub
# if (!require("devtools")) install.packages("devtools")
# devtools::install_github("SulmanOlieko/smfa")
```

> **Note:** `sfaR` is automatically installed as a dependency — you do not need to install it separately.

## Quick-Start Example

This minimal example demonstrates the core workflow using the `ricephil` dataset from the `sfaR` package, with three Filipino rice farm-size groups (small, medium, large).

### Step 1: Load data and create groups

```{r quickstart-data}
library(smfa)
data("ricephil", package = "sfaR")

# Create technology groups based on farm area terciles
ricephil$group <- cut(
  ricephil$AREA,
  breaks = quantile(ricephil$AREA, probs = c(0, 1 / 3, 2 / 3, 1), na.rm = TRUE),
  labels = c("small", "medium", "large"),
  include.lowest = TRUE
)
table(ricephil$group)
```

### Step 2: Fit the metafrontier model

```{r quickstart-fit}
meta_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1, # production frontier (S=1) or cost frontier (S=-1)
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)
```

### Step 3: Summarise results

```{r quickstart-summary}
summary(meta_lp)
```

### Step 4: Extract firm-level efficiencies

```{r quickstart-eff}
eff <- efficiencies(meta_lp)
head(eff)
```

Key output columns:

| Column | Description |
|--------|-------------|
| `TE_group_BC` | Group TE (Battese & Coelli 1988) |
| `TE_meta_BC` | Metafrontier TE |
| `MTR_BC` | Metatechnology ratio |

## What Next?

- **Standard SFA groups** → see `vignette("sfacross-metafrontier")`
- **Unobserved/latent groups** → see `vignette("sfalcmcross-metafrontier")`
- **Sample selection** → see `vignette("sfaselectioncross-metafrontier")`
- **Extracting all outputs** → see `vignette("efficiency-extraction")`

---
title: "Extracting Efficiencies and MTRs"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Extracting Efficiencies and MTRs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)
```

## Overview

Once a `metafrontier` model has been fitted, a range of S3 methods are available to
extract and inspect results. This vignette demonstrates all of them using a simple
`sfacross` + LP example.

## Setup: Fit a Model

```{r fit}
library(smfa)
data("ricephil", package = "sfaR")

ricephil$group <- cut(
  ricephil$AREA,
  breaks        = quantile(ricephil$AREA, probs = c(0, 1/3, 2/3, 1), na.rm = TRUE),
  labels        = c("small", "medium", "large"),
  include.lowest = TRUE
)

meta_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)
```

## `summary()` — Full Model Summary

Prints the complete model output including group-specific SFA results, metafrontier
coefficients (if available), and efficiency statistics by group.

```{r summary}
summary(meta_lp)
```

## `efficiencies()` — Firm-Level Efficiency and MTR Scores

Returns a data frame with one row per observation containing all efficiency estimates and
metatechnology ratios. All estimators are available for all `groupType` values, though
the exact columns vary slightly by model type.

```{r efficiencies}
eff <- efficiencies(meta_lp)
head(eff)
```

### Column Reference

| Column | `sfacross` | `sfalcmcross` | `sfaselectioncross` |
|--------|:----------:|:-------------:|:-------------------:|
| `id` | ✓ | ✓ | ✓ |
| `group` / `Group_c` | ✓ | ✓ | ✓ |
| `u_g` | ✓ | ✓ | ✓ |
| `TE_group_JLMS` | ✓ | ✓ | ✓ |
| `TE_group_BC` | ✓ | ✓ | ✓ |
| `TE_group_BC_reciprocal` | ✓ | ✓ | ✓ |
| `uLB_g`, `uUB_g` | ✓ | — | — |
| `m_g`, `TE_group_mode` | ✓ | — | — |
| `PosteriorProb_c`, `PosteriorProb_c1` … | — | ✓ | — |
| `u_meta` | ✓ | ✓ | ✓ |
| `TE_meta_JLMS` | ✓ | ✓ | ✓ |
| `TE_meta_BC` | ✓ | ✓ | ✓ |
| `MTR_JLMS` | ✓ | ✓ | ✓ |
| `MTR_BC` | ✓ | ✓ | ✓ |

### Subsetting by Group

```{r subset}
# All small farms
eff_small <- eff[eff$group == "small", ]

# Descriptive statistics
summary(eff_small[, c("TE_group_BC", "TE_meta_BC", "MTR_BC")])

# Group-level means
aggregate(cbind(TE_group_BC, TE_meta_BC, MTR_BC) ~ group, data = eff, FUN = mean)
```

### Visualising the Distribution

```{r plot}
# MTR distribution by group (base R)
boxplot(MTR_BC ~ group, data = eff,
        main = "Metatechnology Ratio by Farm Size",
        xlab = "Group", ylab = "MTR (BC)",
        col  = c("#e8f5ef", "#b2dfdb", "#80cbc4"))

# Histogram of TE_meta_BC
hist(eff$TE_meta_BC, breaks = 30,
     main = "Distribution of Metafrontier TE",
     xlab = "TE_meta_BC", col = "#2d8f65", border = "white")
```

## `coef()` — Estimated Coefficients

Returns the metafrontier coefficient vector (for QP and SFA methods; `NULL` for LP).

```{r coef}
# First fit a QP model
meta_qp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil, group = "group", S = 1, udist = "hnormal",
  groupType  = "sfacross", metaMethod = "qp"
)
coef(meta_qp)
```

## `vcov()` — Variance-Covariance Matrix

Returns the variance-covariance matrix of the metafrontier coefficients (for models that
estimate metafrontier parameters).

```{r vcov}
vcov(meta_qp)
```

## `logLik()` — Log-Likelihood

Returns the total log-likelihood value of the model (sum of group-level log-likelihoods
plus the metafrontier log-likelihood where applicable).

```{r loglik}
logLik(meta_lp)
```

## `ic()` — Information Criteria

Returns all three information criteria: AIC, BIC, and HQIC.

```{r ic}
ic(meta_lp)
#>        AIC       BIC      HQIC
#> 1  184.579  253.710   212.113
```

## `nobs()` — Number of Observations

```{r nobs}
nobs(meta_lp)  # Total observations across all groups
```

## `fitted()` — Fitted Values

Returns the fitted frontier values from the model.

```{r fitted}
fv <- fitted(meta_lp)
head(fv)
```

## `residuals()` — Residuals

Returns the composite error residuals from the group-level stochastic frontier models.

```{r residuals}
res <- residuals(meta_lp)
head(res)
```

## Comparing Multiple Models

You can compare information criteria across methods to select the best model:

```{r compare}
meta_lp    <- smfa(log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
                               data = ricephil, group = "group", S = 1,
                               udist = "hnormal", groupType = "sfacross",
                               metaMethod = "lp")
meta_qp    <- smfa(log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
                               data = ricephil, group = "group", S = 1,
                               udist = "hnormal", groupType = "sfacross",
                               metaMethod = "qp")
meta_huang <- smfa(log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
                               data = ricephil, group = "group", S = 1,
                               udist = "hnormal", groupType = "sfacross",
                               metaMethod = "sfa", sfaApproach = "huang")

# Combined information criteria table
models <- list(LP = meta_lp, QP = meta_qp, Huang = meta_huang)
do.call(rbind, lapply(names(models), function(nm) {
  ic_vals <- ic(models[[nm]])
  data.frame(Model = nm, AIC = ic_vals[["AIC"]],
             BIC = ic_vals[["BIC"]], HQIC = ic_vals[["HQIC"]])
}))
```

---
title: "Sample Selection SFA Metafrontier (groupType = \"sfaselectioncross\")"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Sample Selection SFA Metafrontier (groupType = "sfaselectioncross")}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE
)
```

## Overview

**Sample selection bias** arises when the observed sample is not a random draw from the
population. For example:

- Only firms above a revenue threshold are surveyed.
- Only farms that adopted a technology are observed using it.
- Participation in a programme is voluntary, so only volunteers are observed.

If selection into the sample is correlated with firm efficiency, ignoring this leads to
biased frontier estimates. `sfaselectioncross` implements the **two-step ML estimator of
Greene (2010)**, leveraging the sample selection correction provided via `sfaR`
(Dakpo et al. 2022), which corrects for this bias using a probit selection equation
(Heckman 1979 correction).

The selection model requires:
- A **binary selection indicator** `d` (1 = selected/observed, 0 = not selected).
- A **selection equation formula** `selectionF` specifying which variables drive selection.
  At least one variable must appear in `selectionF` but not in the main frontier formula.
- Only selected observations (`d == 1`) participate in the frontier and receive efficiency
  estimates. Efficiency for non-selected observations is `NA`.

## Data Preparation (Simulated Example)

We simulate data following the approach in the `sfaR` documentation:

```{r data}
library(smfa)

N <- 500; set.seed(12345)
z1 <- rnorm(N); z2 <- rnorm(N)
v1 <- rnorm(N); v2 <- rnorm(N)
g  <- rnorm(N)
e1 <- v1
e2 <- 0.7071 * (v1 + v2)
ds <- z1 + z2 + e1
d  <- ifelse(ds > 0, 1, 0)        # 1 = selected into the sample
group <- ifelse(g > 0, 1, 0)      # two technology groups
u  <- abs(rnorm(N))
x1 <- abs(rnorm(N)); x2 <- abs(rnorm(N))
y  <- abs(x1 + x2 + e2 - u)
dat <- as.data.frame(cbind(y = y, x1 = x1, x2 = x2,
                            z1 = z1, z2 = z2, d = d, group = group))

# About 50% of observations are selected
table(dat$d)
#>    0    1
#> 1013  987
```

## Method 1: sfaselectioncross + LP Metafrontier

```{r lp}
meta_sel_lp <- smfa(
  formula    = log(y) ~ log(x1) + log(x2),
  selectionF = d ~ z1 + z2,      # selection equation: d is the binary indicator
  data       = dat,
  group      = "group",
  S          = 1L,
  udist      = "hnormal",
  groupType  = "sfaselectioncross",
  modelType  = "greene10",        # Greene (2010) two-step ML correction
  lType      = "kronrod",         # integration method for the selection likelihood
  Nsub       = 20,               # number of sub-intervals for numerical integration
  uBound     = Inf,
  method     = "bfgs",
  itermax    = 2000,
  metaMethod = "lp"
)
summary(meta_sel_lp)
```

> **Note:** The `selectionF` argument is compulsory for `groupType = "sfaselectioncross"`.
> The left-hand side must be the binary selection variable (`d`). At least one regressor in
> the selection equation should *not* appear in the main frontier formula (exclusion
> restriction for identification).

## Method 2: sfaselectioncross + QP Metafrontier

```{r qp}
meta_sel_qp <- smfa(
  formula    = log(y) ~ log(x1) + log(x2),
  selectionF = d ~ z1 + z2,
  data       = dat,
  group      = "group",
  S          = 1L,
  udist      = "hnormal",
  groupType  = "sfaselectioncross",
  modelType  = "greene10",
  lType      = "kronrod",
  Nsub       = 20,
  uBound     = Inf,
  method     = "bfgs",
  itermax    = 2000,
  metaMethod = "qp"
)
summary(meta_sel_qp)
```

## Method 3: sfaselectioncross + SFA (Huang)

```{r huang}
meta_sel_huang <- smfa(
  formula     = log(y) ~ log(x1) + log(x2),
  selectionF  = d ~ z1 + z2,
  data        = dat,
  group       = "group",
  S           = 1L,
  udist       = "hnormal",
  groupType   = "sfaselectioncross",
  modelType   = "greene10",
  lType       = "kronrod",
  Nsub        = 100,
  uBound      = Inf,
  method      = "bfgs",
  itermax     = 2000,
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_sel_huang)
```

## Method 4: sfaselectioncross + SFA (O'Donnell)

```{r odonnell}
meta_sel_odonnell <- smfa(
  formula     = log(y) ~ log(x1) + log(x2),
  selectionF  = d ~ z1 + z2,
  data        = dat,
  group       = "group",
  S           = 1L,
  udist       = "hnormal",
  groupType   = "sfaselectioncross",
  modelType   = "greene10",
  lType       = "kronrod",
  Nsub        = 100,
  uBound      = Inf,
  method      = "bfgs",
  itermax     = 2000,
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_sel_odonnell)
```

## Interpreting the Selection Correction

The first-stage probit model estimates the selection probability. The key additional
parameter in the frontier model is `rho` — the correlation between the selection equation
error and the frontier equation noise.

```{r rho}
# The rho parameter appears in the summary output:
# ----------------------------------------------------------------
#              Selection bias parameter
# ----------------------------------------------------------------
#           Coefficient Std. Error z value  Pr(>|z|)
# rho          0.89550    0.28696  3.1207  0.001804 **

# A significant rho indicates selection bias IS present and the
# correction is important.
```

| `rho` value | Interpretation |
|-------------|----------------|
| ≈ 0, p > 0.05 | No significant selection bias; standard SFA may be sufficient |
| > 0, p < 0.05 | Positive selection — efficient firms are more likely selected |
| < 0, p < 0.05 | Negative selection — inefficient firms are more likely selected |

## Extracting Efficiencies

Only selected observations (those with `d == 1`) receive efficiency estimates:

```{r eff}
eff_sel <- efficiencies(meta_sel_lp)

# Non-selected observations have NA efficiencies
sum(is.na(eff_sel$TE_group_BC))   # count of non-selected obs

# Subset for selected observations in group 1
sel_grp1 <- eff_sel[eff_sel$group == 1 & !is.na(eff_sel$TE_group_BC), ]
summary(sel_grp1[, c("TE_group_BC", "TE_meta_BC", "MTR_BC")])
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nobs.R
\name{nobs}
\alias{nobs}
\alias{nobs.smfa}
\title{Extract total number of observations used in frontier models}
\usage{
\method{nobs}{smfa}(object, ...)
}
\arguments{
\item{object}{a \code{smfa}
object for which the number of total observations is to be extracted.}

\item{\dots}{Currently ignored.}
}
\value{
A single number, normally an integer.
}
\description{
This function extracts the total number of 'observations' from a
fitted point frontier model.
}
\details{
\code{nobs} gives the number of observations actually
used by the estimation procedure.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data
}
\keyword{attribute}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smfa.R
\name{smfa}
\alias{smfa}
\alias{print.smfa}
\title{Stochastic metafrontier estimation}
\usage{
smfa(
  formula,
  muhet,
  uhet,
  vhet,
  thet,
  logDepVar = TRUE,
  data,
  subset,
  weights,
  wscale = TRUE,
  group = NULL,
  S = 1L,
  udist = "hnormal",
  start = NULL,
  scaling = FALSE,
  modelType = "greene10",
  groupType = "sfacross",
  metaMethod = "lp",
  sfaApproach = "ordonnell",
  selectionF = NULL,
  lcmClasses = 2L,
  whichStart = 2L,
  initAlg = "nm",
  initIter = 100L,
  lType = "ghermite",
  Nsub = 100L,
  uBound = Inf,
  intol = 1e-06,
  method = "bfgs",
  hessianType = NULL,
  simType = "halton",
  Nsim = 100L,
  prime = 2L,
  burn = 10L,
  antithetics = FALSE,
  seed = 12345L,
  itermax = 2000L,
  printInfo = FALSE,
  tol = 1e-12,
  gradtol = 1e-06,
  stepmax = 0.1,
  qac = "marquardt",
  ...
)

\method{print}{smfa}(x, ...)
}
\arguments{
\item{formula}{A symbolic description of the frontier model to be estimated,
based on the generic function \code{\link[stats]{formula}}. For
\code{groupType = "sfaselectioncross"}, this argument specifies the
frontier (outcome) equation and must be a standard formula whose left-hand
side is the output (or cost) variable and whose right-hand side contains
the frontier regressors (see also \code{selectionF}).}

\item{muhet}{A one-part formula to account for heterogeneity in the mean of
the pre-truncated normal distribution. Applicable only when
\code{groupType = "sfacross"} and \code{udist = "tnormal"}. The variables
specified model the conditional mean
\eqn{\mu_i = \bm{\omega}'\mathbf{Z}_{\mu}} of the truncated normal
inefficiency distribution (see section \sQuote{Details}).}

\item{uhet}{A one-part formula to account for heteroscedasticity in the
one-sided error variance. Applicable for all three model types. The
variance of the inefficiency term is modelled as
\eqn{\sigma^2_u = \exp(\bm{\delta}'\mathbf{Z}_u)}, where
\eqn{\mathbf{Z}_u} are the inefficiency drivers and \eqn{\bm{\delta}}
the associated coefficients (see section \sQuote{Details}).}

\item{vhet}{A one-part formula to account for heteroscedasticity in the
two-sided error variance. Applicable for all three model types. The
variance of the noise term is modelled as
\eqn{\sigma^2_v = \exp(\bm{\phi}'\mathbf{Z}_v)}, where
\eqn{\mathbf{Z}_v} are the heteroscedasticity variables and
\eqn{\bm{\phi}} the coefficients (see section \sQuote{Details}).}

\item{thet}{A one-part formula to account for technological heterogeneity
in the construction of the latent classes. Applicable only when
\code{groupType = "sfalcmcross"}. The variables specified enter the logit
formulation that determines the prior class membership probabilities
\eqn{\pi(i,j)} (see section \sQuote{Details}).}

\item{logDepVar}{Logical. Informs whether the dependent variable is logged
(\code{TRUE}) or not (\code{FALSE}). Default \code{TRUE}. Must match the
transformation applied to the left-hand side of \code{formula}.}

\item{data}{A data frame containing all variables referenced in
\code{formula}, \code{selectionF}, \code{muhet}, \code{uhet}, \code{vhet},
\code{thet}, and \code{group}.}

\item{subset}{An optional vector specifying a subset of observations to be
used in the estimation process.}

\item{weights}{An optional vector of weights to be used for weighted
log-likelihood estimation. Should be \code{NULL} or a numeric vector with
strictly positive values. When \code{NULL} (default), all observations
receive equal weight.}

\item{wscale}{Logical. When \code{weights} is not \code{NULL}, a scaling
transformation is applied such that the weights sum to the sample size:
\deqn{w_{\mathrm{new}} = n \times
    \frac{w_{\mathrm{old}}}{\sum w_{\mathrm{old}}}}
Default \code{TRUE}. When \code{FALSE}, the raw weights are used without
scaling.}

\item{group}{Character string. The name of the column in \code{data}
identifying the technology group of each observation. The column is coerced
to a factor internally and must have at least two unique values. When
\code{groupType = "sfalcmcross"} and \code{group} is \code{NULL}, a single
pooled latent class model is estimated and class assignments serve as groups
(see section \sQuote{Details}).}

\item{S}{Integer. Frontier orientation.
\itemize{
\item \code{S = 1} (default): production or profit frontier,
\eqn{\varepsilon_i = v_i - u_i}.
\item \code{S = -1}: cost frontier,
\eqn{\varepsilon_i = v_i + u_i}.
}}

\item{udist}{Character string. Distribution for the one-sided error term
\eqn{u_i \ge 0}. The following distributions are available for
\code{groupType = "sfacross"}:
\itemize{
\item \code{"hnormal"} (default): half-normal distribution
(Aigner \emph{et al.}, 1977; Meeusen and van den Broeck, 1977).
\item \code{"exponential"}: exponential distribution.
\item \code{"tnormal"}: truncated normal distribution (Stevenson, 1980).
\item \code{"rayleigh"}: Rayleigh distribution (Hajargasht, 2015).
\item \code{"uniform"}: uniform distribution (Li, 1996; Nguyen, 2010).
\item \code{"gamma"}: Gamma distribution, estimated by maximum simulated
likelihood (Greene, 2003).
\item \code{"lognormal"}: log-normal distribution, estimated by maximum
simulated likelihood (Migon and Medici, 2001; Wang and Ye, 2020).
\item \code{"weibull"}: Weibull distribution, estimated by maximum
simulated likelihood (Tsionas, 2007).
\item \code{"genexponential"}: generalised exponential distribution
(Papadopoulos, 2020).
\item \code{"tslaplace"}: truncated skewed Laplace distribution
(Wang, 2012).
}
For \code{groupType = "sfaselectioncross"} and \code{"sfalcmcross"}, only
\code{"hnormal"} is currently supported.}

\item{start}{Numeric vector. Optional starting values for the maximum
likelihood (ML) or maximum simulated likelihood (MSL) estimation of the
group-level frontier models. When \code{NULL} (default), starting values
are computed automatically. For \code{groupType = "sfacross"}, they are
derived from OLS residuals. For \code{groupType = "sfalcmcross"}, they
depend on \code{whichStart}.}

\item{scaling}{Logical. Applicable only when \code{groupType = "sfacross"}
and \code{udist = "tnormal"}. When \code{TRUE}, the scaling property model
(Wang and Schmidt, 2002) is estimated, whereby
\eqn{u_i = h(\mathbf{Z}_u, \bm{\delta}) u^*_i} and \eqn{u^*_i} follows
a truncated normal distribution \eqn{N^+(\tau, \exp(c_u))}. Default
\code{FALSE}.}

\item{modelType}{Character string. Applicable only when
\code{groupType = "sfaselectioncross"}. Specifies the model used to correct
for selection bias. Currently, only \code{"greene10"} (default) is
supported, corresponding to the two-step approach of Greene (2010): a
probit model is estimated for the selection equation, and its inverse Mills
ratio is included as a correction term in the stochastic frontier second
step.}

\item{groupType}{Character string. Type of frontier model estimated for each
technology group. Three options are available:
\itemize{
\item \code{"sfacross"} (default): standard cross-sectional stochastic
frontier analysis (\code{\link[sfaR]{sfacross}}). Groups are defined
by the \code{group} variable. All 10 distributions for \code{udist}
are supported, along with heteroscedasticity in both error components
(\code{uhet}, \code{vhet}), heterogeneity in the truncated mean
(\code{muhet}), and the scaling property.
\item \code{"sfaselectioncross"}: sample selection stochastic frontier
analysis (\code{\link[sfaR]{sfaselectioncross}}). Corrects for sample
selection bias via the generalised Heckman approach (Greene, 2010).
Requires \code{selectionF}. Only observations for which the selection
indicator equals one enter the frontier and metafrontier; efficiency
estimates for non-selected observations are \code{NA}. Only
\code{udist = "hnormal"} is supported.
\item \code{"sfalcmcross"}: latent class stochastic frontier analysis
(\code{\link[sfaR]{sfalcmcross}}). Estimates a finite mixture of
frontier models with the number of classes determined by
\code{lcmClasses}. When \code{group} is supplied, a separate latent
class model is estimated per group-stratum and combined for the
metafrontier. When \code{group} is omitted, a single pooled model is
estimated and class assignments serve as technology groups. Supports
\code{thet} for class-membership covariates and \code{uhet},
\code{vhet} for within-class heteroscedasticity. Only
\code{udist = "hnormal"} is supported.
}}

\item{metaMethod}{Character string. Method for estimating the global
metafrontier that envelopes all group frontiers. Three options are
available:
\itemize{
\item \code{"lp"} (default): deterministic linear programming envelope.
Finds the parameter vector \eqn{\bm{\beta}^*} minimising
\eqn{\sum_i |\ln \hat{f}(x_i, \bm{\beta}^*) - \ln \hat{f}(x_i,
      \hat{\bm{\beta}}_{(g)})|}
subject to \eqn{\ln \hat{f}(x_i, \bm{\beta}^*) \ge \ln \hat{f}(x_i,
      \hat{\bm{\beta}}_{(g)})} for all observations and all groups
(Battese \emph{et al.}, 2004).
\item \code{"qp"}: deterministic quadratic programming envelope.
Minimises the sum of squared deviations under the same envelope
constraint.
\item \code{"sfa"}: stochastic metafrontier estimated by a second-stage
pooled SFA. The specific construction of the dependent variable is
determined by \code{sfaApproach}.
}}

\item{sfaApproach}{Character string. Applicable only when
\code{metaMethod = "sfa"}. Determines how the second-stage SFA is
constructed:
\itemize{
\item \code{"ordonnell"} (default): The LP envelope of the group frontier
predicted values is re-estimated with a stochastic frontier, following
O'Donnell, Rao, and Battese (2008). The second-stage SFA directly
targets the global technology envelope.
\item \code{"huang"}: the group-specific fitted frontier value
\eqn{\ln \hat{y}^g_i} for each observation is used as the dependent
variable in a pooled cross-sectional SFA
(Huang, Huang, and Liu, 2014). The technology gap \eqn{U_i \ge 0} and
second-stage noise \eqn{V_i} are estimated directly by the SFA
procedure.
\item \code{"ordonnell"}: the column-wise maximum of all group-fitted
frontier values (the deterministic LP envelope) is used as the dependent
variable in the second-stage SFA (O'Donnell, Rao, and Battese, 2008).
}}

\item{selectionF}{A two-sided formula specifying the sample selection
equation, e.g., \code{selected ~ z1 + z2}. The left-hand side must be a
binary (0/1) indicator already present in \code{data}: \code{1} means the
observation participates in the frontier and metafrontier; \code{0} means
it is excluded (efficiency estimates will be \code{NA}). Alternatively, a
named list of formulas, one per group level, may be supplied to allow
group-specific selection equations. Required when
\code{groupType = "sfaselectioncross"}; ignored otherwise.}

\item{lcmClasses}{Integer. Number of latent classes to be estimated per group
when \code{groupType = "sfalcmcross"}. Must be between \code{2} and
\code{5} (default \code{2}). The optimal number of classes can be selected
based on information criteria (see \code{\link[sfaR]{ic}}).}

\item{whichStart}{Integer. Strategy for obtaining starting values in the
latent class model (\code{groupType = "sfalcmcross"}):
\itemize{
\item \code{1}: starting values are obtained from the method of moments.
\item \code{2} (default): the model is initialised by first solving a
homoscedastic pooled cross-sectional SFA using the algorithm specified
by \code{initAlg} for at most \code{initIter} iterations.
}}

\item{initAlg}{Character string. Optimisation algorithm used during the
initialisation of the latent class model when \code{whichStart = 2}.
Only algorithms from the \code{maxLik} package are supported:
\itemize{
\item \code{"nm"} (default): Nelder-Mead (see
\code{\link[maxLik]{maxNM}}).
\item \code{"bfgs"}: Broyden-Fletcher-Goldfarb-Shanno (see
\code{\link[maxLik]{maxBFGS}}).
\item \code{"bhhh"}: Berndt-Hall-Hall-Hausman (see
\code{\link[maxLik]{maxBHHH}}).
\item \code{"nr"}: Newton-Raphson (see \code{\link[maxLik]{maxNR}}).
\item \code{"cg"}: Conjugate Gradient (see
\code{\link[maxLik]{maxCG}}).
\item \code{"sann"}: Simulated Annealing (see
\code{\link[maxLik]{maxSANN}}).
}}

\item{initIter}{Integer. Maximum number of iterations for the initialisation
algorithm when \code{whichStart = 2} and
\code{groupType = "sfalcmcross"}. Default \code{100}.}

\item{lType}{Character string. Specifies how the likelihood is evaluated for
the selection model (\code{groupType = "sfaselectioncross"}). Five options
are available:
\itemize{
\item \code{"ghermite"} (default): Gauss-Hermite quadrature (see
\code{\link[fastGHQuad]{gaussHermiteData}}).
\item \code{"kronrod"}: Gauss-Kronrod quadrature (see
\code{\link[stats]{integrate}}).
\item \code{"hcubature"}: adaptive integration over hypercubes (see
\code{\link[cubature]{hcubature}}).
\item \code{"pcubature"}: p-adaptive cubature (see
\code{\link[cubature]{pcubature}}).
\item \code{"msl"}: maximum simulated likelihood (controlled by
\code{simType}, \code{Nsim}, \code{prime}, \code{burn},
\code{antithetics}, and \code{seed}).
}}

\item{Nsub}{Integer. Number of quadrature nodes or integration subdivisions
when \code{lType} is \code{"ghermite"}, \code{"kronrod"},
\code{"hcubature"}, or \code{"pcubature"}. Applicable only when
\code{groupType = "sfaselectioncross"}. Default \code{100}.}

\item{uBound}{Numeric. Upper bound for the numerical integration of the
inefficiency component when \code{lType} is \code{"kronrod"},
\code{"hcubature"}, or \code{"pcubature"}. For Gauss-Hermite the bound is
automatically infinite. Applicable only when
\code{groupType = "sfaselectioncross"}. Default \code{Inf}.}

\item{intol}{Numeric. Integration tolerance for the quadrature approaches
\code{"kronrod"}, \code{"hcubature"}, and \code{"pcubature"}. Applicable
only when \code{groupType = "sfaselectioncross"}. Default \code{1e-6}.}

\item{method}{Character string. Optimisation algorithm for the main ML/MSL
estimation of each group-level frontier model. Default \code{"bfgs"}.
Eleven algorithms are available:
\itemize{
\item \code{"bfgs"}: Broyden-Fletcher-Goldfarb-Shanno (see
\code{\link[maxLik]{maxBFGS}}).
\item \code{"bhhh"}: Berndt-Hall-Hall-Hausman (see
\code{\link[maxLik]{maxBHHH}}).
\item \code{"nr"}: Newton-Raphson (see \code{\link[maxLik]{maxNR}}).
\item \code{"nm"}: Nelder-Mead (see \code{\link[maxLik]{maxNM}}).
\item \code{"cg"}: Conjugate Gradient (see
\code{\link[maxLik]{maxCG}}).
\item \code{"sann"}: Simulated Annealing (see
\code{\link[maxLik]{maxSANN}}).
\item \code{"ucminf"}: quasi-Newton optimisation with BFGS updating of
the inverse Hessian and soft line search (see
\code{\link[ucminf]{ucminf}}).
\item \code{"mla"}: Marquardt-Levenberg algorithm (see
\code{\link[marqLevAlg]{mla}}).
\item \code{"sr1"}: Symmetric Rank 1 trust-region method (see
\code{\link[trustOptim]{trust.optim}}).
\item \code{"sparse"}: trust-region method with sparse Hessian (see
\code{\link[trustOptim]{trust.optim}}).
\item \code{"nlminb"}: PORT routines optimisation (see
\code{\link[stats]{nlminb}}).
}}

\item{hessianType}{Integer. Specifies which Hessian is returned for the
group-level frontier estimation. The accepted values match those of the
underlying \code{sfaR} function for each \code{groupType}:
\itemize{
\item For \code{groupType = "sfacross"}: if \code{1} (default), the
analytic Hessian is returned; if \code{2}, the BHHH Hessian
\eqn{\mathbf{G}'\mathbf{G}} is estimated.
\item For \code{groupType = "sfalcmcross"}: if \code{1} (default),
the analytic Hessian is returned; if \code{2}, the BHHH Hessian is
estimated.
\item For \code{groupType = "sfaselectioncross"}: if \code{1}, the
analytic Hessian is returned; if \code{2} (default), the BHHH
Hessian \eqn{\mathbf{G}'\mathbf{G}} is estimated. The BHHH default
reflects the two-step nature of the selection estimator.
}
When \code{NULL} (the package default), each group-level model uses the
natural default of the corresponding \code{sfaR} function, ensuring that
standard errors computed by \code{smfa} are identical to those
from a standalone \code{sfaR} call on the same group subset.}

\item{simType}{Character string. Simulation method for maximum simulated
likelihood (MSL). Applicable to \code{groupType = "sfacross"} when
\code{udist} is \code{"gamma"}, \code{"lognormal"}, or \code{"weibull"},
and to \code{groupType = "sfaselectioncross"} when \code{lType = "msl"}:
\itemize{
\item \code{"halton"} (default): Halton quasi-random sequences.
\item \code{"ghalton"}: Generalised-Halton sequences.
\item \code{"sobol"}: Sobol low-discrepancy sequences.
\item \code{"uniform"}: pseudo-random uniform draws.
}}

\item{Nsim}{Integer. Number of simulation draws for MSL. Default \code{100}.}

\item{prime}{Integer. Prime number used to construct Halton or
Generalised-Halton sequences. Default \code{2}.}

\item{burn}{Integer. Number of leading draws discarded from the Halton
sequence to reduce serial correlation. Default \code{10}.}

\item{antithetics}{Logical. If \code{TRUE}, antithetic draws are added: the
first \code{Nsim/2} draws are taken, and the remaining \code{Nsim/2} are
\eqn{1 - \text{draw}}. Default \code{FALSE}.}

\item{seed}{Integer. Random seed for simulation draws, ensuring
reproducibility of MSL estimates. Default \code{12345}.}

\item{itermax}{Integer. Maximum number of iterations for the main
optimisation. Default \code{2000}. For \code{method = "sann"}, it is
recommended to increase this substantially (e.g., \code{itermax = 20000}).}

\item{printInfo}{Logical. If \code{TRUE}, optimisation progress is printed
during estimation of each group-level model. Default \code{FALSE}.}

\item{tol}{Numeric. Convergence tolerance. The algorithm is considered
converged when the change in the log-likelihood between successive
iterations is smaller than \code{tol} in absolute value. Default
\code{1e-12}.}

\item{gradtol}{Numeric. Gradient convergence tolerance. The algorithm is
considered converged when the Euclidean norm of the gradient is smaller
than \code{gradtol}. Default \code{1e-6}.}

\item{stepmax}{Numeric. Maximum step length used by the \code{"ucminf"}
algorithm. Default \code{0.1}.}

\item{qac}{Character string. Quadratic Approximation Correction for the
\code{"bhhh"} and \code{"nr"} algorithms when the Hessian is not negative
definite:
\itemize{
\item \code{"marquardt"} (default): step length is decreased while
also shifting closer to the gradient direction.
\item \code{"stephalving"}: step length is halved, preserving the
current direction.
}
See \code{\link[maxLik]{maxBHHH}} and \code{\link[maxLik]{maxNR}} for
details.}

\item{...}{Additional arguments passed through to the second-stage SFA
call when \code{metaMethod = "sfa"}.}

\item{x}{An object of class \code{"smfa"}, as returned by
\code{smfa}, for use with the \code{print} method.}
}
\value{
\code{smfa} returns an object of class
\code{"smfa"}, which is a list containing:
\item{call}{The matched call.}
\item{groupModels}{A named list of fitted group-level frontier objects,
one per technology group. Each element is of class \code{"sfacross"},
\code{"sfaselectioncross"}, or \code{"sfalcmcross"}, depending on
\code{groupType}.}
\item{metaSfaObj}{The fitted metafrontier object. For
\code{metaMethod = "sfa"}, an object of class \code{"sfacross"}
from the second-stage SFA. The dependent variable column in
\code{metaSfaObj$dataTable} is named according to the approach used:
\code{"lp_envelope"} when \code{sfaApproach = "ordonnell"} (the
column-wise maximum of all group-evaluated frontier values is the
dependent variable) and \code{"group_fitted_values"} when
\code{sfaApproach = "huang"} (each observation's own-group fitted
frontier value is the dependent variable). For \code{metaMethod = "lp"}
or \code{"qp"}, a list containing the optimisation result and the
estimated envelope coefficients.}
\item{metaRes}{Estimated metafrontier coefficients (with standard errors,
z-values, and p-values for \code{metaMethod = "sfa"}, or the plain
coefficient vector for deterministic envelopes).}
\item{formula}{The \code{formula} supplied to the call.}
\item{metaMethod}{The metafrontier estimation method used.}
\item{sfaApproach}{The second-stage SFA approach; \code{NA} when
\code{metaMethod} is not \code{"sfa"}.}
\item{groupType}{The type of group-level frontier model estimated.}
\item{group}{The name of the grouping variable.}
\item{groups}{Character vector of unique group labels.}
\item{S}{The frontier orientation (\code{1} or \code{-1}).}
\item{dataTable}{The data used in estimation, augmented with
\code{.mf_yhat_group} (group-specific fitted frontier values) and
\code{.mf_yhat_meta} (metafrontier fitted values).}
\item{lcmNoGroup}{Logical. \code{TRUE} when \code{groupType = "sfalcmcross"}
and \code{group} was not supplied.}
\item{lcmObj}{When \code{lcmNoGroup = TRUE}, the pooled
\code{sfalcmcross} object.}
}
\description{
\code{\link{smfa}} estimates a stochastic metafrontier model
for cross-sectional or pooled data. The function follows the theoretical
frameworks of Battese, Rao, and O'Donnell (2004) and O'Donnell, Rao, and
Battese (2008), and additionally implements the two-stage stochastic approach
of Huang, Huang, and Liu (2014). Three types of group-level frontier models
are supported: standard stochastic frontier analysis
(\code{\link[sfaR]{sfacross}}), sample selection stochastic frontier
analysis (\code{\link[sfaR]{sfaselectioncross}}), and latent class stochastic
frontier analysis (\code{\link[sfaR]{sfalcmcross}}).
}
\details{
\subsection{Standard stochastic frontier (\code{groupType = "sfacross"})}{
The stochastic frontier model is defined as:
\deqn{y_i = \alpha + \mathbf{x}_i'\bm{\beta} + v_i - Su_i}
where \eqn{y} is the output (cost, revenue, or profit), \eqn{\mathbf{x}}
is the vector of frontier regressors, \eqn{u_i \ge 0} is the one-sided
inefficiency term with variance \eqn{\sigma^2_u}, and \eqn{v_i} is the
symmetric noise term with variance \eqn{\sigma^2_v}.

Estimation is by ML for all distributions except \code{"gamma"},
\code{"lognormal"}, and \code{"weibull"}, for which MSL is used with
Halton, Generalised-Halton, Sobol, or uniform draws. Antithetic draws are
available for the uniform case.

To account for heteroscedasticity, the variances are modelled as
\eqn{\sigma^2_u = \exp(\bm{\delta}'\mathbf{Z}_u)} and
\eqn{\sigma^2_v = \exp(\bm{\phi}'\mathbf{Z}_v)}. For the truncated normal
distribution, heterogeneity in the pre-truncation mean is modelled as
\eqn{\mu_i = \bm{\omega}'\mathbf{Z}_{\mu}}. The scaling property (Wang and
Schmidt, 2002) can also be imposed for the truncated normal.
}

\subsection{Sample selection frontier (\code{groupType = "sfaselectioncross"})}{
This model extends the Heckman (1979) selection framework to the
stochastic frontier setting (Greene, 2010; Dakpo \emph{et al.}, 2021).
The selection and frontier equations are:
\deqn{y_{1i}^* = \mathbf{Z}_{si}'\bm{\gamma} + w_i, \quad
    w_i \sim \mathcal{N}(0,1)}
\deqn{y_{2i}^* = \mathbf{x}_i'\bm{\beta} + v_i - Su_i}
where \eqn{y_{1i} = \mathbf{1}(y_{1i}^* > 0)} is the binary selection
indicator and \eqn{y_{2i} = y_{2i}^*} is observed only when
\eqn{y_{1i} = 1}. Selection bias arises from
\eqn{\rho = \mathrm{Corr}(w_i, v_i) \ne 0}. Only selected observations
enter the frontier and metafrontier estimation; efficiency estimates for
non-selected observations are \code{NA}.
}

\subsection{Latent class frontier (\code{groupType = "sfalcmcross"})}{
The latent class model (Orea and Kumbhakar, 2004) estimates a finite
mixture of \eqn{J} frontier models:
\deqn{y_i = \alpha_j + \mathbf{x}_i'\bm{\beta}_j + v_{i|j} - Su_{i|j}}
The prior class probability follows a logit specification:
\deqn{\pi(i,j) = \frac{\exp(\bm{\theta}_j'\mathbf{Z}_{hi})}
    {\sum_{m=1}^{J}\exp(\bm{\theta}_m'\mathbf{Z}_{hi})}}
Class assignment is based on the maximum posterior probability computed via
Bayes' rule. When \code{group} is omitted, a single pooled model is
estimated and class assignments serve as technology groups.
}

\subsection{Metafrontier estimation}{
The global metafrontier \eqn{f(x_i, \bm{\beta}^*)} envelopes all
group frontiers. With LP (Battese \emph{et al.}, 2004),
\eqn{\bm{\beta}^*} minimises
\eqn{\sum_i |\ln \hat{f}(x_i, \bm{\beta}^*) - \ln \hat{f}(x_i,
  \hat{\bm{\beta}}_{(g)})|}
subject to \eqn{\ln \hat{f}(x_i, \bm{\beta}^*) \ge \ln \hat{f}(x_i,
  \hat{\bm{\beta}}_{(g)})}. QP minimises the squared analogue. The
stochastic approaches (Huang \emph{et al.}, 2014; O'Donnell \emph{et al.},
2008) treat the technology gap \eqn{U_i} as a one-sided error in a
second-stage SFA. Group and metafrontier efficiencies are:
\deqn{TE_i^g = \exp(-u_i^g), \quad
    MTR_i = \exp(-U_i), \quad
    TE_i^* = TE_i^g \times MTR_i}
Both Jondrow \emph{et al.} (1982) and Battese and Coelli (1988) estimators
are provided for each measure. See \code{\link{efficiencies}} for details.
}
}
\examples{
###########################################################################
## -------- SECTION 1: Standard SFA Group Frontier ----------------------##
## Using the rice production dataset (ricephil) from Battese et al.      ##
## Groups are formed based on farm area terciles (small/medium/large).   ##
###########################################################################

data("ricephil", package = "sfaR")
ricephil$group <- cut(ricephil$AREA,
  breaks = quantile(ricephil$AREA, probs = c(0, 1 / 3, 2 / 3, 1), na.rm = TRUE),
  labels = c("small", "medium", "large"),
  include.lowest = TRUE
)

## 1a. sfacross groups + LP metafrontier
##     Deterministic envelope via linear programming (Battese et al., 2004).
meta_sfacross_lp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "lp"
)
summary(meta_sfacross_lp)
# Retrieve individual efficiency and metatechnology ratio estimates:
ef_lp <- efficiencies(meta_sfacross_lp)
head(ef_lp)

## 1b. sfacross groups + QP metafrontier
##     Deterministic envelope via quadratic programming.
meta_sfacross_qp <- smfa(
  formula    = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data       = ricephil,
  group      = "group",
  S          = 1,
  udist      = "hnormal",
  groupType  = "sfacross",
  metaMethod = "qp"
)
summary(meta_sfacross_qp)

\donttest{
## 1c. sfacross groups + Two-stage SFA metafrontier (Huang et al., 2014)
##     The group-specific fitted frontier values serve as the dependent
##     variable in the second-stage SFA, yielding a stochastic technology gap.
meta_sfacross_huang <- smfa(
  formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data        = ricephil,
  group       = "group",
  S           = 1,
  udist       = "hnormal",
  groupType   = "sfacross",
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_sfacross_huang)
ef_huang <- efficiencies(meta_sfacross_huang)
head(ef_huang)

## 1d. sfacross groups + O'Donnell et al. (2008) stochastic metafrontier
##     The LP deterministic envelope is used as the second-stage dependent
##     variable: the metafrontier is estimated stochastically around the
##     envelope.
meta_sfacross_odonnell <- smfa(
  formula     = log(PROD) ~ log(AREA) + log(LABOR) + log(NPK),
  data        = ricephil,
  group       = "group",
  S           = 1,
  udist       = "hnormal",
  groupType   = "sfacross",
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_sfacross_odonnell)
}

###########################################################################
## -------- SECTION 2: Latent Class (LCM) Group Frontier ---------------##
## No observed group variable: a pooled sfalcmcross model assigns       ##
## observations to 2 latent technology classes; these classes become the ##
## technology groups for the metafrontier.                               ##
###########################################################################

data("utility", package = "sfaR")

## 2a. sfalcmcross (pooled, 2 classes) + LP metafrontier
meta_lcm_lp <- smfa(
  formula    = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
  data       = utility,
  S          = -1,
  groupType  = "sfalcmcross",
  lcmClasses = 2,
  metaMethod = "lp"
)
summary(meta_lcm_lp)
ef_lcm_lp <- efficiencies(meta_lcm_lp)
head(ef_lcm_lp)

\donttest{
## 2b. sfalcmcross (pooled, 2 classes) + QP metafrontier
meta_lcm_qp <- smfa(
  formula    = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
  data       = utility,
  S          = -1,
  groupType  = "sfalcmcross",
  lcmClasses = 2,
  metaMethod = "qp"
)
summary(meta_lcm_qp)

## 2c. sfalcmcross (pooled, 2 classes) + Two-stage SFA metafrontier
##     (Huang et al., 2014)
meta_lcm_huang <- smfa(
  formula     = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
  data        = utility,
  S           = -1,
  groupType   = "sfalcmcross",
  lcmClasses  = 2,
  metaMethod  = "sfa",
  sfaApproach = "huang"
)
summary(meta_lcm_huang)
ef_lcm_huang <- efficiencies(meta_lcm_huang)
head(ef_lcm_huang)

## 2d. sfalcmcross (pooled, 2 classes) + O'Donnell et al. (2008)
meta_lcm_odonnell <- smfa(
  formula     = log(tc / wf) ~ log(y) + log(wl / wf) + log(wk / wf),
  data        = utility,
  S           = -1,
  groupType   = "sfalcmcross",
  lcmClasses  = 2,
  metaMethod  = "sfa",
  sfaApproach = "ordonnell"
)
summary(meta_lcm_odonnell)
}

###########################################################################
## -------- SECTION 3: Sample Selection SFA Group Frontier -------------##
###########################################################################

## 3a. Small toy example for automatic testing (< 5s)
N <- 100
set.seed(12345)
z1 <- rnorm(N); v1 <- rnorm(N); g <- rnorm(N)
ds <- z1 + v1; d <- ifelse(ds > 0, 1, 0)
group <- ifelse(g > 0, 1, 0)
x1 <- rnorm(N); y <- x1 + rnorm(N) - abs(rnorm(N))
dat <- data.frame(y = y, x1 = x1, z1 = z1, d = d, group = group)

meta_toy <- smfa(
  formula    = y ~ x1,
  selectionF = d ~ z1,
  data       = dat,
  group      = "group",
  groupType  = "sfaselectioncross",
  lType      = "ghermite",
  Nsub       = 10,
  itermax    = 100,
  metaMethod = "lp"
)
summary(meta_toy)

\donttest{
## 3b. More complex selection models
## Simulated dataset with a Heckman selection mechanism.

N <- 2000
set.seed(12345)
z1 <- rnorm(N); z2 <- rnorm(N); v1 <- rnorm(N); v2 <- rnorm(N); g <- rnorm(N)
e1 <- v1; e2 <- 0.7071 * (v1 + v2)
ds <- z1 + z2 + e1; d <- ifelse(ds > 0, 1, 0)
group <- ifelse(g > 0, 1, 0)
u <- abs(rnorm(N)); x1 <- rnorm(N); x2 <- rnorm(N)
y <- x1 + x2 + e2 - u
dat <- data.frame(y = y, x1 = x1, x2 = x2, z1 = z1, z2 = z2, d = d, group = group)

meta_sel_lp <- smfa(
  formula    = y ~ x1 + x2,
  selectionF = d ~ z1 + z2,
  data       = dat,
  group      = "group",
  S          = 1L,
  udist      = "hnormal",
  groupType  = "sfaselectioncross",
  modelType  = "greene10",
  lType      = "kronrod",
  Nsub       = 100,
  metaMethod = "lp"
)
summary(meta_sel_lp)
}

}
\references{
Aigner, D. J., Lovell, C. A. K., and Schmidt, P. 1977. Formulation and
estimation of stochastic frontier production function models.
\emph{Journal of Econometrics}, \bold{6}(1), 21--37.
\doi{10.1016/0304-4076(77)90052-5}

Battese, G. E., and Coelli, T. J. 1988. Prediction of firm-level technical
efficiencies with a generalized frontier production function and panel data.
\emph{Journal of Econometrics}, \bold{38}(3), 387--399.
\doi{10.1016/0304-4076(88)90053-X}

Battese, G. E., Rao, D. S. P., and O'Donnell, C. J. 2004. A metafrontier
production function for estimation of technical efficiencies and technology
gaps for firms operating under different technologies.
\emph{Journal of Productivity Analysis}, \bold{21}(1), 91--103.
\doi{10.1023/B:PROD.0000012454.06094.29}

Greene, W. 2003. Simulated likelihood estimation of the normal-gamma
stochastic frontier function. \emph{Journal of Productivity Analysis},
\bold{19}(2-3), 179--190.
\doi{10.1023/A:1022853416499}

Greene, W. 2010. A stochastic frontier model with correction for sample
selection. \emph{Journal of Productivity Analysis}, \bold{34}(1), 15--24.
\doi{10.1007/s11123-009-0159-1}

Hajargasht, G. 2015. Stochastic frontiers with a Rayleigh distribution.
\emph{Journal of Productivity Analysis}, \bold{44}(2), 199--208.
\doi{10.1007/s11123-014-0417-8}

Heckman, J. J. 1979. Sample selection bias as a specification error.
\emph{Econometrica}, \bold{47}(1), 153--161.
\doi{10.2307/1912352}

Huang, C. J., Huang, T.-H., and Liu, N.-H. 2014. A new approach to
estimating the metafrontier production function based on a stochastic
frontier framework. \emph{Journal of Productivity Analysis},
\bold{42}(3), 241--254. \doi{10.1007/s11123-014-0402-2}

Jondrow, J., Lovell, C. A. K., Materov, I. S., and Schmidt, P. 1982. On
the estimation of technical inefficiency in the stochastic frontier
production function model. \emph{Journal of Econometrics},
\bold{19}(2-3), 233--238.
\doi{10.1016/0304-4076(82)90004-5}

Li, Q. 1996. Estimating a stochastic production frontier when the adjusted
error is symmetric. \emph{Economics Letters}, \bold{52}(3), 221--228.
\doi{10.1016/S0165-1765(96)00857-9}

Meeusen, W., and van den Broeck, J. 1977. Efficiency estimation from
Cobb-Douglas production functions with composed error.
\emph{International Economic Review}, \bold{18}(2), 435--444.
\doi{10.2307/2525757}

Migon, H. S., and Medici, E. 2001. Bayesian inference for generalised
exponential models. Working paper, Universidade Federal do Rio de Janeiro.

Nguyen, N. B. 2010. Estimation of technical efficiency in stochastic
frontier analysis. PhD thesis, Bowling Green State University.

O'Donnell, C. J., Rao, D. S. P., and Battese, G. E. 2008. Metafrontier
frameworks for the study of firm-level efficiencies and technology ratios.
\emph{Empirical Economics}, \bold{34}(2), 231--255.
\doi{10.1007/s00181-007-0119-4}

Orea, L., and Kumbhakar, S. C. 2004. Efficiency measurement using a latent
class stochastic frontier model. \emph{Empirical Economics}, \bold{29}(1),
169--183. \doi{10.1007/s00181-003-0184-2}

Dakpo, K. H., Jeanneaux, P., and Latruffe, L. 2016. Modelling
pollution-generating technologies in performance benchmarking: Recent
developments, limits and future prospects in the nonparametric framework.
\emph{European Journal of Operational Research}, \bold{250}(2), 347--359.
\doi{10.1016/j.ejor.2015.07.024}

Papadopoulos, A. 2015. The half-normal specification for the two-tier
stochastic frontier model. \emph{Journal of Productivity Analysis},
\bold{43}(2), 225--230. \doi{10.1007/s11123-014-0389-8}

Stevenson, R. E. 1980. Likelihood functions for generalised stochastic
frontier estimation. \emph{Journal of Econometrics}, \bold{13}(1), 57--66.
\doi{10.1016/0304-4076(80)90042-1}

Dakpo, K. H., Latruffe, L., Desjeux, Y., and Jeanneaux, P. 2021. Latent
class modelling for a robust assessment of productivity: Application to
French grazing livestock farms. \emph{Journal of Agricultural Economics},
\bold{72}(3), 760--781. \doi{10.1111/1477-9552.12422}

Dakpo, K. H., Latruffe, L., Desjeux, Y., and Jeanneaux, P. 2022. Modeling
heterogeneous technologies in the presence of sample selection: The case of
dairy farms and the adoption of agri-environmental schemes in France.
\emph{Agricultural Economics}, \bold{53}(3), 422--438.
\doi{10.1111/agec.12683}

Tsionas, E. G. 2007. Efficiency measurement with the Weibull stochastic
frontier. \emph{Oxford Bulletin of Economics and Statistics},
\bold{69}(5), 693--706. \doi{10.1111/j.1468-0084.2007.00475.x}

Wang, H.-J. 2012. Stochastic frontier models. In \emph{A Companion to
Theoretical Econometrics}, ed. B. H. Baltagi, Blackwell, Oxford.

Wang, H.-J., and Schmidt, P. 2002. One-step and two-step estimation of the
effects of exogenous variables on technical efficiency levels.
\emph{Journal of Productivity Analysis}, \bold{18}(2), 129--144.
\doi{10.1023/A:1016565719882}

Dakpo, K. H., Desjeux, Y., and Latruffe, L. 2023. sfaR: Stochastic Frontier
Analysis using R. R package version 1.0.1.
\url{https://CRAN.R-project.org/package=sfaR}
}
\seealso{
\code{\link[sfaR]{sfacross}}, \code{\link[sfaR]{sfaselectioncross}},
\code{\link[sfaR]{sfalcmcross}}, \code{\link{efficiencies}},
\code{\link{summary.smfa}}, \code{\link[sfaR]{ic}}
}
\keyword{metafrontier}
\keyword{models}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary}
\alias{summary}
\alias{summary.smfa}
\alias{print.summary.smfa}
\title{Summary of results for stochastic metafrontier models}
\usage{
\method{summary}{smfa}(object, ...)

\method{print}{summary.smfa}(x, digits = max(3, getOption("digits") - 2), ...)
}
\arguments{
\item{object}{An object of class \code{'smfa'} returned by the
function \code{\link{smfa}}.}

\item{...}{Currently ignored.}

\item{x}{An object of class \code{'summary.smfa'}.}

\item{digits}{Numeric. Number of digits displayed in values.}
}
\value{
The \code{\link{summary}} method returns a list of class
\code{'summary.smfa'}
that contains the same elements as an object returned by \code{\link{smfa}}
with the following additional elements:

\item{AIC}{Akaike information criterion.}

\item{BIC}{Bayesian information criterion.}

\item{HQIC}{Hannan-Quinn information criterion.}

\item{metaRes}{Matrix of metafrontier estimates, their standard errors, z-values,
and asymptotic P-values.}

\item{effStats}{A list of efficiency statistics including group means and
class membership probabilities.}

\item{grpSummaries}{A list of summary objects for each group model.}
}
\description{
Create and print summary results for stochastic metafrontier models returned by
\code{\link{smfa}}.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function for cross-sectional or pooled data.
}
\keyword{methods}
\keyword{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitted.R
\name{fitted}
\alias{fitted}
\alias{fitted.smfa}
\title{Extract fitted values of stochastic metafrontier models}
\usage{
\method{fitted}{smfa}(object, ...)
}
\arguments{
\item{object}{A stochastic metafrontier model returned
by \code{\link{smfa}}.}

\item{...}{Currently ignored.}
}
\value{
A vector of fitted values is returned.
}
\description{
\code{\link{fitted}} returns the fitted frontier values from stochastic
metafrontier models estimated with \code{\link{smfa}}.
}
\note{
The fitted values are ordered in the same way as the corresponding
observations in the dataset used for the estimation.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data.
}
\keyword{fitted}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efficiencies.R
\name{efficiencies}
\alias{efficiencies}
\alias{efficiencies.smfa}
\title{Compute efficiency estimates and metatechnology ratios from stochastic metafrontier models}
\usage{
\method{efficiencies}{smfa}(object, level = 0.95, newData = NULL, ...)
}
\arguments{
\item{object}{An object of class \code{"smfa"} returned by
\code{\link{smfa}}.}

\item{level}{A number strictly between 0 and 0.9999 specifying the nominal
coverage for (in-)efficiency confidence intervals. Default \code{0.95}.
This argument is passed to the underlying \code{efficiencies} method of the
group-level model (class \code{"sfacross"}, \code{"sfalcmcross"}, or
\code{"sfaselectioncross"}).}

\item{newData}{Optional data frame for out-of-sample prediction of efficiency
estimates. When \code{NULL} (default), efficiencies are computed for the
observations used in the estimation.}

\item{...}{Further arguments (currently ignored).}
}
\value{
A data frame with one row per observation (in the original row order),
containing the following columns. The exact set of columns depends on
\code{groupType}:

\strong{Columns present for all model types:}
\describe{
\item{\code{id}}{
Observation identifier. Contains the row name of each observation as
it appeared in the data supplied to \code{\link{smfa}}.
When the data frame has no explicit row names, sequential integers
(\code{"1"}, \code{"2"}, \ldots) are used. This column is always the
first column of the returned data frame.}
\item{\code{<group>} or \code{Group_c}}{
The technology group identifier for each observation. For
\code{groupType = "sfacross"} and \code{"sfaselectioncross"}, this
column takes the name of the user-supplied \code{group} variable and
contains the group label to which each observation belongs. For
\code{groupType = "sfalcmcross"}, it is named \code{Group_c} and
contains the integer index of the latent class assigned by the maximum
posterior probability criterion.}
\item{\code{u_g}}{
Group-specific conditional mean of the inefficiency term, computed as
\eqn{E[u_i \mid \varepsilon_i]}. This is the JLMS (Jondrow, Lovell,
Materov, and Schmidt, 1982) point estimate of the inefficiency at the
group-frontier level. For \code{groupType = "sfaselectioncross"},
\code{u_g} is \code{NA} for observations not selected into the sample
(selection indicator = 0).}
\item{\code{TE_group_JLMS}}{
Group-specific technical efficiency using the Jondrow, Lovell,
Materov, and Schmidt (1982) estimator:
\eqn{TE^g_i = \exp(-E[u_i \mid \varepsilon_i])}. For
\code{groupType = "sfaselectioncross"}, \code{NA} for non-selected
observations.}
\item{\code{TE_group_BC}}{
Group-specific technical efficiency using the Battese and Coelli
(1988) estimator:
\eqn{TE^g_i = E[\exp(-u_i) \mid \varepsilon_i]}. For
\code{groupType = "sfaselectioncross"}, \code{NA} for non-selected
observations.}
\item{\code{TE_group_BC_reciprocal}}{
Reciprocal of the Battese and Coelli (1988) group technical efficiency:
\eqn{1 / TE^{g,BC}_i}. For production frontiers this equals the
cost-efficiency index implied by the BC estimator. Present for all three
model types. For \code{groupType = "sfaselectioncross"}, \code{NA} for
non-selected observations.}
\item{\code{u_meta}}{
Metafrontier inefficiency, measuring the technology-gap component
\eqn{U_i \ge 0} that separates the group frontier from the global
metafrontier. Computed from the second-stage SFA when
\code{metaMethod = "sfa"}, or derived from the LP/QP gap as
\eqn{U_i = \max\{S \cdot (\ln \hat{y}^*_i - \ln \hat{y}^g_i), 0\}}
when \code{metaMethod = "lp"} or \code{"qp"}.}
\item{\code{TE_meta_JLMS}}{
Metafrontier technical efficiency based on the JLMS group efficiency:
\eqn{TE^*_{JLMS,i} = TE^g_{JLMS,i} \times MTR_{JLMS,i}}.}
\item{\code{TE_meta_BC}}{
Metafrontier technical efficiency based on the Battese and Coelli
(1988) group efficiency:
\eqn{TE^*_{BC,i} = TE^g_{BC,i} \times MTR_{BC,i}}.}
\item{\code{MTR_JLMS}}{
Metatechnology ratio computed using the JLMS group efficiency:
\eqn{MTR_{JLMS,i} = TE^*_{JLMS,i} / TE^g_{JLMS,i} = \exp(-U_i)}.
Values range from 0 to 1. A value of 1 indicates that the group
frontier for this observation coincides with the metafrontier.}
\item{\code{MTR_BC}}{
Metatechnology ratio computed using the Battese and Coelli (1988)
group efficiency:
\eqn{MTR_{BC,i} = TE^*_{BC,i} / TE^g_{BC,i} = \exp(-U_i)}.}
}

\strong{Additional columns for \code{groupType = "sfacross"} only:}
\describe{
\item{\code{uLB_g}, \code{uUB_g}}{
Lower and upper bounds of the \code{level} confidence interval for the
conditional mean inefficiency \code{u_g}, constructed using the
asymptotic distribution of the conditional estimator. Available for
distributions with closed-form expressions for the confidence bounds,
such as \code{udist = "hnormal"} and \code{udist = "tnormal"}.}
\item{\code{m_g}}{
Mode of the conditional distribution of the one-sided error term
\eqn{u_i \mid \varepsilon_i}. This is an alternative point estimate of
inefficiency. Available for distributions for which the conditional mode
has a closed-form expression.}
\item{\code{TE_group_mode}}{
Group-specific technical efficiency evaluated at the conditional mode:
\eqn{TE^g_{\mathrm{mode},i} = \exp(-m_i)}.}
\item{\code{teBCLB_g}, \code{teBCUB_g}}{
Lower and upper bounds of the \code{level} confidence interval for the
Battese and Coelli (1988) group technical efficiency \code{TE_group_BC}.
Constructed from the corresponding bounds on the conditional
distribution of \eqn{\exp(-u_i \mid \varepsilon_i)}.}
}

\strong{Additional columns for \code{groupType = "sfalcmcross"} only:}
\describe{
\item{\code{PosteriorProb_c}}{
Posterior probability that observation \eqn{i} belongs to its assigned
class (the one with the highest posterior probability). Computed via
Bayes' rule as
\eqn{P(j \mid y_i, x_i) \propto \pi(i,j) \, P(i \mid j)},
where \eqn{\pi(i,j)} is the prior class probability and \eqn{P(i \mid j)}
is the class-conditional likelihood.}
\item{\code{PosteriorProb_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Posterior probability of belonging to latent class \eqn{J}, computed
via Bayes' rule for each class separately. One column is produced for
each estimated class.}
\item{\code{PriorProb_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Prior (unconditional) probability of belonging to latent class \eqn{J},
given by the logistic specification
\eqn{\pi(i,J) = \exp(\bm{\theta}_J'\mathbf{Z}_{hi}) /
      \sum_m \exp(\bm{\theta}_m'\mathbf{Z}_{hi})}.}
\item{\code{u_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Conditional mean of the inefficiency term for class \eqn{J}:
\eqn{E[u_{i \mid J} \mid \varepsilon_{i \mid J}]}.}
\item{\code{teBC_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Battese and Coelli (1988) technical efficiency for class \eqn{J}:
\eqn{E[\exp(-u_{i \mid J}) \mid \varepsilon_{i \mid J}]}.}
\item{\code{teBC_reciprocal_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Reciprocal of the class-\eqn{J} Battese and Coelli (1988) efficiency:
\eqn{1/TE^{BC}_{i \mid J}}.}
\item{\code{ineff_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Inefficiency estimate for the observation restricted to class \eqn{J}
(i.e. the value assigned to the class to which the observation
\emph{does} belong; \code{NA} for other classes).}
\item{\code{effBC_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Battese and Coelli (1988) efficiency for the observation's assigned
class; \code{NA} for non-assigned classes.}
\item{\code{ReffBC_cJ} (per class, \eqn{J = 1, 2, \ldots})}{
Reciprocal Battese and Coelli (1988) efficiency for the observation's
assigned class; \code{NA} for non-assigned classes.}
}
}
\description{
\code{efficiencies} returns all efficiency estimates and metatechnology ratio
(MTR) measures for objects of class \code{"smfa"} returned by
\code{\link{smfa}}. The function supports models estimated via
linear programming (LP), quadratic programming (QP), and stochastic
second-stage SFA (\code{"sfa"}), and for each observation it computes the
group-specific technical efficiency, the metafrontier technical efficiency,
and the metatechnology ratio (MTR), using both the Jondrow, Lovell, Materov,
and Schmidt (1982) (JLMS) and the Battese and Coelli (1988) (BC) estimators.
Additional model-specific columns are returned depending on \code{groupType}.
}
\details{
\subsection{Group-specific efficiency estimates}{
For each group, the group-level frontier model is estimated by maximising
the log-likelihood using the distribution specified by \code{udist} in
\code{\link{smfa}}. Given the estimated composite error
\eqn{\varepsilon_i = v_i - Su_i}, the conditional distribution of
\eqn{u_i \mid \varepsilon_i} is used to compute:
\itemize{
\item the JLMS estimator (Jondrow, Lovell, Materov, and Schmidt, 1982):
\eqn{\hat{u}_i = E[u_i \mid \varepsilon_i]}, and
\eqn{TE^g_{JLMS,i} = \exp(-\hat{u}_i)};
\item the BC estimator (Battese and Coelli, 1988):
\eqn{TE^g_{BC,i} = E[\exp(-u_i) \mid \varepsilon_i]};
\item the mode estimator: \eqn{m_i = \mathrm{mode}[u_i \mid \varepsilon_i]},
and \eqn{TE^g_{\mathrm{mode},i} = \exp(-m_i)};
\item confidence bounds on \eqn{u_i} and \eqn{TE^g_{BC,i}} at the
nominal level \code{level}.
}
For \code{groupType = "sfaselectioncross"}, all estimates are \code{NA}
for observations not selected into the sample (binary selection indicator
equal to 0). For \code{groupType = "sfalcmcross"}, the composite
efficiencies \code{u_g}, \code{TE_group_JLMS}, and \code{TE_group_BC}
are computed using the posterior-probability-weighted class assignments.
}

\subsection{Metatechnology ratio and metafrontier efficiency}{
The MTR measures how far the group frontier lies below the metafrontier
for each observation. Let \eqn{\ln \hat{y}^g_i} be the group-specific
fitted frontier value and \eqn{\ln \hat{y}^*_i} the metafrontier fitted
value.
\itemize{
\item For \code{metaMethod = "lp"} or \code{"qp"} (Battese, Rao, and
O'Donnell, 2004):
\deqn{MTR_i = \exp\!\bigl(
          -\max\!\bigl\{S \cdot (\ln \hat{y}^*_i - \ln \hat{y}^g_i),\, 0\bigr\}
        \bigr)}
where \eqn{S = 1} for production/profit frontiers and \eqn{S = -1}
for cost frontiers. The technology gap
\eqn{U_i = \max\{S \cdot (\ln \hat{y}^*_i - \ln \hat{y}^g_i), 0\}}
is stored in \code{u_meta}.
\item For \code{metaMethod = "sfa"} with \code{sfaApproach = "huang"}
(Huang, Huang, and Liu, 2014):
\deqn{MTR_i = TE^*_i = \exp(-U_i)}
where \eqn{U_i} is the one-sided error term from the second-stage SFA.
\item For \code{metaMethod = "sfa"} with
\code{sfaApproach = "ordonnell"} (O'Donnell, Rao, and Battese, 2008):
\eqn{MTR_i = TE^{*,\mathrm{sfa}}_i / TE^g_i}, where
\eqn{TE^{*,\mathrm{sfa}}_i} is the technical efficiency from the
second-stage SFA fitted on the LP envelope values.
}
The metafrontier technical efficiency is then:
\deqn{TE^*_i = TE^g_i \times MTR_i}
computed separately for the JLMS and BC group efficiency estimators.
Both \code{MTR_JLMS} and \code{MTR_BC} are reported, distinguishing
which group-level efficiency estimator was used as the basis.
}
}
\references{
Battese, G. E., and Coelli, T. J. 1988. Prediction of firm-level technical
efficiencies with a generalized frontier production function and panel data.
\emph{Journal of Econometrics}, \bold{38}(3), 387--399.
\doi{10.1016/0304-4076(88)90053-X}

Battese, G. E., Rao, D. S. P., and O'Donnell, C. J. 2004. A metafrontier
production function for estimation of technical efficiencies and technology
gaps for firms operating under different technologies.
\emph{Journal of Productivity Analysis}, \bold{21}(1), 91--103.
\doi{10.1023/B:PROD.0000012454.06094.29}

Huang, C. J., Huang, T.-H., and Liu, N.-H. 2014. A new approach to
estimating the metafrontier production function based on a stochastic
frontier framework. \emph{Journal of Productivity Analysis},
\bold{42}(3), 241--254. \doi{10.1007/s11123-014-0402-2}

Jondrow, J., Lovell, C. A. K., Materov, I. S., and Schmidt, P. 1982. On
the estimation of technical inefficiency in the stochastic frontier
production function model. \emph{Journal of Econometrics},
\bold{19}(2-3), 233--238.
\doi{10.1016/0304-4076(82)90004-5}

O'Donnell, C. J., Rao, D. S. P., and Battese, G. E. 2008. Metafrontier
frameworks for the study of firm-level efficiencies and technology ratios.
\emph{Empirical Economics}, \bold{34}(2), 231--255.
\doi{10.1007/s00181-007-0119-4}

Orea, L., and Kumbhakar, S. C. 2004. Efficiency measurement using a latent
class stochastic frontier model. \emph{Empirical Economics}, \bold{29}(1),
169--183. \doi{10.1007/s00181-003-0184-2}

Dakpo, K. H., Desjeux, Y., and Latruffe, L. 2023. sfaR: Stochastic Frontier
Analysis using R. R package version 1.0.1.
\url{https://CRAN.R-project.org/package=sfaR}
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier
analysis model fitting function using cross-sectional or pooled data;
\code{\link[sfaR]{efficiencies}}, for the underlying group-level efficiency
extractor.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.R
\name{coef}
\alias{coef}
\alias{coef.smfa}
\alias{coef.summary.smfa}
\title{Extract coefficients of stochastic metafrontier models}
\usage{
\method{coef}{smfa}(object, ...)

\method{coef}{summary.smfa}(object, ...)
}
\arguments{
\item{object}{A stochastic metafrontier model returned by \code{\link{smfa}},
or an object of class \code{'summary.smfa'}.}

\item{...}{Currently ignored.}
}
\value{
For objects of class \code{'summary.smfa'},
\code{\link{coef}} returns a matrix with four columns. Namely, the
estimated coefficients, their standard errors, z-values,
and (asymptotic) P-values.

For objects of class \code{'smfa'}, \code{\link{coef}}
returns a numeric vector of the estimated coefficients.
}
\description{
From an object of class \code{'summary.smfa'},
\code{\link{coef}} extracts the coefficients,
their standard errors, z-values, and (asymptotic) P-values.

From on object of class \code{'smfa'}, it extracts
only the estimated coefficients.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data.
}
\keyword{coefficients}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcov.R
\name{vcov}
\alias{vcov}
\alias{vcov.smfa}
\title{Compute variance-covariance matrix of stochastic metafrontier models}
\usage{
\method{vcov}{smfa}(object, ...)
}
\arguments{
\item{object}{A stochastic metafrontier model returned
by \code{\link{smfa}}.}

\item{...}{Currently ignored}
}
\value{
The variance-covariance matrix of the maximum likelihood
coefficients is returned.
}
\description{
\code{\link{vcov}} computes the variance-covariance matrix of the maximum
likelihood (ML) coefficients from stochastic metafrontier models estimated with
\code{\link{smfa}}.
}
\details{
The variance-covariance matrix is obtained by the inversion of the
negative Hessian matrix. Depending on the distribution and the
\code{'hessianType'} option, the analytical/numeric Hessian or the bhhh
Hessian is evaluated.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data.
}
\keyword{methods}
\keyword{vcov}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residuals.R
\name{residuals}
\alias{residuals}
\alias{residuals.smfa}
\title{Extract residuals of stochastic metafrontier models}
\usage{
\method{residuals}{smfa}(object, ...)
}
\arguments{
\item{object}{A stochastic metafrontier model returned
by \code{\link{smfa}}.}

\item{\dots}{Currently ignored.}
}
\value{
\code{\link{residuals}} returns a vector of residuals values.
}
\description{
This function returns the residuals' values from stochastic metafrontier models
estimated with \code{\link{smfa}}.
}
\note{
The residuals values are ordered in the same way as the corresponding
observations in the dataset used for the estimation.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data.
}
\keyword{methods}
\keyword{residuals}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.R
\name{logLik}
\alias{logLik}
\alias{logLik.smfa}
\title{Extract log-likelihood value of stochastic metafrontier models}
\usage{
\method{logLik}{smfa}(object, individual = FALSE, ...)
}
\arguments{
\item{object}{A stochastic metafrontier model returned
by \code{\link{smfa}}.}

\item{individual}{Logical. If \code{FALSE} (default), the sum of all
observations' log-likelihood values is returned. If \code{TRUE}, a vector of
each observation's log-likelihood value is returned.}

\item{...}{Currently ignored.}
}
\value{
\code{\link{logLik}} returns either an object of class
\code{'logLik'}, which is the log-likelihood value with the total number of
observations (\code{nobs}) and the number of free parameters (\code{df}) as
attributes, when \code{individual = FALSE}, or a list of elements, containing
the log-likelihood of each observation (\code{logLik}), the total number of
observations (\code{Nobs}) and the number of free parameters (\code{df}),
when \code{individual = TRUE}.
}
\description{
\code{\link{logLik}} extracts the log-likelihood value(s) from stochastic
metafrontier models estimated with \code{\link{smfa}}.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data.
}
\keyword{likelihood}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ic.R
\name{ic}
\alias{ic}
\alias{ic.smfa}
\title{Extract information criteria of stochastic metafrontier models}
\usage{
\method{ic}{smfa}(object, IC = NULL, ...)
}
\arguments{
\item{object}{A stochastic metafrontier model returned
by \code{\link{smfa}}.}

\item{IC}{Character string. Information criterion measure. Three criteria
are available: \itemize{ \item \code{'AIC'} for Akaike information criterion
(default) \item \code{'BIC'} for Bayesian information criterion \item
\code{'HQIC'} for Hannan-Quinn information criterion }.}

\item{...}{Currently ignored.}
}
\value{
\code{\link{ic}} returns a data frame with the values of the
information criteria (AIC, BIC and HQIC) of the maximum likelihood coefficients.
If the \code{IC} argument is provided, it returns only the requested criterion
as a numeric value.
}
\description{
\code{\link{ic}} returns information criterion from stochastic
metafrontier models estimated with \code{\link{smfa}}.
}
\details{
The different information criteria are computed as follows: \itemize{ \item
AIC: \eqn{-2 \log{LL} + 2 * K} \item BIC: \eqn{-2 \log{LL} + \log{N} * K}
\item HQIC: \eqn{-2 \log{LL} + 2 \log{\left[\log{N}\right]} * K} } where
\eqn{LL} is the maximum likelihood value, \eqn{K} the number of parameters
estimated and \eqn{N} the number of observations.
}
\seealso{
\code{\link{smfa}}, for the stochastic metafrontier analysis model
fitting function using cross-sectional or pooled data.
}
\keyword{AIC}
\keyword{BIC}
\keyword{HQIC}
\keyword{methods}
